/*
 * WinApp.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "decoder/stdafx.h"
#include "WinApp.h"

#include <MenuBar.h>
#include <MenuItem.h>
#include <Alert.h>
#include "MainFrame.h"
#include "WinView.h"

#include "decoder/GraphicDriver.h"

//
namespace {
	const char *const lpAboutMsg = 
		"SD35 for BeOS Ver.0.03.0\n\tDR3/KT\n\n"
		"Copyright " B_UTF8_COPYRIGHT "1999 A.Mizuguchi";
};

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace App;
using namespace ALSD;
#endif
/***********************************************************
	construction
***********************************************************/
CWinApp::CWinApp(int argc, char *argv[])
: BApplication( "application/x-vnd.FtGUN-EmptyWindow" )
{
	strcpy( m_lpFileLst, argv[1] );

	m_lpSD = NULL;
}

CWinApp::~CWinApp(void)
{
}

/*************************************************************
	実行準備
*************************************************************/
void CWinApp::ReadyToRun(void)
{
	// Create mainframe
	CMainFrame *lpWnd = new CMainFrame(
						BRect( 50,50,690,530 ),
						"MainFrame", B_TITLED_WINDOW, 0 );
					
	// Create menu
	BMenuBar *lpMenuBar = CreateMenuBar();
	
	// CreateView	
	CWinView *lpWinView = new CWinView(BRect(0,0,0,0),
					 "WinView", B_FOLLOW_ALL_SIDES,
					 B_WILL_DRAW );
	lpWnd->InitView(
			lpWinView,
			lpMenuBar );
	
	lpWnd->Show();

	// Create System3.x
	m_lpSD		= new CScenarioDecoder( m_lpFileLst );
	m_lpView	= m_lpSD->GetGrpDriver();
	m_lpView->SetLocalInfo( (void *)lpWnd );
	
	// Set System3.x
	lpWinView->SetGrpView( m_lpView );	
	lpWnd->SetSD( m_lpSD );

	// resume thread
	m_ThreadID = ::spawn_thread( RunSD, "run", B_NORMAL_PRIORITY, this );	
	lpWnd->SetTID( m_ThreadID );
	::resume_thread( m_ThreadID );
	
	return;
}

/***********************************************************
	run thread
***********************************************************/
int32 CWinApp::RunSD(void *_lpInfo)
{
	CWinApp *lpThis = (CWinApp *)_lpInfo;
	
	lpThis->m_lpSD->Run();
}

/*************************************************************
	メニューの生成
*************************************************************/
BMenuBar *CWinApp::CreateMenuBar( void )
{
	BMenuBar *lpMenuBar = new BMenuBar( BRect(0,0,0,0), B_EMPTY_STRING );
	BMenu	 *lpFileMenu = new BMenu( "ファイル" );
	BMenu	 *lpHelpMenu = new BMenu( "ヘルプ" );
//	BMenu	 *lpFileMenu = new BMenu( "File" );
//	BMenu	 *lpHelpMenu = new BMenu( "Help" );
	
	lpMenuBar->AddItem( lpFileMenu );
	lpMenuBar->AddItem( lpHelpMenu );
	
	// File->Exit
	lpFileMenu->AddItem( new BMenuItem(
							"Exit" B_UTF8_ELLIPSIS,
							new BMessage( B_QUIT_REQUESTED )
						));
	lpFileMenu->SetTargetForItems( this );
	// Help->Version info
	lpHelpMenu->AddItem( new BMenuItem(
							"Version info" B_UTF8_ELLIPSIS,
					 		new BMessage( B_ABOUT_REQUESTED )
					 	));
	lpHelpMenu->SetTargetForItems( this );

	//
	return lpMenuBar;
}



/*************************************************************
	アバウトダイアログ	
*************************************************************/
void CWinApp::AboutRequested( void )
{
	BAlert *lpAlertPanel = new BAlert( "about box", 
										lpAboutMsg, 
										"OK" );
	lpAlertPanel->Go(NULL);
}

