/*
 * AliceCG_PMS256.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"

#include "AliceCG_PMS256.h"
#include "EALSD.h"
#include "OffScreen.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_ALCG;
using namespace ALSD_OFFSCR;
#endif

/**********************************************************************
	RXgNV
**********************************************************************/
CAliceCG_PMS256::CAliceCG_PMS256( const unsigned char *_lpMem )
: CAliceCG_PMS( _lpMem )
{
	//
	GetHeader();
	if( m_Header.Bpp != 8 )throw EALSD_Msg( "error:not PMS256 Bpp(%d)\n", m_Header.Bpp );
	if( m_Header.X < 0 || m_Header.Y < 0 ){
		throw EALSD_Msg( "error:PMS256 X(%d)orY(%d)\n", m_Header.X, m_Header.Y );
	}
	// pbgǂݍ
	GetPalette();
}
CAliceCG_PMS256::~CAliceCG_PMS256()
{
}

/**********************************************************************
	_PosFlag == TRUE ΍W	
**********************************************************************/
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
void CAliceCG_PMS256::DrawBitmap( ALSD::COffScreen &_Screen,
								  long _x, long _y, long _PosFlag, 
								  stGrpRect *_lpRect,
								  long _Brightness )
#else
void CAliceCG_PMS256::DrawBitmap( COffScreen &_Screen,
								  long _x, long _y, long _PosFlag, 
								  stGrpRect *_lpRect,
								  long _Brightness )
#endif
{
	long dx, dy;
	switch( _PosFlag ){
		case -1:	// ftHg
			dx = m_Header.X;
			dy = m_Header.Y;
			break;
		case  0:	// ΍W
			dx = _x;
			dy = _y;
			break;
		case  1:	// ΍W
			dx = m_Header.X + _x;
			dy = m_Header.Y + _y;
			break;
	}

	//
	_lpRect->left	= dx;
	_lpRect->top	= dy;
	_lpRect->right	= dx + m_Header.Width;
	_lpRect->bottom = dy + m_Header.Height;

	// WJ
	unsigned char	*lpPic = new unsigned char[m_Header.Height * m_Header.Width];
	Expand256( lpPic, -1 );	

	// bN
	stGrpScreenDesc Desc;
	stGrpScreenBits Bits;
	_Screen.Lock( &Bits, &Desc );

	// 
	long red, green, blue;
	unsigned long *lpDstPos = Bits.lpCBits;
	unsigned long *lpDstWork;
	unsigned char *lpSrcWork = lpPic;
	long pitch = Desc.lPitch / 4;
	lpDstPos = lpDstPos + ( dx + dy * pitch  );
	for( long y = 0; y < m_Header.Height; y++ ){
		lpDstWork = lpDstPos;
		for( long x = 0; x < m_Header.Width; x++ ){
			if( m_SpColor == -1 || *lpSrcWork != m_SpColor ){
				red = m_Palette.entry[*lpSrcWork].red;
				green=m_Palette.entry[*lpSrcWork].green;
				blue= m_Palette.entry[*lpSrcWork].blue;

				if( _Brightness != 255 ){
					red		= red	* _Brightness / 255;
					green	= green * _Brightness / 255;
					blue	= blue	* _Brightness / 255;
				}
				*lpDstWork = (red << 16) | (green << 8) | (blue);
			}		
			lpDstWork++;
			lpSrcWork++;
		}
		lpDstPos += pitch;
	}

	delete[] lpPic;
	//
	_Screen.Unlock();

}



