/*
 * CommandDriver.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "CommandDriver.h"

#include "ScenarioDecoder.h"
#include "ScenarioParser.h"
#include "ExpressionDriver.h"
#include "EALSD.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_CMDDRV;
#endif
/**********************************************************************
	RXgNV
**********************************************************************/
CCommandDriver::CCommandDriver( CScenarioDecoder *_lpScenarioDecoder )
{
	//
	m_Loop	= 0;
	//
	Repeat = 0;
	m_OffsetTime = 0;
	m_NowTime	 = 0;
	m_GPC		 = 7;
	//
	m_lpSD = _lpScenarioDecoder;
	m_lpED = NULL;
	m_lpGD = NULL;

	// ֐e[u̓o^
	m_lpCmdFuncTbl['A'-'A'] = &CCommandDriver::_CommandA;
	m_lpCmdFuncTbl['B'-'A'] = &CCommandDriver::_CommandB;
	m_lpCmdFuncTbl['C'-'A'] = &CCommandDriver::_CommandC;
	m_lpCmdFuncTbl['D'-'A'] = &CCommandDriver::_CommandD;
	m_lpCmdFuncTbl['E'-'A'] = &CCommandDriver::_CommandE;
	m_lpCmdFuncTbl['F'-'A'] = &CCommandDriver::_CommandF;
	m_lpCmdFuncTbl['G'-'A'] = &CCommandDriver::_CommandG;
	m_lpCmdFuncTbl['H'-'A'] = &CCommandDriver::_CommandH;
	m_lpCmdFuncTbl['I'-'A'] = &CCommandDriver::_CommandI;
	m_lpCmdFuncTbl['J'-'A'] = &CCommandDriver::_CommandJ;
	m_lpCmdFuncTbl['K'-'A'] = &CCommandDriver::_CommandK;
	m_lpCmdFuncTbl['L'-'A'] = &CCommandDriver::_CommandL;
	m_lpCmdFuncTbl['M'-'A'] = &CCommandDriver::_CommandM;
	m_lpCmdFuncTbl['N'-'A'] = &CCommandDriver::_CommandN;
	m_lpCmdFuncTbl['O'-'A'] = &CCommandDriver::_CommandO;
	m_lpCmdFuncTbl['P'-'A'] = &CCommandDriver::_CommandP;
	m_lpCmdFuncTbl['Q'-'A'] = &CCommandDriver::_CommandQ;
	m_lpCmdFuncTbl['R'-'A'] = &CCommandDriver::_CommandR;
	m_lpCmdFuncTbl['S'-'A'] = &CCommandDriver::_CommandS;
	m_lpCmdFuncTbl['T'-'A'] = &CCommandDriver::_CommandT;
	m_lpCmdFuncTbl['U'-'A'] = &CCommandDriver::_CommandU;
	m_lpCmdFuncTbl['V'-'A'] = &CCommandDriver::_CommandV;
	m_lpCmdFuncTbl['W'-'A'] = &CCommandDriver::_CommandW;
	m_lpCmdFuncTbl['X'-'A'] = &CCommandDriver::_CommandX;
	m_lpCmdFuncTbl['Y'-'A'] = &CCommandDriver::_CommandY;
	m_lpCmdFuncTbl['Z'-'A'] = &CCommandDriver::_CommandZ;

	// C
	memset( m_lpCmdFuncTblC, 0, sizeof(LPCOMMAND1ST) * 26 );	
	m_lpCmdFuncTblC['B'-'A'] = &CCommandDriver::CommandCB;
	m_lpCmdFuncTblC['C'-'A'] = &CCommandDriver::CommandCC;
	m_lpCmdFuncTblC['D'-'A'] = &CCommandDriver::CommandCD;
	m_lpCmdFuncTblC['E'-'A'] = &CCommandDriver::CommandCE;
	m_lpCmdFuncTblC['F'-'A'] = &CCommandDriver::CommandCF;
	m_lpCmdFuncTblC['K'-'A'] = &CCommandDriver::CommandCK;
	m_lpCmdFuncTblC['L'-'A'] = &CCommandDriver::CommandCL;
	m_lpCmdFuncTblC['M'-'A'] = &CCommandDriver::CommandCM;
	m_lpCmdFuncTblC['P'-'A'] = &CCommandDriver::CommandCP;
	m_lpCmdFuncTblC['S'-'A'] = &CCommandDriver::CommandCS;
	m_lpCmdFuncTblC['T'-'A'] = &CCommandDriver::CommandCT;
	m_lpCmdFuncTblC['U'-'A'] = &CCommandDriver::CommandCU;
	m_lpCmdFuncTblC['V'-'A'] = &CCommandDriver::CommandCV;
	m_lpCmdFuncTblC['X'-'A'] = &CCommandDriver::CommandCX;

	// I
	memset( m_lpCmdFuncTblI, 0, sizeof(LPCOMMAND1ST) * 26 );
	m_lpCmdFuncTblI['A'-'A'] = &CCommandDriver::CommandIA;
	m_lpCmdFuncTblI['C'-'A'] = &CCommandDriver::CommandIC;
	m_lpCmdFuncTblI['G'-'A'] = &CCommandDriver::CommandIG;
	m_lpCmdFuncTblI['K'-'A'] = &CCommandDriver::CommandIK;
	m_lpCmdFuncTblI['M'-'A'] = &CCommandDriver::CommandIM;
	m_lpCmdFuncTblI['X'-'A'] = &CCommandDriver::CommandIX;
	m_lpCmdFuncTblI['Y'-'A'] = &CCommandDriver::CommandIY;
	m_lpCmdFuncTblI['Z'-'A'] = &CCommandDriver::CommandIZ;



}

CCommandDriver::~CCommandDriver()
{
}

void CCommandDriver::Initialize(void)
{
	m_lpGD = m_lpSD->GetGrpDriver();
	m_lpED = m_lpSD->GetExpDriver();
	m_lpGD->GetTickCount( &m_OffsetTime );
}

/**********************************************************************
	1st pass select
**********************************************************************/
long CCommandDriver::ExecCommandA2Z( long Cmd )
{
	(this->*m_lpCmdFuncTbl[Cmd-'A'])();
	
	return ALSD_OK;
}

