/*
 * CommandDriverB.cpp
 *		- "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "CommandDriver.h"

#include "ScenarioDecoder.h"
#include "ScenarioParser.h"
#include "ExpressionDriver.h"
#include "GraphicDriver.h"
#include "SelectionCtrl.h"
#include "MessageCtrl.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_CMDDRV;
#endif
/**********************************************************************
	B command
**********************************************************************/
void CCommandDriver::_CommandB(void)
{
	long Cmd = m_lpSD->GetParser()->GetC();
	switch( Cmd ){
		case 0:
			CommandB0(); break;
		case 1:
			CommandB1(); break;
		case 2:
			CommandB2(); break;
		case 3:
			CommandB3(); break;
		case 4:
			CommandB4(); break;
		case 10:
			CommandB10(); break;
		case 11:
			CommandB11(); break;
		case 21:
			CommandB21(); break;
		case 22:
			CommandB22(); break;
		case 23:
			CommandB23(); break;
		case 24:
			CommandB24(); break;
		default:
			throw EALSD_Msg( "error: B%c unknown command\n", Cmd );
			break;
	}
}

/**********************************************************************
	bZ[WEBhȄ
**********************************************************************/
void CCommandDriver::CommandB0( void )
{
	int num = m_lpED->GetExpValue();

	if( num ){
		m_lpSD->GetMsgCtrl()->Clear( false );
	}else{
		m_lpSD->GetMsgCtrl()->Clear( true );
	}

	TRACE( "command:B0( %d )\n", num );

}

/**********************************************************************
	I̐ݒ
**********************************************************************/
void CCommandDriver::CommandB1( void )
{
	int num = m_lpED->GetExpValue(); - 1;
	int X1  = m_lpED->GetExpValue();
	int Y1  = m_lpED->GetExpValue();
	int X2  = m_lpED->GetExpValue();
	int Y2  = m_lpED->GetExpValue();
	int V   = m_lpED->GetExpValue();

	m_lpSD->GetSelCtrl()->SetSelInfo( num, X1, Y1, X2, Y2, V );
	TRACE( "command:B1( %d, %d, %d, %d, %d )\n", num+1, X1, Y1, X2, Y2, V );
}

/**********************************************************************
	IWindow̐؂ւ
**********************************************************************/
void CCommandDriver::CommandB2( void )
{
	int num = m_lpED->GetExpValue(); - 1 ;
	int W   = m_lpED->GetExpValue();
	int C1  = m_lpED->GetExpValue();
	int C2  = m_lpED->GetExpValue();
	int C3  = m_lpED->GetExpValue();
	int C4	= m_lpED->GetExpValue();

	m_lpSD->GetSelCtrl()->ChangeSelector( num, W, C1, C2, C3, C4 );
	TRACE("command:B2( %d, %d, %d, %d, %d, %d)\n", num + 1, W, C1, C2, C3, C4);
}

/**********************************************************************
	bZ[WEBhE̍Wݒ
**********************************************************************/
void CCommandDriver::CommandB3( void )
{
	int num = m_lpED->GetExpValue() - 1;
	int X1  = m_lpED->GetExpValue();
	int Y1  = m_lpED->GetExpValue();
	int X2  = m_lpED->GetExpValue();
	int Y2  = m_lpED->GetExpValue();
	int V   = m_lpED->GetExpValue();

	m_lpSD->GetMsgCtrl()->SetMsgInfo( num, X1, Y1, X2, Y2, V ); 

	TRACE( "command:B3( %d, %d, %d, %d, %d, %d )\n", num+1, X1, Y1, X2, Y2, V);
}

/**********************************************************************
	bZ[WEBhEؑւ
**********************************************************************/
void CCommandDriver::CommandB4( void )
{
	int num = m_lpED->GetExpValue() - 1;
	int W   = m_lpED->GetExpValue();
	int C1  = m_lpED->GetExpValue();
	int C2  = m_lpED->GetExpValue();
	int N   = m_lpED->GetExpValue();
	int M   = m_lpED->GetExpValue();

	m_lpSD->GetMsgCtrl()->ChangeMsg( num, W, C1, C2, N, M );
	
	TRACE( "command:B4( %d, %d, %d, %d, %d, %d )\n", num+1, W, C1, C2, N, M);
}

/**********************************************************************
	bZ[Wɕ\W߂
**********************************************************************/
void CCommandDriver::CommandB10( void )
{
	long *var_x = m_lpED->GetVariableNo();
	long *var_y = m_lpED->GetVariableNo();

	m_lpSD->GetMsgCtrl()->GetLocate( var_x, var_y );

	TRACE( "command:B10( ... )\n" );
}

/**********************************************************************
	݂̑IAbZ[WEBhE̔ԍ擾
**********************************************************************/
void CCommandDriver::CommandB11( void )
{
	long *var_s = m_lpED->GetVariableNo();
	long *var_m = m_lpED->GetVariableNo();

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:B11( ... )\n" );
}

/**********************************************************************
	݂̃Jg̑IEBhE̍W擾
**********************************************************************/
void CCommandDriver::CommandB21( void )
{
	long num = m_lpED->GetExpValue();
	long *var_x = m_lpED->GetVariableNo();
	long *var_y = m_lpED->GetVariableNo();

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:B21( ... )\n" );
}

/**********************************************************************
	݂̃Jg̑IEBhẼTCY擾
**********************************************************************/
void CCommandDriver::CommandB22( void )
{
	long num = m_lpED->GetExpValue();
	long *var_x_size = m_lpED->GetVariableNo();
	long *var_y_size = m_lpED->GetVariableNo();

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:B22( ... )\n" );
}

/**********************************************************************
    ݂̃Jg̃bZ[WEBhE̍W擾B
**********************************************************************/
void CCommandDriver::CommandB23( void )
{
	long num = m_lpED->GetExpValue();
	long *var_x = m_lpED->GetVariableNo();
	long *var_y = m_lpED->GetVariableNo();

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:B23( ... )\n" );
}

/**********************************************************************
    ݂̃Jg̃bZ[WEBhẼTCY擾B
**********************************************************************/
void CCommandDriver::CommandB24( void )
{
	long num = m_lpED->GetExpValue();
	long *var_x_size = m_lpED->GetVariableNo();
	long *var_y_size = m_lpED->GetVariableNo();

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:B24( ... )\n" );
}

/**********************************************************************
	ݒ肳ĂIEBhE̍W擾
**********************************************************************/
void CCommandDriver::CommandB31( void )
{
	long num = m_lpED->GetExpValue();
	long *var_x = m_lpED->GetVariableNo();
	long *var_y = m_lpED->GetVariableNo();

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:B31( ... )\n" );
}

/**********************************************************************
    ݒ肳ĂIEBhẼTCY擾B
**********************************************************************/
void CCommandDriver::CommandB32( void )
{
	long num = m_lpED->GetExpValue();
	long *var_x_size = m_lpED->GetVariableNo();
	long *var_y_size = m_lpED->GetVariableNo();

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:B32( ... )\n" );
}

/**********************************************************************
    ݒ肳Ă郁bZ[WEBhE̍W擾B
**********************************************************************/
void CCommandDriver::CommandB33( void )
{
	long num = m_lpED->GetExpValue();
	long *var_x = m_lpED->GetVariableNo();
	long *var_y = m_lpED->GetVariableNo();

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:B33( ... )\n" );
}

/**********************************************************************
    ݒ肳Ă郁bZ[WEBhẼTCY擾B
**********************************************************************/
void CCommandDriver::CommandB34( void )
{
	long num = m_lpED->GetExpValue();
	long *var_x_size = m_lpED->GetVariableNo();
	long *var_y_size = m_lpED->GetVariableNo();

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:B34( ... )\n" );
}


