/*
 * CommandDriverL.cpp
 *		- "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "CommandDriver.h"

#include "ScenarioDecoder.h"
#include "ScenarioParser.h"
#include "ExpressionDriver.h"
#include "GraphicDriver.h"
#include "FileCtrl.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_CMDDRV;
#endif
/**********************************************************************
	L command
**********************************************************************/
void CCommandDriver::_CommandL(void)
{
	switch( m_lpSD->GetParser()->GetC() ){
		case 'C':
			CommandLC(); break;
		case 'D':
			CommandLD(); break;
		case 'E':
			CommandLE(); break;
		case 'G':
			CommandLG(); break;
		case 'H':
			switch( m_lpSD->GetParser()->GetC() ){
				case 'D':
					CommandLHD(); break;
				case 'G':
					CommandLHG(); break;
				case 'M':
					CommandLHM(); break;
				case 'S':
					CommandLHS(); break;
				case 'W':
					CommandLHW(); break;
				default:
					throw EALSD_Msg( "error: LH? unknown command\n" );
					break;
			}
			break;
		case 'L':
			CommandLL(); break;
		case 'M':
			CommandLM(); break;
		case 'O':
			CommandLO(); break;
		case 'P':
			CommandLP(); break;
		case 'T':
			CommandLT(); break;
		case 'U':
			CommandLU(); break;
		case 'V':
			CommandLV(); break;
		default:
			throw EALSD_Msg( "error: L? unknown command\n" );
			break;
	}
}

/**********************************************************************
	bf\
**********************************************************************/
void CCommandDriver::CommandLC( void )
{
	string filename;
	int x  = m_lpED->GetExpValue();
	int y  = m_lpED->GetExpValue();
	m_lpSD->GetParser()->GetString( filename, ':' );

	m_lpGD->LoadDIB( x, y, filename.c_str() );

	TRACE( "command:LC( %d, %d )\n", x, y);
}
/**********************************************************************
	Z[uf[^̑SǍ
**********************************************************************/
void CCommandDriver::CommandLD( void )
{
	long num = m_lpED->GetExpValue() - 1;

	*m_lpED->GetSystemVar() = 
		m_lpSD->GetFileCtrl()->LoadDataAll( num );
	TRACE( "command:LD( ... )\n" );
}
/**********************************************************************
	ϐ̒lt@Cǂ
**********************************************************************/
void CCommandDriver::CommandLE( void )
{
	string filename;
	long type = m_lpSD->GetParser()->GetC();
	m_lpSD->GetParser()->GetString( filename, ':' );

	long *lpVar, num, var;
	long *rnd = m_lpED->GetSystemVar();
	switch( type ){
		case 0: // ϐ
			lpVar=m_lpED->GetVariableNo();
			num = m_lpED->GetExpValue();
			*rnd = m_lpSD->GetFileCtrl()->LoadData( filename, lpVar, num );
			break;
		case 1: // 
			var = m_lpED->GetExpValue() - 1;
			num = m_lpED->GetExpValue();
			*rnd = m_lpSD->GetFileCtrl()->LoadDataString( filename, var, num );
			break;
	}
	TRACE( "command:LE( ... )\n" );
}
void CCommandDriver::CommandLG( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LG( ... )\n" );
}

/**********************************************************************
	LHR}h{IɕKǂݍ݂njƂȂ
**********************************************************************/
void CCommandDriver::CommandLHD( void )
{
	long num	= m_lpSD->GetParser()->GetC();
	long no		= m_lpED->GetExpValue() - 1;

	if( num == 3 )*m_lpED->GetSystemVar() = 0;
	TRACE( "command:LHD( ... )\n" );
}
void CCommandDriver::CommandLHG( void )
{
	long num	= m_lpSD->GetParser()->GetC();
	long no		= m_lpED->GetExpValue() - 1; 

/*	if( num == 3 ){
		// t@C̑݃`FbN AgNiNAΉ
		*m_lpED->GetSystemVar() 
			= m_lpSD->GetFileCtrl()->CheckData( DATATYPE_CG , no );
	}
*/
	if( num == 3 )*m_lpED->GetSystemVar() = 0;
	TRACE( "command:LHG( ... )\n" );
}
void CCommandDriver::CommandLHM( void )
{
	long num	= m_lpSD->GetParser()->GetC();
	long no		= m_lpED->GetExpValue() - 1;

	if( num == 3 )*m_lpED->GetSystemVar() = 0;
	TRACE( "command:LHM( ... )\n" );
}
void CCommandDriver::CommandLHS( void )
{
	long num	= m_lpSD->GetParser()->GetC();
	long no		= m_lpED->GetExpValue() - 1;

	if( num == 3 )*m_lpED->GetSystemVar() = 0;
	TRACE( "command:LHS( ... )\n" );
}
void CCommandDriver::CommandLHW( void )
{
	long num	= m_lpSD->GetParser()->GetC();
	long no		= m_lpED->GetExpValue() - 1;

	if( num == 3 )*m_lpED->GetSystemVar() = 0;
	TRACE( "command:LHW( ... )\n" );
}
/**********************************************************************
	ϐ̒lNt@Cǂݍ
**********************************************************************/
void CCommandDriver::CommandLL( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LL( ... )\n" );
}
void CCommandDriver::CommandLM( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LM( ... )\n" );
}
void CCommandDriver::CommandLO( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LO( ... )\n" );
}
/**********************************************************************
	Z[uf[^̈ꕔ[h
**********************************************************************/
void CCommandDriver::CommandLP( void )
{
	long num = m_lpED->GetExpValue() - 1;
	long *point = m_lpED->GetVariableNo();
	long  count = m_lpED->GetExpValue();
	long  page  = m_lpED->GetPrevVarPage();
	long  index = m_lpED->GetPrevVarIndex();

	*m_lpED->GetSystemVar() = 
		m_lpSD->GetFileCtrl()->LoadDataPart( num, page, index, count );
	TRACE( "command:LP( ... )\n" );
}
/**********************************************************************
	^CX^v̓Ǎ
**********************************************************************/
void CCommandDriver::CommandLT( void )
{
	long num = m_lpED->GetExpValue() - 1;
	long *var= m_lpED->GetVariableNo();

	var[0] = 1980;
	var[1] = 12;
	var[2] = 31;
	var[3] = 23;
	var[4] = 59;
	var[5] = 1;

	*m_lpED->GetSystemVar() = 0;
	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:LT( ... )\n" );
}
void CCommandDriver::CommandLU( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LU( ... )\n" );
}
void CCommandDriver::CommandLV( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LV( ... )\n" );
}
/**********************************************************************
	t@CI
**********************************************************************/
void CCommandDriver::CommandLXG( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LXG( ... )\n" );
}
/**********************************************************************
	t@C쐬܂̓I[v
**********************************************************************/
void CCommandDriver::CommandLXO( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LXO( ... )\n" );
}
/**********************************************************************
	t@C쐬܂̓N[Y
**********************************************************************/
void CCommandDriver::CommandLXC( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LXC( ... )\n" );
}
/**********************************************************************
	bft@C[h
**********************************************************************/
void CCommandDriver::CommandLXL( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LXL( ... )\n" );
}
/**********************************************************************
	t@CTCY̎擾
**********************************************************************/
void CCommandDriver::CommandLXS( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LXS( ... )\n" );
}
/**********************************************************************
	t@C|C^̈ʒuit@C擪̃oCgjݒ肷
**********************************************************************/
void CCommandDriver::CommandLXP( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LXP( ... )\n" );
}
/**********************************************************************
	t@Cf[^ǂݎ
**********************************************************************/
void CCommandDriver::CommandLXR( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LXR( ... )\n" );
}
/**********************************************************************
	t@CɃf[^
**********************************************************************/
void CCommandDriver::CommandLXW( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:LXW( ... )\n" );
}


