/*
 * CommandDriverM.cpp
 *		- "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "CommandDriver.h"

#include "ScenarioDecoder.h"
#include "ScenarioParser.h"
#include "ExpressionDriver.h"
#include "GraphicDriver.h"
#include "MessageCtrl.h"
#include "SelectionCtrl.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_CMDDRV;
using namespace ALSD_EXPDRV;
#endif
/**********************************************************************
	M command
**********************************************************************/
void CCommandDriver::_CommandM(void)
{
	switch( m_lpSD->GetParser()->GetC() ){
		case 'A':
			CommandMA(); break;
		case 'C':
			CommandMC(); break;
		case 'D':
			CommandMD(); break;
		case 'E':
			CommandME(); break;
		case 'F':
			CommandMF(); break;
		case 'G':
			switch( m_lpSD->GetParser()->GetC() ){
				case 0:
					CommandMG0(); break;
				case 1:
					CommandMG1(); break;
				case 2:
					CommandMG2(); break;
				case 3:
					CommandMG3(); break;
				case 4:
					CommandMG4(); break;
				case 5:
					CommandMG5(); break;
				case 6:
					CommandMG6(); break;
				case 7:
					CommandMG7(); break;
				case 100:
					CommandMG100(); break;
				default:
					throw EALSD_Msg( "error: MG? unknown command\n" );
			}
			break;
		case 'H':
			CommandMH(); break;
		case 'I':
			CommandMI(); break;
		case 'L':
			CommandML(); break;
		case 'M':
			CommandMM(); break;
		case 'P':
			CommandMP(); break;
		case 'S':
			CommandMS(); break;
		case 'T':
			CommandMT(); break;
		case 'V':
			CommandMV(); break;
		case 'X':
			switch( m_lpSD->GetParser()->GetC() ){
				case 'A':
					CommandMXA(); break;
				case 'M':
					CommandMXM(); break;
				case 'S':
					CommandMXS(); break;
				default:
					throw EALSD_Msg( "error: MX? unknown command\n" );
			}
			break;
		case 'Z':
			switch( m_lpSD->GetParser()->GetC() ){
				case 0:
					CommandMZ0(); break;
				default:
					throw EALSD_Msg( "error: MZ? unknown command\n" );
			}
			break;
		default:
			throw EALSD_Msg( "error: M? unknown command\n" );
			break;
	}
}

/**********************************************************************
	̘A
**********************************************************************/
void CCommandDriver::CommandMA( void )
{
	long num1 = m_lpED->GetExpValue() - 1;
	long num2 = m_lpED->GetExpValue() - 1;

//	strncat(m_lpED->GetStrVariable(num1),
//			m_lpED->GetStrVariable(num2),
//			ALSD_STRVAR_LEN );
	string work = m_lpED->GetStrVariable(num2);
	strcat(m_lpED->GetStrVariable(num1), work.c_str() );
	TRACE( "command:MA( ... )\n" );
}
/**********************************************************************
	r
**********************************************************************/
void CCommandDriver::CommandMC( void )
{
	long num1 = m_lpED->GetExpValue() - 1;
	long num2 = m_lpED->GetExpValue() - 1;

	long *var = m_lpED->GetSystemVar();

	if( strcmp( m_lpED->GetStrVariable(num1),
				m_lpED->GetStrVariable(num2)) ){
		// sv
		*var = 0;
		TRACE( "command:MC( ... )/sv\n" );
	}else{
		*var = 1;
		TRACE( "command:MC( ... )/v\n" );
	}
}
/**********************************************************************
	ϐw蒷Rs[
**********************************************************************/
void CCommandDriver::CommandMD( void )
{
	long dst_str_no = m_lpED->GetExpValue() - 1;
	long src_str_no = m_lpED->GetExpValue() - 1;
	long len		= m_lpED->GetExpValue();
	
	memcpy( m_lpED->GetStrVariable( dst_str_no ),
			m_lpED->GetStrVariable( src_str_no ),
			len * 2 );
	
	TRACE( "command:MD( ... )\n" );
}
/**********************************************************************
	ʒuw̕Rs[
**********************************************************************/
void CCommandDriver::CommandME( void )
{
	TRACE( "command:ME( ... )\n" );
	long dst_str_no = m_lpED->GetExpValue() - 1;
	long dst_pos	= m_lpED->GetExpValue();
	long src_str_no = m_lpED->GetExpValue() - 1;
	long src_pos	= m_lpED->GetExpValue();
	long len		= m_lpED->GetExpValue();

	strncpy(m_lpED->GetStrVariable( dst_str_no ) + dst_pos * 2,
			m_lpED->GetStrVariable( src_str_no ) + src_pos * 2,
			len * 2);
	*(m_lpED->GetStrVariable( dst_str_no ) + dst_pos * 2 + len * 2) = '\0';

}
/**********************************************************************
	\𕶎ϐɎ擾邩Ȃsw
**********************************************************************/
void CCommandDriver::CommandMF( void )
{
	long *lpVar=m_lpED->GetVariableNo();
	long  dst = m_lpED->GetExpValue() - 1;
	long  key = m_lpED->GetExpValue() - 1;
	long start= m_lpED->GetExpValue();

	char *lpStart=m_lpED->GetStrVariable( dst ) + start * 2;
	char *lpPos = strstr( lpStart, m_lpED->GetStrVariable( key ) );

	if( lpPos ){
		*m_lpED->GetSystemVar() = 0;
		*lpVar = ( lpPos - lpStart ) / 2;
	}else{
		*m_lpED->GetSystemVar() = 255;
	}

	m_lpED->GetSystemVar();

	TRACE( "command:MF( ... )\n" );
}
/**********************************************************************
	\𕶎ϐɎ擾邩Ȃsw
**********************************************************************/
void CCommandDriver::CommandMG0( void )
{
	long sw = m_lpED->GetExpValue();

	TRACE( "command:MG0( ... )/" );
	if( sw ){
		m_lpSD->GetMsgCtrl()->SetGetMessage( true );
		TRACE(" ON\n" );
	}else{
		m_lpSD->GetMsgCtrl()->SetGetMessage( false );
		TRACE(" OFF\n" );
	}
}
/**********************************************************************
	ϐ̊Jnʒuݒ
**********************************************************************/
void CCommandDriver::CommandMG1( void )
{
	long str_no = m_lpED->GetExpValue() - 1;

	m_lpSD->GetMsgCtrl()->SetGetMessageVarStart( str_no );
	TRACE( "command:MG1( ... )\n" );
}
/**********************************************************************
	RR}hݒ
**********************************************************************/
void CCommandDriver::CommandMG2( void )
{
	long sw = m_lpED->GetExpValue();
	
	if( sw ){
		m_lpSD->GetMsgCtrl()->SetGetMessageRUpdate( false );
	}else{
		m_lpSD->GetMsgCtrl()->SetGetMessageRUpdate( true );
	}
	TRACE( "command:MG2( ... )\n" );
}
/**********************************************************************
	AR}h̓ݒ	
**********************************************************************/
void CCommandDriver::CommandMG3( void )
{
	long sw = m_lpED->GetExpValue();

	m_lpSD->GetMsgCtrl()->SetGetMessageAMode( sw );
	TRACE( "command:MG3( ... )\n" );
}
/**********************************************************************
	ϐւ̎擾Jn
**********************************************************************/
void CCommandDriver::CommandMG4( void )
{
	long no = m_lpED->GetExpValue() - 1;

	if( no < 0 )no = 0;
	m_lpSD->GetMsgCtrl()->SetGetMessageVarNow( no );
	*m_lpED->GetStrVariable( no ) = '\0';
	TRACE( "command:MG4( ... )\n" );
}
/**********************************************************************
	\擾̔ԍ擾
**********************************************************************/
void CCommandDriver::CommandMG5( void )
{
	long *var = m_lpED->GetVariableNo();

	*var = m_lpSD->GetMsgCtrl()->GetGetMessageVarNow();
	TRACE( "command:MG5( ... )\n" );
}
/**********************************************************************
	\擾̔ԍIXV^XV
**********************************************************************/
void CCommandDriver::CommandMG6( void )
{
	long sw = m_lpED->GetExpValue();

	m_lpSD->GetMsgCtrl()->UpdateGetMessage( sw );
	TRACE( "command:MG6( ... )\n" );
}
/**********************************************************************
	擾ςݕ̎擾
**********************************************************************/
void CCommandDriver::CommandMG7( void )
{
	long *var = m_lpED->GetVariableNo();

	*var = 
		strlen( m_lpED->GetStrVariable( 
					m_lpSD->GetMsgCtrl()->GetGetMessageVarNow() )) / 2;
	TRACE( "command:MG7( ... )\n" );
}
/**********************************************************************
	\E\؂ւ
**********************************************************************/
void CCommandDriver::CommandMG100( void )
{
	long sw = m_lpED->GetExpValue();

	TRACE( "command:MG100( ... )" );
	if( !sw ){
		m_lpSD->GetMsgCtrl()->SetOutput( false );
		TRACE(" /Output OFF\n");
	}else{
		m_lpSD->GetMsgCtrl()->SetOutput( true );
		TRACE(" /Output ON\n");
	}
}
/**********************************************************************
	l
**********************************************************************/
void CCommandDriver::CommandMH( void )
{
	long num1 = m_lpED->GetExpValue() - 1;
	long fig  = m_lpED->GetExpValue();
	long num2 = m_lpED->GetExpValue();

	char work[256];
	if( fig ){
		sprintf( work, "%*d", (int)fig, (int)num2 );
	}else{
		sprintf( work, "%d", (int)num2 );
	}
	
	strcpy( m_lpED->GetStrVariable(num1), work );

	TRACE( "command:MH( ... )\n" );
}
/**********************************************************************
	[U[ɂ镶̓
**********************************************************************/
void CCommandDriver::CommandMI( void )
{
	long dst_no = m_lpED->GetExpValue() - 1;
	long max_len= m_lpED->GetExpValue();
	string title;
	m_lpSD->GetParser()->GetString( title, ':' );

	*m_lpED->GetSystemVar() = strlen( 
		m_lpED->GetStrVariable( dst_no ) ) / 2;

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:MI( ... )\n" );
}
/**********************************************************************
	(EBhE)
**********************************************************************/
void CCommandDriver::CommandMJ( void )
{
	long num = m_lpED->GetExpValue() - 1;
	long x	 = m_lpED->GetExpValue();
	long y	 = m_lpED->GetExpValue();
	long max = m_lpED->GetExpValue();

	*m_lpED->GetSystemVar() = strlen( 
		m_lpED->GetStrVariable( num ) ) / 2;

	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:MJ( ... )\n" );
}
/**********************************************************************
	̒𓾂
**********************************************************************/
void CCommandDriver::CommandML( void )
{
	long *var   = m_lpED->GetVariableNo();
	long str_no = m_lpED->GetExpValue() - 1;

	if( str_no >= 0 ){
		*var = strlen( m_lpED->GetStrVariable(str_no)) / 2;
	}else{
		*var = 0;
	}
	TRACE( "command:ML( ... )\n" );
}
/**********************************************************************
	Rs[
**********************************************************************/
void CCommandDriver::CommandMM( void )
{
	long num1 = m_lpED->GetExpValue() - 1;
	long num2 = m_lpED->GetExpValue() - 1;

	strncpy( m_lpED->GetStrVariable( num1 ),
			 m_lpED->GetStrVariable( num2 ),
			 ALSD_STRVAR_LEN );
	TRACE( "command:MM( ... )\n" );
}
/**********************************************************************
	񁃁ϐϊ
**********************************************************************/
void CCommandDriver::CommandMN( void )
{
	long no  = m_lpSD->GetParser()->GetC();
	long num = m_lpED->GetExpValue() - 1;
	long*var = m_lpED->GetVariableNo();
	long len = strlen( m_lpED->GetStrVariable( num ) );

	if( no == 0 ){
	}else{
	}
	TRACE( "error:[Dummy] implement... /" );
	TRACE( "command:MN( ... )\n" );
}
/**********************************************************************
	w蕶\
**********************************************************************/
void CCommandDriver::CommandMP( void )
{
	long num1 = m_lpED->GetExpValue() - 1;
	long num2 = m_lpED->GetExpValue();
	long len = strlen( m_lpED->GetStrVariable( num1 ) );
	char *work = new char[len+1];
	memcpy( work, m_lpED->GetStrVariable( num1 ), len );
	work[len] = '\0';

	if( m_lpSD->GetSelCtrl()->IsSelecting() ){
		// Io^
		m_lpSD->GetSelCtrl()->AddChoice( work );
	}else{
		// bZ[Wo^
		m_lpSD->GetMsgCtrl()->Message( work );
	}
	delete[] work;
	TRACE( "command:MP( ... )\n" );
}
/**********************************************************************
	̓ǂݍ
**********************************************************************/
void CCommandDriver::CommandMS( void )
{
	string str;
	long num = m_lpED->GetExpValue() - 1;
	m_lpSD->GetParser()->GetString( str, ':' );

	if( str.length() ){
		strcpy( m_lpED->GetStrVariable( num ), str.c_str() );
	}else{
		strcpy( m_lpED->GetStrVariable( num ), "" );
	}
	TRACE( "command:MS( ... )\n" );
}
/**********************************************************************
	EBhẼ^Cgݒ肷
**********************************************************************/
void CCommandDriver::CommandMT( void )
{
	string title;
	m_lpSD->GetParser()->GetString( title, ':' );
	m_lpGD->SetWindowTitle( title.c_str() );

	TRACE( "command:MT( ... )\n" );
}
/**********************************************************************
	ViIo[WVXeɒʒm
**********************************************************************/
void CCommandDriver::CommandMV( void )
{
	long version = m_lpED->GetExpValue();

	m_lpSD->SetScenarioVersion( version );
	TRACE( "command:MV( ViIo[W %f )\n", ((float)version)/100.0f );
}
/**********************************************************************
	ϐ̕E̐ݒ̕ύX
**********************************************************************/
void CCommandDriver::CommandMZ0( void )
{
	long max_len = m_lpED->GetExpValue();
	long max_num = m_lpED->GetExpValue();
	long reserve = m_lpED->GetExpValue();

	m_lpED->ReallocStringVar( max_len * 2, max_num );
	TRACE( "command:MZ0( ... )\n" );
}
void CCommandDriver::CommandMXS( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:MXS( ... )\n" );
}
void CCommandDriver::CommandMXA( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:MXA( ... )\n" );
}
void CCommandDriver::CommandMXM( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:MXM( ... )\n" );
}

