/*
 * CommandDriverN.cpp
 *		- "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "CommandDriver.h"

#include "ScenarioDecoder.h"
#include "ScenarioParser.h"
#include "ExpressionDriver.h"
#include "GraphicDriver.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_CMDDRV;
#endif
/**********************************************************************
	N command
**********************************************************************/
void CCommandDriver::_CommandN(void)
{
	switch( m_lpSD->GetParser()->GetC() ){
		case 'B':
			CommandNB(); break;
		case 'C':
			CommandNC(); break;
		case 'D':
			switch( m_lpSD->GetParser()->GetC() ){
				case 'A':
					CommandNDA(); break;
				case 'C':
					CommandNDC(); break;
				case 'D':
					CommandNDD(); break;
				case 'H':
					CommandNDH(); break;
				case 'M':
					CommandNDM(); break;
				case '+':
					CommandND_ADD(); break;
				case '-':
					CommandND_SUB(); break;
				case '*':
					CommandND_MUL(); break;
				case '/':
					CommandND_DIV(); break;
				default:
					throw EALSD_Msg( "error: ND? unknown command\n" );
				}
			break;
		case 'I':
			CommandNI(); break;
		case 'O':
			CommandNO(); break;
		case 'R':
			CommandNR(); break;
		case 'T':
			CommandNT(); break;
		case '+':
			CommandN_ADD(); break;
		case '-':
			CommandN_SUB(); break;
		case '*':
			CommandN_MUL(); break;
		case '/':
			CommandN_DIV(); break;
		case '>':
			CommandN_GT(); break;
		case '<':
			CommandN_LT(); break;
		case '=':
			CommandN_EQ(); break;
		case '\\':
			CommandN_NE(); break;
		case '&':
			CommandN_AND(); break;
		case '|':
			CommandN_OR(); break;
		case '^':
			CommandN_XOR(); break;
		case '~':
			CommandN_NOT(); break;
		default:
			throw EALSD_Msg( "error: N? unknown command\n" );
			break;
	}
}

void CCommandDriver::CommandN_ADD( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_ADD( ... )\n" );
}
void CCommandDriver::CommandN_SUB( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_SUB( ... )\n" );
}
void CCommandDriver::CommandN_MUL( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_MUL( ... )\n" );
}
void CCommandDriver::CommandN_DIV( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_DIV( ... )\n" );
}

void CCommandDriver::CommandN_GT( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_GT( ... )\n" );
}
void CCommandDriver::CommandN_LT( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_LT( ... )\n" );
}
void CCommandDriver::CommandN_EQ( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_EQ( ... )\n" );
}
void CCommandDriver::CommandN_NE( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_NE( ... )\n" );
}

void CCommandDriver::CommandN_AND( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_AND( ... )\n" );
}
void CCommandDriver::CommandN_OR( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_OR( ... )\n" );
}
void CCommandDriver::CommandN_XOR( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_XOR( ... )\n" );
}
void CCommandDriver::CommandN_NOT( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:N_NOT( ... )\n" );
}
/**********************************************************************
	ϐRs[
**********************************************************************/
void CCommandDriver::CommandNB( void )
{
	long *var1 = m_lpED->GetVariableNo();
	long *var2 = m_lpED->GetVariableNo();
	long count = m_lpED->GetExpValue();

	for( long i = 0; i < count; i++){
		var1[i] = var2[i];
	}
	TRACE( "command:NB( ... )\n" );
}
/**********************************************************************
	lONA
**********************************************************************/
void CCommandDriver::CommandNC( void )
{
	long *var = m_lpED->GetVariableNo();
	long count= m_lpED->GetExpValue();

	for( int i = 0; i < count; i++ ){
		var[i] = 0;
	}
	TRACE( "command:NC( ... )\n" );
}
/**********************************************************************
	l
**********************************************************************/
void CCommandDriver::CommandNI( void )
{
	long *var = m_lpED->GetVariableNo();
	long def  = m_lpED->GetExpValue();
	long min  = m_lpED->GetExpValue();
	long max  = m_lpED->GetExpValue();

	*var = (max - min) / 2 + min; 

	TRACE( "error:[Half] implement... /" );
	TRACE( "command:NI( ... )\n" );
}
void CCommandDriver::CommandNP( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:NP( ... )\n" );
}
void CCommandDriver::CommandNR( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:NR( ... )\n" );
}
void CCommandDriver::CommandNO( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:NO( ... )\n" );
}
void CCommandDriver::CommandNT( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:NT( ... )\n" );
}
// {x
void CCommandDriver::CommandND_ADD( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:ND_ADD( ... )\n" );
}
void CCommandDriver::CommandND_SUB( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:ND_SUB( ... )\n" );
}
void CCommandDriver::CommandND_MUL( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:ND_MUL( ... )\n" );
}
void CCommandDriver::CommandND_DIV( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:ND_DIV( ... )\n" );
}
void CCommandDriver::CommandNDA( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:NDA( ... )\n" );
}
void CCommandDriver::CommandNDC( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:NDC( ... )\n" );
}
void CCommandDriver::CommandNDD( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:NDD( ... )\n" );
}
void CCommandDriver::CommandNDH( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:NDH( ... )\n" );
}
void CCommandDriver::CommandNDM( void )
{
	TRACE( "error:No implement... /" );
	TRACE( "command:ND_NDM( ... )\n" );
}

