/*
 * EALSD.h - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#ifndef __EALSD_h__
#define __EALSD_h__
/**********************************************************************
	Alice soft scenario decoder exception
	
**********************************************************************/
#include "ScenarioDecoderSystem.h"
#include "Exception.h"
#include <string>
#include <cstdarg>
//
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD {
#endif
	class EALSD_Msg;
	class EALSD_FileNo;
	class EALSD_Exit;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif
//
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace {
#endif
	const char *const MSG_ERR = "error:%s\n";
	const char *const MSG_EXIT= "exit scenario decoder.\n";
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif
//
// class
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
class ALSD::EALSD_Msg : public err::Exception {
#else
class EALSD_Msg : public Exception {
#endif
public:
	EALSD_Msg( const char *_lpMsg ...){
		va_list vl;
		va_start(vl, _lpMsg );
		vsprintf( m_lpMsg, _lpMsg, vl );
		va_end(vl);
	};
	~EALSD_Msg(){};
	//
	virtual void Message(FILE *fp = stderr){
		fprintf(fp, m_lpMsg );
	}
protected:
	va_list m_vl;
	char m_lpMsg[128];
};

//
// notify file no class
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
class ALSD::EALSD_FileNo : public err::Exception {
#else
class EALSD_FileNo : public Exception {
#endif
public:
	EALSD_FileNo( long _FileNo ){
		m_FileNo = _FileNo;
	};
	virtual ~EALSD_FileNo(){};

	long GetFileNo( void ){return m_FileNo;};

	virtual void Message(FILE *fp = stderr){
		fprintf(fp, "FileNo = %ld\n", m_FileNo );
	};
protected:
	long m_FileNo;
private:
};

//
// notify system exit
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
class ALSD::EALSD_Exit : public err::Exception {
#else
class EALSD_Exit : public Exception {
#endif
public:
	EALSD_Exit(){};
	virtual ~EALSD_Exit(){};

	//
	virtual void Message(FILE *fp = stderr){
		fprintf(fp, MSG_EXIT );
	};
};

#endif
