/*
 * Endian.h - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * LittleEndian.c@xsystem35  get little endian value
 *
 * Copyright (C) 1997-1998 Masaki Chikama (Wren) <chikama@kasumi.ipl.mech.nagoya-u.ac.jp>
 *               1998-                           <masaki-c@is.aist-nara.ac.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * 99/05/03 Atsushi Mizuguchi	EC++ inline header
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
*/

#ifndef __Endian_h__
#define __Endian_h__
/**********************************************************************
	endian value
**********************************************************************/
// 
// unsigned long = DWORD;
// unsigned short= WORD;
// unsigned char = BYTE;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace endian {
#endif
	inline unsigned long GetLE_DW( const unsigned char *_lpMem );
	inline unsigned long GetLE_3B( const unsigned char *_lpMem );
	inline unsigned long GetLE_WD( const unsigned char *_lpMem );
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
inline unsigned long endian::GetLE_DW( const unsigned char *_lpMem ){
#else
inline unsigned long GetLE_DW( const unsigned char *_lpMem ){
#endif
	unsigned long c0, c1, c2, c3;
	c0 = *(_lpMem + 0);
	c1 = *(_lpMem + 1);
	c2 = *(_lpMem + 2);
	c3 = *(_lpMem + 3);
	
	return c0 | c1 << 8 | c2 << 16 | c3 << 24;
};

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
inline unsigned long endian::GetLE_3B( const unsigned char *_lpMem ){
#else
inline unsigned long GetLE_3B( const unsigned char *_lpMem ){
#endif
	unsigned long c0, c1, c2;
	c0 = *(_lpMem + 0);
	c1 = *(_lpMem + 1);
	c2 = *(_lpMem + 2);
	
	return c0 | c1 << 8 | c2 << 16;
}

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
inline unsigned long endian::GetLE_WD( const unsigned char *_lpMem ){
#else
inline unsigned long GetLE_WD( const unsigned char *_lpMem ){
#endif
	unsigned long c0, c1;
	c0 = *(_lpMem + 0);
	c1 = *(_lpMem + 1);
	
	return c0 | c1 << 8;
}

#endif

