/*
 * ExpressionDriver.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * cali.c@xsystem35  vZ̕]
 *
 * Copyright (C) 1997-1998 Masaki Chikama (Wren) <chikama@kasumi.ipl.mech.nagoya-u.ac.jp>
 *               1998-                           <masaki-c@is.aist-nara.ac.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *          0.00    97/11/27 
 *          0.01    97/12/06 vZ̕]
 * @version 0.01-01 97/12/06 bZ[W{폜
 *
 *	99/05/03 Atsushi Mizuguchi EC++
 *	99/05/24 KISAKA Toshihiro	modify for mwcc compatiblility
*/

#include "stdafx.h"
#include "ExpressionDriver.h"

#include "ScenarioDecoder.h"
#include "ScenarioParser.h"
#include "EALSD.h"
#include <stack>

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_EXPDRV;
using std::stack;
#endif
/**********************************************************************
	RXgNV
**********************************************************************/
CExpressionDriver::CExpressionDriver( CScenarioDecoder *_lpScenarioDecoder )
{
	m_lpDS = _lpScenarioDecoder;
	m_lpSystemVar		= new long[ALSD_SYSVAR_MAX];
	m_lpSystemVarAttr	= new stVarAttribute[ALSD_SYSVAR_MAX];
	m_lpStringVar		= new char[ALSD_STRVAR_MAX * ALSD_STRVAR_LEN];
	m_lpVarArrayBuffer	= new stVarArrayBuffer[ALSD_ARRAYVAR_PAGEMAX]; 
	
	memset( m_lpSystemVar, 0, sizeof(long) * ALSD_SYSVAR_MAX );
	memset( m_lpSystemVarAttr, 0, sizeof( stVarAttribute ) * ALSD_SYSVAR_MAX );
	memset( m_lpStringVar, 0, sizeof(char) * ALSD_STRVAR_MAX * ALSD_STRVAR_LEN );
	memset( m_lpVarArrayBuffer, 0, sizeof( stVarArrayBuffer ) * ALSD_ARRAYVAR_PAGEMAX );
}

CExpressionDriver::~CExpressionDriver()
{
	if( m_lpSystemVar )delete[] m_lpSystemVar;
	if( m_lpSystemVarAttr )delete[] m_lpSystemVarAttr;
	if( m_lpStringVar )delete[] m_lpStringVar;
	if( m_lpVarArrayBuffer ){
		for( int i = 0;i < ALSD_ARRAYVAR_PAGEMAX; i++ ){
			if( m_lpVarArrayBuffer[i].lpValue ){
				delete[] m_lpVarArrayBuffer[i].lpValue;
			}
		} 
		delete[] m_lpVarArrayBuffer;
	}	
}

void CExpressionDriver::ReallocStringVar( long _len, long _num )
{
	if( m_lpStringVar )delete[] m_lpStringVar;
	m_lpStringVar		= new char[_num * _len];
	memset( m_lpStringVar, 0, sizeof(char) * _num * _len );

	ALSD_STRVAR_MAX = _num;
	ALSD_STRVAR_LEN = _len;
}



/**********************************************************************

**********************************************************************/
long *CExpressionDriver::GetVariable( void )
{
	long *ret = GetVariable( m_lpDS->GetParser()->GetC() );
//	long debug = m_lpDS->GetParser()->GetC();
	return ret;
}
long *CExpressionDriver::GetVariableNo( void )
{
	long *ret = GetVariable( m_lpDS->GetParser()->GetC() );
	m_lpDS->GetParser()->GetC();
	return ret;
}

char *CExpressionDriver::GetStrVariable( long num )
{
	char *ret = &m_lpStringVar[num * ALSD_STRVAR_LEN];
	return ret;
}

long *CExpressionDriver::GetVariable( long base )
{
	CScenarioParser *lpSP = m_lpDS->GetParser();
	
	if( (base & 0x40) ){ // word valiable
		long offset = lpSP->GetC();
		if( base == 0xc0 ){
			if( offset == 0 ){
				throw EALSD_Msg("error:variable parametor");
			}else if( offset == 1 ) {
				base = (lpSP->GetC() << 8) + lpSP->GetC();	// base
				offset = GetExpValue();				// offset
				return GetOffset(base, offset);
			}else if( offset < 0x40 ) {
				throw EALSD_Msg("error:variable parametor");
			}else{ // 40h - ffh
				return GetOffset(offset, -1);
			}	
		}else{ // 100h - 3fffh
			return GetOffset(((base & 0x3f) * 256) + offset, -1);
		}
	}else{ // byte valiable
		return GetOffset(base & 0x3f, -1);
	}

	return NULL;
}

/**********************************************************************
	ϐ̎擾
**********************************************************************/
long *CExpressionDriver::GetOffset( long _Base, long _Offset )
{
	long *lpIndex;
	long Page;
	long Offset;

#if 0	
	if( m_lpVarArrayBuffer[3].lpValue == (long *)0xfeeefeee ){
		TRACE("ouch\n");
	}
#endif
	m_PrevVarPage	= (m_lpSystemVarAttr + _Base)->Page;
	m_PrevVarNo		= _Base;

	if( m_PrevVarPage == 0 ){
		if( _Offset == -1 ){
			m_PrevVarIndex = _Base;
			return m_lpSystemVar + _Base;
		}else{
			m_PrevVarIndex = _Base + _Offset;
			return m_lpSystemVar +_Base + _Offset;
		}
	}else{
		if( _Offset == -1 ){
			lpIndex	= m_lpSystemVarAttr[_Base].lpVar;
			Page	= m_lpSystemVarAttr[_Base].Page;
			Offset	= m_lpSystemVarAttr[_Base].Offset;
			
			m_PrevVarIndex = Offset + (*lpIndex );
			return (m_lpVarArrayBuffer[Page - 1].lpValue) + m_PrevVarIndex;
		}else{
			Page	= m_lpSystemVarAttr[_Base].Page;
			Offset	= m_lpSystemVarAttr[_Base].Offset;

			m_PrevVarIndex = Offset + _Offset;
			return (m_lpVarArrayBuffer[Page - 1].lpValue) + m_PrevVarIndex;
		}
	}

	return 0;
}

/**********************************************************************
**********************************************************************/
#if 0
long CExpressionDriver::GetExpValue()
{
	CScenarioParser *lpSP = m_lpDS->GetParser();
	stack<long>CalcStack; 
	long c0, c1;
	long iVal,eVal,rVal;

	while( (c0 = lpSP->GetC() ) != ALSD_CALI_TERMINATER) {
		if( (c0 & 0x80) != 0 ) { // l
			CalcStack.push( *GetVariable(c0) );
		} else {
			switch( c0 ){
				case 0x7e:	// !=
					iVal = CalcStack.top(); CalcStack.pop();
					eVal = CalcStack.top(); CalcStack.pop();
					if( eVal != iVal )	CalcStack.push( ALSD_CALI_TRUE  );
					else				CalcStack.push( ALSD_CALI_FALSE );
					break;
				case 0x7d:	// >
					iVal = CalcStack.top(); CalcStack.pop();
					eVal = CalcStack.top(); CalcStack.pop();
					if( eVal > iVal )	CalcStack.push( ALSD_CALI_TRUE  );
					else				CalcStack.push( ALSD_CALI_FALSE );
					break;
				case 0x7c:	// <
					iVal = CalcStack.top(); CalcStack.pop();
					eVal = CalcStack.top(); CalcStack.pop();
					if( eVal < iVal )	CalcStack.push( ALSD_CALI_TRUE  );
					else				CalcStack.push( ALSD_CALI_FALSE );
					break;
				case 0x7b:	// ==
					iVal = CalcStack.top(); CalcStack.pop();
					eVal = CalcStack.top(); CalcStack.pop();
					if( eVal == iVal )	CalcStack.push( ALSD_CALI_TRUE  );
					else				CalcStack.push( ALSD_CALI_FALSE );
					break;
				case 0x7a:	// -
					iVal = CalcStack.top(); CalcStack.pop();
					eVal = CalcStack.top(); CalcStack.pop();
					rVal = eVal - iVal;
					if( rVal < ALSD_CALI_MINVAL )	CalcStack.push( ALSD_CALI_NG );
					else							CalcStack.push( rVal );
					break;
				case 0x79:	// +
					iVal = CalcStack.top(); CalcStack.pop();
					eVal = CalcStack.top(); CalcStack.pop();
					rVal = eVal + iVal;
					if( rVal > ALSD_CALI_MAXVAL )	CalcStack.push( ALSD_CALI_OverF );
					else							CalcStack.push( rVal );
					break;
				case 0x78:	// /
					iVal = CalcStack.top(); CalcStack.pop();
					eVal = CalcStack.top(); CalcStack.pop();
					if( iVal == 0 )	CalcStack.push( ALSD_CALI_ZeroDev );
					else			CalcStack.push( rVal = eVal / iVal );
					break;
				case 0x77:	// *
					iVal = CalcStack.top(); CalcStack.pop();
					eVal = CalcStack.top(); CalcStack.pop();
					rVal = eVal * iVal;
					if( rVal > ALSD_CALI_MAXVAL )	CalcStack.push( ALSD_CALI_OverF );
					else							CalcStack.push( rVal );
					break;
				case 0x76:	// XOR
					iVal = CalcStack.top(); CalcStack.pop();
					eVal = CalcStack.top(); CalcStack.pop();
					CalcStack.push( eVal ^ iVal );
					break;
				case 0x75:	// OR
					iVal = CalcStack.top(); CalcStack.pop();
					eVal = CalcStack.top(); CalcStack.pop();
					CalcStack.push( eVal | iVal );
					break;
				case 0x74:	// AND
					iVal = CalcStack.top(); CalcStack.pop();
					eVal = CalcStack.top(); CalcStack.pop();
					CalcStack.push( eVal & iVal );
					break;
				default:
					if( (c0 & 0x40 ) == 0 ){ // WORD
						c1 = lpSP->GetC();
						if( c0 == 0 ){
							if( c1 <= 0x33 ){
								throw EALSD_Msg( "error:cali(WORD HIGH)" );
							}
						} else { // 
							c1 += ( (c0 & 0x3f) * 256 );
						}
					} else { //
						c1 = (c0 & 0x3f);
					}
					CalcStack.push( c1 );
					break;
			}
		}

	}
	c0 = CalcStack.top(); CalcStack.pop();
	
	if( CalcStack.size() ){
		throw EALSD_Msg( "error:illigal expression" ); 
	}

	return c0;
}
#else
long CExpressionDriver::GetExpValue()
{
	static long Stack[512];
	CScenarioParser *lpSP = m_lpDS->GetParser();
//	stack<long>CalcStack; 
	static long *CalcStack = Stack;
	long *debug = CalcStack;
	long c0, c1;
	long iVal,eVal,rVal;

	while( (c0 = lpSP->GetC() ) != ALSD_CALI_TERMINATER) {
		if( (c0 & 0x80) != 0 ) { // l
			*CalcStack = *GetVariable(c0); CalcStack++;
		} else {
			switch( c0 ){
				case 0x7e:	// !=
					iVal = *--CalcStack;
					eVal = *--CalcStack;
					if( eVal != iVal )	*CalcStack = ALSD_CALI_TRUE;
					else				*CalcStack = ALSD_CALI_FALSE;
					CalcStack++;
					break;
				case 0x7d:	// >
					iVal = *--CalcStack;
					eVal = *--CalcStack;
					if( eVal > iVal )	*CalcStack = ALSD_CALI_TRUE;
					else				*CalcStack = ALSD_CALI_FALSE;
					CalcStack++;
					break;
				case 0x7c:	// <
					iVal = *--CalcStack;
					eVal = *--CalcStack;
					if( eVal < iVal )	*CalcStack = ALSD_CALI_TRUE;
					else				*CalcStack = ALSD_CALI_FALSE;
					CalcStack++;
					break;
				case 0x7b:	// ==
					iVal = *--CalcStack;
					eVal = *--CalcStack;
					if( eVal == iVal )	*CalcStack = ALSD_CALI_TRUE;
					else				*CalcStack = ALSD_CALI_FALSE;
					CalcStack++;
					break;
				case 0x7a:	// -
					iVal = *--CalcStack;
					eVal = *--CalcStack;
					rVal = eVal - iVal;
					if( rVal < ALSD_CALI_MINVAL )*CalcStack = ALSD_CALI_NG;
					else						 *CalcStack = rVal;
					CalcStack++;
					break;
				case 0x79:	// +
					iVal = *--CalcStack;
					eVal = *--CalcStack;
					rVal = eVal + iVal;
					if( rVal > ALSD_CALI_MAXVAL )*CalcStack = ALSD_CALI_OverF;
					else						 *CalcStack = rVal;
					CalcStack++;
					break;
				case 0x78:	// /
					iVal = *--CalcStack;
					eVal = *--CalcStack;
					if( iVal == 0 )	*CalcStack = ALSD_CALI_ZeroDev;
					else			*CalcStack = eVal / iVal;
					CalcStack++;
					break;
				case 0x77:	// *
					iVal = *--CalcStack;
					eVal = *--CalcStack;
					rVal = eVal * iVal;
					if( rVal > ALSD_CALI_MAXVAL )*CalcStack = ALSD_CALI_OverF;
					else						 *CalcStack = rVal;
					CalcStack++;
					break;
				case 0x76:	// XOR
					iVal = *--CalcStack;
					eVal = *--CalcStack;
					*CalcStack = eVal ^ iVal;
					CalcStack++;
					break;
				case 0x75:	// OR
					iVal = *--CalcStack;
					eVal = *--CalcStack;
					*CalcStack = eVal | iVal;
					CalcStack++;
					break;
				case 0x74:	// AND
					iVal = *--CalcStack;
					eVal = *--CalcStack;
					*CalcStack = eVal & iVal;
					CalcStack++;
					break;
				default:
					if( (c0 & 0x40 ) == 0 ){ // WORD
						c1 = lpSP->GetC();
						if( c0 == 0 ){
							if( c1 <= 0x33 ){
								throw EALSD_Msg( "error:cali(WORD HIGH)" );
							}
						} else { // 
							c1 += ( (c0 & 0x3f) * 256 );
						}
					} else { //
						c1 = (c0 & 0x3f);
					}
					*CalcStack = c1; CalcStack++;
					break;
			}
		}

	}
	c0 = *--CalcStack;
	
	if( CalcStack != debug ){
		throw EALSD_Msg( "error:illigal expression" ); 
	}

	return c0;
}
#endif
/**********************************************************************
	z̊mۂƉ
**********************************************************************/
void CExpressionDriver::AllocateArray( long _page, long _size, long _save )
{
	// 
	if( m_lpVarArrayBuffer[_page - 1].lpValue ){
		delete[] m_lpVarArrayBuffer[_page - 1].lpValue;
	}
	m_lpVarArrayBuffer[_page - 1].Max = _size;
	m_lpVarArrayBuffer[_page - 1].lpValue = new long[_size];
	m_lpVarArrayBuffer[_page - 1].SaveFlag= _save;
	memset( m_lpVarArrayBuffer[_page - 1].lpValue, 0, sizeof(long) * _size );
}

/**********************************************************************
	zɒlݒ
**********************************************************************/
void CExpressionDriver::DefineArray( long *_point_var,
									 long _data_var, 
									 long _offset,
									 long _page )
{
	m_lpSystemVarAttr[_data_var].lpVar = _point_var;
	m_lpSystemVarAttr[_data_var].Offset= _offset;
	m_lpSystemVarAttr[_data_var].Page  = _page;
}

/**********************************************************************
	ϐۑ
**********************************************************************/
size_t CExpressionDriver::SaveStringVar( FILE *fp )
{
	long Header[4];
	memset( Header, 0, sizeof(Header) );
	
	char *lpMem = new char[ALSD_STRVAR_LEN * ALSD_STRVAR_MAX];
	memset( lpMem, 0, ALSD_STRVAR_LEN * ALSD_STRVAR_MAX );

	char *lpTmp = lpMem;
	for( long i = 0; i < ALSD_STRVAR_MAX; i++){
		strcpy( lpTmp, &m_lpStringVar[i * ALSD_STRVAR_LEN] );
		lpTmp += strlen( &m_lpStringVar[i * ALSD_STRVAR_LEN] ) + 1;
	}

	Header[0] = lpTmp - lpMem;
	Header[1] = ALSD_STRVAR_MAX;
	Header[2] = ALSD_STRVAR_LEN;
	Header[3] = 0;

	// 
	fwrite( Header, sizeof(long), 4, fp );
	fwrite( lpMem , Header[0], 1, fp );

	delete[] lpMem;

	return Header[0] + (sizeof(long) * 4);
}

/**********************************************************************
	ϐ擾
**********************************************************************/
size_t CExpressionDriver::LoadStringVar( FILE *fp )
{
	// wb_擾
	long Header[4];
	fread( Header, sizeof(long), 4, fp );
	// ̍Ċm
	ReallocStringVar( Header[2], Header[1] );
	
	// {̓ǂݍ
	char *lpMem = new char[ALSD_STRVAR_LEN * ALSD_STRVAR_MAX];
	fread( lpMem, Header[0], 1, fp );

	// WJ
	char *lpTmp = lpMem;
	for( long i = 0; i < ALSD_STRVAR_MAX; i++){
		strcpy( &m_lpStringVar[i * ALSD_STRVAR_LEN], lpTmp );
		lpTmp += strlen( lpTmp ) + 1;
	}

	delete[] lpMem;

	return Header[0] + (sizeof(long) * 4);
}

/**********************************************************************
	VXeϐ̏
**********************************************************************/
size_t CExpressionDriver::SaveSystemVar( FILE *fp )
{
	long Header[4];

	Header[0] = ALSD_SYSVAR_MAX * sizeof(unsigned short);
	Header[1] = 0; // page;
	Header[2] = 0;
	Header[3] = 0;

	// WORDɓWJ
	unsigned short *lpMem = new unsigned short[ALSD_SYSVAR_MAX];
	for( long i = 0; i < ALSD_SYSVAR_MAX; i++){
		lpMem[i] = (unsigned short)m_lpSystemVar[i];
	}
	
	// 
	fwrite( Header, sizeof(long), 4, fp );
	fwrite( lpMem , sizeof(unsigned short), ALSD_SYSVAR_MAX, fp );

	delete[] lpMem;

	return Header[0] + (sizeof(long) * 4 );
}

/**********************************************************************
	VXeϐ̓ǂݍ
**********************************************************************/
size_t CExpressionDriver::LoadSystemVar( FILE *fp )
{
	long Header[4];
	fread( Header, sizeof(long), 4, fp );

	// ǂݍ
	unsigned short *lpMem = new unsigned short[ALSD_SYSVAR_MAX];
	fread( lpMem, Header[0], 1, fp );

	// WORDɓWJ
	for( long i = 0; i < ALSD_SYSVAR_MAX; i++){
		m_lpSystemVar[i] = (long)lpMem[i];
	}

	delete[] lpMem;

	return Header[0] + (sizeof(long) * 4 );
}

/**********************************************************************
	zy[Wɏ
**********************************************************************/
size_t CExpressionDriver::SaveArrayVar ( FILE *fp, long _Page )
{
	// Z[utO`FbN
	if( m_lpVarArrayBuffer[_Page - 1].SaveFlag ){
//		return 0;
	}
	
	long size = m_lpVarArrayBuffer[_Page - 1].Max;
	long Header[4];
	Header[0] = size * sizeof(unsigned short);
	Header[1] = _Page; // page;
	Header[2] = 0;
	Header[3] = 0;

	// WORDɓWJ
	long *lpArray = m_lpVarArrayBuffer[_Page - 1].lpValue;
	unsigned short *lpMem = new unsigned short[size];
	for( long i = 0; i < size; i++){
		lpMem[i] = (unsigned short)lpArray[i];
	}
	
	// 
	fwrite( Header, sizeof(long), 4, fp );
	fwrite( lpMem , sizeof(unsigned short), size, fp );

	delete[] lpMem;

	return Header[0] + (sizeof(long) * 4 );
}

/**********************************************************************
	zy[Wɓǂݍ
**********************************************************************/
size_t CExpressionDriver::LoadArrayVar ( FILE *fp, long _Page )
{
	// Z[utO`FbN
	if( m_lpVarArrayBuffer[_Page - 1].SaveFlag ){
//		return 0;
	}
	
	// wb_ǂݍ
	long Header[4];
	fread( Header, sizeof(long), 4, fp );
	
	// zm
	long size = Header[0] / sizeof(unsigned short);
	AllocateArray( _Page, size, 1 );

	// ǂݍ
	unsigned short *lpMem = new unsigned short[size];
	fread( lpMem, Header[0], 1, fp );

	// longɓWJ
	long *lpArray = m_lpVarArrayBuffer[_Page - 1].lpValue;
	for( long i = 0; i < size; i++){
		lpArray[i] = (long)lpMem[i];
	}
	
	delete[] lpMem;

	return Header[0] + (sizeof(long) * 4 );
}

