/*
 * ExpressionDriver.h - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#ifndef __ExpressionDriver_h__
#define __ExpressionDriver_h__
/**********************************************************************
	scenario decoder system expression driver
	
**********************************************************************/
#include "ScenarioDecoderSystem.h"

//
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD {
#endif
	class CExpressionDriver;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif
//
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD_EXPDRV{
#endif
	const long ALSD_SYSVAR_MAX 		=	1024;
	const long ALSD_SYSVARLONG_MAX	=	128;
	static long ALSD_STRVAR_MAX       =	5000;
	static long ALSD_STRVAR_LEN       =	128;
	const long ALSD_ARRAYVAR_PAGEMAX =	256;

	const long ALSD_CALI_TERMINATER	= 0x7f;
	const long ALSD_CALI_TRUE		= 1;
	const long ALSD_CALI_FALSE		= 0;
	const long ALSD_CALI_MAXVAL		= 65535;
	const long ALSD_CALI_MINVAL		= 0;
	const long ALSD_CALI_ZeroDev	= ALSD_CALI_MINVAL;
	const long ALSD_CALI_OverF		= ALSD_CALI_MAXVAL;
	const long ALSD_CALI_NG			= ALSD_CALI_MINVAL;

	struct stVarAttribute {
		long	*lpVar;
		long	 Page;
		long	 Offset;
	};
	struct stVarArrayBuffer {
		long	 Max;
		long	*lpValue;
		long	 SaveFlag;
	};

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif
//
// class
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
class ALSD::CExpressionDriver {
#else
class CExpressionDriver {
#endif
public:
	CExpressionDriver( CScenarioDecoder *_lpScenarioDecoder );
	virtual ~CExpressionDriver();

	//
	long *GetVariable( void );
	long *GetVariableNo( void );
	char *GetStrVariable( long num );
	long *GetVariable( long base );
	long  GetExpValue( void );

	long *GetSystemVar( void ){ return m_lpSystemVar;};
	
	long GetPrevVarIndex( void ){ return m_PrevVarIndex;};
	long GetPrevVarPage ( void ){ return m_PrevVarPage;};

	void ReallocStringVar( long _len, long _num );

	//
	size_t SaveStringVar( FILE *fp );
	size_t SaveSystemVar( FILE *fp );
	size_t SaveArrayVar ( FILE *fp, long _Page );
	size_t LoadStringVar( FILE *fp );
	size_t LoadSystemVar( FILE *fp );
	size_t LoadArrayVar ( FILE *fp, long _Page );


#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	ALSD_EXPDRV::stVarArrayBuffer *GetArrayPointer( long _Page ){
#else
	stVarArrayBuffer *GetArrayPointer( long _Page ){
#endif
		return &m_lpVarArrayBuffer[ _Page - 1 ];					
	}

	// Command D
	long GetPrevVarNo( void ){ return m_PrevVarNo;};
	void AllocateArray( long _page, long _size, long _save );
	void DefineArray( long *_point_var, long _data_var, long _offset, long _page );
protected:
	//
	CScenarioDecoder	*m_lpDS;
	//
	long						*m_lpSystemVar;		// VXeϐ
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	ALSD_EXPDRV::stVarAttribute	*m_lpSystemVarAttr;	// VXeϐ
#else
	stVarAttribute *m_lpSystemVarAttr;	// VXeϐ
#endif
	char	*m_lpStringVar;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	ALSD_EXPDRV::stVarArrayBuffer *m_lpVarArrayBuffer; // DR}hp
#else
	stVarArrayBuffer *m_lpVarArrayBuffer; // DR}hp
#endif

	//
	long	 m_PrevVarPage;
	long	 m_PrevVarNo;
	long	 m_PrevVarIndex;


	//
	long *GetOffset( long _Base, long _Offset );
private:
};

#endif
