/*
 * FileAld.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "FileAld.h"

#include "FileData.h"
#include "EALSD.h"
#include "FilePtr.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace endian;
using namespace std;
#endif
/**********************************************************************
	RXgNV
**********************************************************************/
CFileAld::CFileAld()
{
	m_lpFileMap = NULL;
	m_lpFilePtr = NULL;
	m_strFileName = "";
	m_FileNo	= 0;
}

CFileAld::~CFileAld()
{
	Destroy();
}

HRESULT CFileAld::Destroy(void)
{
	if( m_lpFileMap )delete[] m_lpFileMap;
	if( m_lpFilePtr )delete[] m_lpFilePtr;

	m_lpFileMap = NULL;
	m_lpFilePtr = NULL;
	m_strFileName = "";

	for( long i = 0; i < m_vcFileData.size(); i++){
		delete m_vcFileData[i];
	}

	return ALSD_OK;
}

/**********************************************************************
	t@CI[v
**********************************************************************/
HRESULT CFileAld::Open( const char *_lpFileName, FILEDATATYPE _type, long _FileNo )
{
	Destroy();

	//
	m_Type = _type;
	m_FileNo = _FileNo;

	/////////////////////////////////////////////////////////
	// t@C̏
	unsigned char Buffer[6];
	CFilePtr fp( _lpFileName, "rb" );
	m_strFileName = _lpFileName;

	//
	long FileSize = fp.GetSize() >> 8;
	fread( Buffer, 1, 6, fp );
	long PtrSize = GetLE_3B( Buffer + 0 );
	long MapSize = GetLE_3B( Buffer + 3 ) - PtrSize;

	// check
	if( PtrSize > FileSize || MapSize > (int)FileSize ){
		throw EALSD_Msg("file format err ALD");
	}

	/////////////////////////////////////////////////////////
	// t@C}bv擾
	m_lpFileMap = new unsigned char[MapSize * 256];
	fseek( fp, PtrSize << 8 , SEEK_SET);
	fread( m_lpFileMap, 256, MapSize, fp);
	
	//////////////////////////////////////////////////////////
	// t@C|C^̎擾
	m_lpFilePtr = new unsigned char[PtrSize * 256];
	fseek( fp, 0, SEEK_SET);
	fread( m_lpFilePtr, 256, PtrSize, fp);

	//
	return ALSD_OK;
}

/**********************************************************************
	t@CN[Y
**********************************************************************/
HRESULT CFileAld::Close( void )
{
	Destroy();
	//
	return ALSD_OK;
}

/**********************************************************************
	t@Cf[^̎擾
**********************************************************************/
HRESULT CFileAld::Load( CFileData *_lpData, long _Num)
{
	//
	if( m_lpFileMap == NULL )return ALSD_ERR_GENERIC;

	/////////////////////////////////////////////////////////
	// t@Cʒu̎擾
	// ???? A = 1, B = 2 ????
	long FileNo = m_lpFileMap[_Num * 3];
	if (FileNo != m_FileNo ){
		throw EALSD_FileNo( FileNo );
	}
	long FileLoc = GetLE_WD( m_lpFileMap + (_Num * 3 + 1) );
	long FilePtr = GetLE_3B( m_lpFilePtr + (FileLoc * 3) );

	if (FileNo < 0 || FileNo > 10 || FileLoc < 0 || FileLoc > 8192 || FilePtr < 0) {
		throw EALSD_Msg( "invaild file request" );
	}

	/////////////////////////////////////////////////////////
	// t@C̓ǂݍ
	//
	char *lpMem;
	char lpName[256];
	long size;
	unsigned char Buffer[256 + 16];
	CFilePtr fp( m_strFileName.c_str(), "rb" );

	// wb_
	FilePtr *= 256;
	fseek( fp, FilePtr, SEEK_SET );
	fread( Buffer, 1, 256+16, fp );
	strncpy( lpName, (const char *)(Buffer + 16), 255 );
	// f[^
	FilePtr += GetLE_DW( Buffer );
	size = GetLE_DW( Buffer + 4 );
	lpMem = new char[size];
	
	fseek( fp, FilePtr, SEEK_SET );
	fread( lpMem, 1, size, fp );
	// 
	
	//
	_lpData->SetType( m_Type );
	_lpData->SetNo	( _Num );
	_lpData->SetName( lpName );
	_lpData->SetData( lpMem, size );

//	TRACE(" Load file... (%s)\n", lpName );

	delete[] lpMem;

	//
	return ALSD_OK;
}

/**********************************************************************
	t@Cf[^̎擾
**********************************************************************/
HRESULT CFileAld::Load( CFileData **_lplpData, long _Num)
{
	//
	if( m_lpFileMap == NULL )return ALSD_ERR_GENERIC;

	if( m_vcFileData.size() > _Num && m_vcFileData[_Num] ){
		*_lplpData = m_vcFileData[_Num];
		return ALSD_OK;
	}
	if( m_vcFileData.size() <= _Num ){
		m_vcFileData.resize( _Num + _Num + 1, NULL );
	}

	/////////////////////////////////////////////////////////
	// t@Cʒu̎擾
	// ???? A = 1, B = 2 ????
	long FileNo = m_lpFileMap[_Num * 3];
	if (FileNo != m_FileNo ){
		throw EALSD_FileNo( FileNo );
	}
	long FileLoc = GetLE_WD( m_lpFileMap + (_Num * 3 + 1) );
	long FilePtr = GetLE_3B( m_lpFilePtr + (FileLoc * 3) );

	if (FileNo < 0 || FileNo > 10 || FileLoc < 0 || FileLoc > 8192 || FilePtr < 0) {
		throw EALSD_Msg( "invaild file request" );
	}

	/////////////////////////////////////////////////////////
	// t@C̓ǂݍ
	//
	char *lpMem;
	char lpName[256];
	long size;
	unsigned char Buffer[256 + 16];
	CFilePtr fp( m_strFileName.c_str(), "rb" );

	// wb_
	FilePtr *= 256;
	fseek( fp, FilePtr, SEEK_SET );
	fread( Buffer, 1, 256+16, fp );
	strncpy( lpName, (const char *)(Buffer + 16), 255 );
	// f[^
	FilePtr += GetLE_DW( Buffer );
	size = GetLE_DW( Buffer + 4 );
	lpMem = new char[size];
	
	fseek( fp, FilePtr, SEEK_SET );
	fread( lpMem, 1, size, fp );
	// 
	
	//
	CFileData *lpData = new CFileData;
	lpData->SetType	( m_Type );
	lpData->SetNo	( _Num );
	lpData->SetName	( lpName );
	lpData->SetData	( lpMem, size );

	*_lplpData = lpData;
	m_vcFileData[_Num] = lpData;

	delete[] lpMem;

	//
	return ALSD_OK;
}



