/*
 * FileCtrl.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "FileCtrl.h"

#include "FileAld.h"
#include "FilePtr.h"
#include "FileData.h"
#include "EALSD.h"
#include "ExpressionDriver.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_EXPDRV;
using namespace ALSD_FILECTR;
using namespace std;
#endif
/**********************************************************************
	RXgNV
**********************************************************************/
CFileCtrl::CFileCtrl( const char *_lpFileList, CScenarioDecoder *_lpSD )
{
	m_lpSD = _lpSD;
	for( int i = 0;i < ALSD_ALDTYPE_MAX;i++ ){
		for( int j = 0;j < ALSD_ALDFILE_MAX;j++ ){
			m_lpAld[i][j] = NULL;
		}
	}
	Create( _lpFileList );

}

CFileCtrl::~CFileCtrl()
{
	Destroy();
}

/**********************************************************************
	j
**********************************************************************/
long CFileCtrl::Create( const char *_lpFileList )
{
	CFilePtr fp( _lpFileList, "rt" );
	//
	long No;
	long Len;
	char Buffer[256];
	char FileName[256];
	// sڂPATH
	fgets( Buffer, sizeof(Buffer), fp );
	Buffer[strcspn(Buffer,"\r\n")] = '\0';
	strcpy( m_lpPath, Buffer);


	while( fgets( Buffer, sizeof(Buffer), fp ) ){
		//
		Buffer[strcspn(Buffer,"\r\n")] = '\0';
		Len = strlen( Buffer ); // ԑ厖
		if( Len < 6 )continue;
		// gqASDH
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
		if( ALSD::stricmp( &Buffer[strlen(Buffer) - 4], ".ASD" ) ){
#else
		if( stricmp( &Buffer[strlen(Buffer) - 4], ".ASD" ) ){
#endif
			// ????
		}
		// gqALDH
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
		if( ALSD::stricmp( &Buffer[strlen(Buffer) - 4], ".ALD" ) ){
#else
		if( stricmp( &Buffer[strlen(Buffer) - 4], ".ALD" ) ){
#endif
			continue;
		}
		// t@CNo 
		No = toupper(Buffer[strlen( Buffer ) - 5]) - 'A';
		if( No >= ALSD_ALDFILE_MAX ){
			continue;
		}
		
		// t@C
		strcpy( FileName, m_lpPath );
		strcat( FileName, Buffer );
		switch( toupper( FileName[strlen( FileName ) - 6]) ){
			case 'S': // Scenario
				if( !No ){
					Buffer[strlen(Buffer) - 4] = '\0';
					strcpy( m_lpCoreName, Buffer );
				}
				if( m_lpAld[DATATYPE_SCO][No] )delete m_lpAld[DATATYPE_SCO][No];
				m_lpAld[DATATYPE_SCO][No] = new CFileAld();
				m_lpAld[DATATYPE_SCO][No]->Open( FileName, DATATYPE_SCO, No+1 );
				break;
			case 'G': // Graphics
				if( m_lpAld[DATATYPE_CG][No] )delete m_lpAld[DATATYPE_CG][No];
				m_lpAld[DATATYPE_CG][No] = new CFileAld();
				m_lpAld[DATATYPE_CG][No]->Open( FileName, DATATYPE_CG, No+1 );
				break;
			case 'M': // MIDI
				if( m_lpAld[DATATYPE_MIDI][No] )delete m_lpAld[DATATYPE_MIDI][No];
				m_lpAld[DATATYPE_MIDI][No] = new CFileAld();
				m_lpAld[DATATYPE_MIDI][No]->Open( FileName, DATATYPE_MIDI, No+1 );
				break;
			case 'W': // Wave
				if( m_lpAld[DATATYPE_WAV][No] )delete m_lpAld[DATATYPE_WAV][No];
				m_lpAld[DATATYPE_WAV][No] = new CFileAld();
				m_lpAld[DATATYPE_WAV][No]->Open( FileName, DATATYPE_WAV, No+1 );
				break;
			case 'D': // DATA
				break;
		}

	}
	

	//
	return ALSD_OK;
}

long CFileCtrl::Destroy( void )
{
	for( int i = 0;i < ALSD_ALDTYPE_MAX;i++ ){
		for( int j = 0;j < ALSD_ALDFILE_MAX;j++ ){
			if( m_lpAld[i][j] )delete m_lpAld[i][j];
		}
	}
	
	//
	return ALSD_OK;
}

/**********************************************************************
	t@C̑݃`FbN(L΂PA΂O
**********************************************************************/
long CFileCtrl::CheckData( FILEDATATYPE _Type, long _Page )
{
	CFileData FileData;
	//
	if( m_lpAld[_Type][0] == NULL )return 0;

	// 
	try {
		m_lpAld[_Type][0]->Load( &FileData, _Page );
	}
	catch( EALSD_FileNo &e ){
		// i[Ăt@C`A.ALDŖꍇ
		long fileno = e.GetFileNo();
		if( m_lpAld[_Type][fileno-1] == NULL ){
			return 0;
		}
		m_lpAld[_Type][fileno-1]->Load( &FileData, _Page );
	}

	//
	return 1;	
}


/**********************************************************************
	t@C[h
**********************************************************************/
long CFileCtrl::LoadData( CFileData *_lpData, FILEDATATYPE _Type,
						  long _Page )
{
	//
	if( m_lpAld[_Type][0] == NULL ){
		throw EALSD_Msg( "not found ALD[%d][%d]\n", _Type, _Page);
	}

	// 
	try {
		m_lpAld[_Type][0]->Load( _lpData, _Page );
	}
	catch( EALSD_FileNo &e ){
		// i[Ăt@C`A.ALDŖꍇ
		long fileno = e.GetFileNo();
		if( m_lpAld[_Type][fileno-1] == NULL ){
			throw EALSD_Msg( "not found ALD([T][F][P]=)[%d][%d][%d]\n", _Type, fileno, _Page);
		}
		m_lpAld[_Type][fileno-1]->Load( _lpData, _Page );
	}

	//
	return ALSD_OK;
}
/**********************************************************************
	t@C[h
**********************************************************************/
long CFileCtrl::LoadData( CFileData **_lplpData, FILEDATATYPE _Type,
						  long _Page )
{
	//
	if( m_lpAld[_Type][0] == NULL ){
		throw EALSD_Msg( "not found ALD[%d][%d]\n", _Type, _Page);
	}

	// 
	try {
		m_lpAld[_Type][0]->Load( _lplpData, _Page );
	}
	catch( EALSD_FileNo &e ){
		// i[Ăt@C`A.ALDŖꍇ
		long fileno = e.GetFileNo();
		if( fileno < 1 )return ALSD_OK; // S{noΉ

		if( m_lpAld[_Type][fileno-1] == NULL ){
			throw EALSD_Msg( "not found ALD([T][F][P]=)[%d][%d][%d]\n", _Type, fileno, _Page);
		}
		m_lpAld[_Type][fileno-1]->Load( _lplpData, _Page );
	}

	//
	return ALSD_OK;
}
/**********************************************************************
	ϐ̓ǂݍ
**********************************************************************/
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
long CFileCtrl::LoadData( const std::string &_strFileName,
						  long *_lpMem, long _Count )
#else
long CFileCtrl::LoadData( const string &_strFileName,
						  long *_lpMem, long _Count )
#endif
{
	// t@CI[v
	string path( m_lpPath );
	path += _strFileName;
	try {
		CFilePtr fp( path.c_str(), "rb" );
		// 16bitœǂ
		unsigned short *lpWord = new unsigned short[_Count];
		size_t size = fread( (void *)lpWord, sizeof( unsigned short ), _Count, fp );
		for( long i = 0; i < _Count; i++){
			_lpMem[i] = (long)lpWord[i];
		}
		delete[] lpWord;
		//
		if( size != _Count ){
			return ALSD_FILECTR_LOADERR;
		}else{
			return ALSD_FILECTR_LOADOK;
		}
	}
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	catch( err::Exception &e ){
#else
	catch( Exception &e ){
#endif
		e.Message();
		return ALSD_FILECTR_LOADERR;
	}
}
/**********************************************************************
	ϐ̏
**********************************************************************/
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
long CFileCtrl::SaveData( const std::string &_strFileName,
						  long *_lpMem, long _Count )
#else
long CFileCtrl::SaveData( const string &_strFileName,
						  long *_lpMem, long _Count )
#endif
{
	// t@CI[v
	string path( m_lpPath );
	path += _strFileName;
	try {
		CFilePtr fp( path.c_str(), "wb" );
		// 16bitɕϊ
		unsigned short *lpWord = new unsigned short[_Count];
		for( int i = 0; i < _Count; i++){
			lpWord[i] = (unsigned short)_lpMem[i];
		}
		//
		size_t size = fwrite( (void *)lpWord, sizeof( unsigned short ), _Count, fp );
		delete[] lpWord;
		//
		if( size != _Count ){
			return ALSD_FILECTR_SAVEERR;
		}else{
			return ALSD_FILECTR_SAVEOK0;
		}
	}
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	catch( err::Exception &e ){
#else
	catch( Exception &e ){
#endif
		e.Message();
		return ALSD_FILECTR_SAVEERR;
	}
}

/**********************************************************************
	ϐ̓ǂݍ
**********************************************************************/
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
long CFileCtrl::LoadDataString( const std::string &_strFileName,
							    long _num, long _Count )
#else
long CFileCtrl::LoadDataString( const string &_strFileName,
							    long _num, long _Count )
#endif
{
	// t@CI[v
	string path( m_lpPath );
	path += _strFileName;
	try {
		CFilePtr fp( path.c_str(), "rb" );
		// t@CTCY擾
		fseek( fp, 0, SEEK_END );
		long FileSize = ftell(fp);
		char *lpWork = new char[FileSize];
		
		// ۂƓǂ
		fseek( fp, 0, SEEK_SET );
		fread( lpWork, sizeof(char), FileSize, fp );

		// 
		char *lpPos = lpWork;
		long len;
		for( int i = 0; i < _Count; i++){
			strncpy( m_lpSD->GetExpDriver()->GetStrVariable( _num + i ),
					 lpPos, ALSD_STRVAR_LEN );
			len = strlen( lpPos ) + 1;
			if( len > ALSD_STRVAR_LEN )len = ALSD_STRVAR_LEN;
			lpPos += len;
		}
		delete[] lpWork;
		
		return ALSD_FILECTR_LOADOK;
	}
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	catch( err::Exception &e ){
#else
	catch( Exception &e ){
#endif
		e.Message();
		return ALSD_FILECTR_LOADERR;
	}
}

/**********************************************************************
	ϐ̏
**********************************************************************/
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
long CFileCtrl::SaveDataString( const std::string &_strFileName,
							    long _num, long _Count )
#else
long CFileCtrl::SaveDataString( const string &_strFileName,
							    long _num, long _Count )
#endif
{
	// t@CI[v
	string path( m_lpPath );
	path += _strFileName;
	try {
		CFilePtr fp( path.c_str(), "wb" );
		//
		long len;
		size_t size;
		for( long i = 0; i < _Count; i++){
			len = strlen( m_lpSD->GetExpDriver()->GetStrVariable( _num + i )) + 1;
			if( len > ALSD_STRVAR_LEN )len = ALSD_STRVAR_LEN;
			size = fwrite( m_lpSD->GetExpDriver()->GetStrVariable( _num + i ),
						   sizeof( char ), len, fp );
			if( size != len ){
				return ALSD_FILECTR_SAVEERR;
			}
		}
		return ALSD_FILECTR_SAVEOK0;
	}
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	catch( err::Exception &e ){
#else
	catch( Exception &e ){
#endif
		e.Message();
		return ALSD_FILECTR_SAVEERR;
	}
}

/**********************************************************************
	Sۑ
**********************************************************************/
long CFileCtrl::SaveDataAll( long _num )
{
	try {
		// t@C߂
		char lpCoreName[256];
		strcpy( lpCoreName, m_lpCoreName );
		lpCoreName[strlen(lpCoreName) - 1] += _num;
		string path( m_lpPath );
		path += lpCoreName;
		path += ".asd";
		// J
		CFilePtr fp( path.c_str(), "wb" );
		stSaveHeader SaveHeader;
		size_t size = sizeof(SaveHeader);
		memset( &SaveHeader, 0, sizeof(SaveHeader));
		fseek( fp, sizeof(SaveHeader), SEEK_SET );

		// X^bN
		SaveHeader.stackinfo = size;
		size += m_lpSD->GetParser()->SaveStack( fp );

		// ϐ
		SaveHeader.StringVar = size;
		size += m_lpSD->GetExpDriver()->SaveStringVar( fp );

		// ϐAz
		SaveHeader.SystemVar[0] = size;
		size += m_lpSD->GetExpDriver()->SaveSystemVar( fp );

		for( long i = 1; i < ALSD_ARRAYVAR_PAGEMAX; i++ ){
			SaveHeader.SystemVar[i] = size;
			size += m_lpSD->GetExpDriver()->SaveArrayVar( fp, i );
			if( SaveHeader.SystemVar[i] == size ){
				SaveHeader.SystemVar[i] = 0;
			}
		}

		// wb_
		memcpy( SaveHeader.Type, ALSD_SAVEDATA_TYPE, 32 );
		SaveHeader.Version = ALSD_SAVEDATA_VERSION;
		SaveHeader.ScenarioIndex = m_lpSD->GetParser()->GetAddress();
		SaveHeader.ScenarioPage  = m_lpSD->GetParser()->GetPage();
		m_lpSD->GetSelCtrl()->GetSaveData( &SaveHeader );
		m_lpSD->GetMsgCtrl()->GetSaveData( &SaveHeader );
		
		fseek( fp, 0, SEEK_SET );
		fwrite( &SaveHeader, sizeof(SaveHeader), 1, fp );

		return ALSD_FILECTR_SAVEOK1;

	}
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	catch( err::Exception &e ){
#else
	catch( Exception &e ){
#endif
		e.Message();
		return ALSD_FILECTR_SAVEERR;
	}
}

/**********************************************************************
	Sǂݍ݂
**********************************************************************/
long CFileCtrl::LoadDataAll( long _num )
{
	try {
		// t@C߂ĊJ
		char lpCoreName[256];
		strcpy( lpCoreName, m_lpCoreName );
		lpCoreName[strlen(lpCoreName) - 1] += _num;
		string path( m_lpPath );
		path += lpCoreName;
		path += ".asd";
		CFilePtr fp( path.c_str(), "rb" );

		stSaveHeader SaveHeader;
		size_t size = sizeof(SaveHeader);


		// TCY`FbN
		fseek( fp, 0, SEEK_END );
		long FileSize = ftell( fp );
		if( FileSize == 0 )throw EALSD_Msg( "File size ZERO.\n" );
		fseek( fp, 0, SEEK_SET );

		// wb_ǂݍ
		fread( &SaveHeader, sizeof(SaveHeader), 1, fp );
		m_lpSD->GetParser()->FarJamp( SaveHeader.ScenarioPage, SaveHeader.ScenarioIndex );
		m_lpSD->GetSelCtrl()->SetLoadData( &SaveHeader );
		m_lpSD->GetMsgCtrl()->SetLoadData( &SaveHeader );
		
		// X^bN
		if( SaveHeader.stackinfo != size )throw EALSD_Msg( "error::Load data stack start\n" );
		size += m_lpSD->GetParser()->LoadStack( fp );

		// ϐ
		if( SaveHeader.StringVar != size )throw EALSD_Msg( "error:Load data string start\n" );
		size += m_lpSD->GetExpDriver()->LoadStringVar( fp );

		// ϐAz
		if( SaveHeader.SystemVar[0] != size )throw EALSD_Msg( "error:Load data system var start\n" );
		size += m_lpSD->GetExpDriver()->LoadSystemVar( fp );

		for( long i = 1; i < ALSD_ARRAYVAR_PAGEMAX; i++ ){
			if( SaveHeader.SystemVar[i] != size )throw EALSD_Msg( "error:Load data array start\n" );
			size += m_lpSD->GetExpDriver()->LoadArrayVar( fp, i );
		}

		//
		return ALSD_FILECTR_LOADOK;
	}
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	catch( err::Exception &e ){
#else
	catch( Exception &e ){
#endif
		e.Message();
		//
		return ALSD_FILECTR_LOADERR;
	}
}

/**********************************************************************
	ϐ̈Ȃǂ̃f[^ꕔZ[u(lϐ) /QP
**********************************************************************/
long CFileCtrl::SaveDataPart( long _Num,   long _Page,
								 long _Index, long _Count )
{
	char *lpMem = NULL;
	try{
		// t@C߂ĊJ
		char lpCoreName[256];
		strcpy( lpCoreName, m_lpCoreName );
		lpCoreName[strlen(lpCoreName) - 1] += _Num;
		string path( m_lpPath );
		path += lpCoreName;
		path += ".asd";
		long FileSize;
		{
			CFilePtr fp( path.c_str(), "rb" );
		
			// ۓǂ
			FileSize = fp.GetSize();
			lpMem = new char[FileSize];
			fread( lpMem, 1, FileSize, fp );
		}
		// Z[uf[^Γǂݍ݈ʒu߂
		stSaveHeader *lpHeader = (stSaveHeader *)lpMem;
		if( lpHeader->SystemVar[_Page] == 0 )throw EALSD_Msg( "wy[Wt@CɊ܂܂Ȃ\n" );
		unsigned short *lpSave =
			(unsigned short *)( (char *)(lpMem + lpHeader->SystemVar[_Page] + sizeof( long ) * 4) );
		lpSave += _Index;
		
		// WJ(4b->2b)
		long *lpVar;
		if( _Page ){
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
			ALSD_EXPDRV::stVarArrayBuffer *lpArray = 
#else
			stVarArrayBuffer *lpArray = 
#endif
				m_lpSD->GetExpDriver()->GetArrayPointer( _Page );
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
			_Count= std::min( lpArray->Max - _Index, _Count );
#else
			_Count= min( lpArray->Max - _Index, _Count );
#endif
			lpVar = lpArray->lpValue + _Index;
		}else{
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
			_Count= std::min( ALSD_EXPDRV::ALSD_SYSVAR_MAX, _Count );
#else
			_Count= min( ALSD_SYSVAR_MAX, _Count );
#endif
			lpVar = m_lpSD->GetExpDriver()->GetSystemVar() + _Index;
		}
		for( long i = 0; i < _Count; i++){
			lpSave[i] = lpVar[i];	
		}

		// 
		CFilePtr fp(path.c_str(), "wb");
		fwrite( lpMem, sizeof(char), FileSize, fp);
		
		delete[] lpMem;
	
		return ALSD_FILECTR_SAVEOK1;
	}
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	catch( err::Exception &e ){
#else
	catch( Exception &e ){
#endif
		if( lpMem )delete[] lpMem;
		e.Message();
		return ALSD_FILECTR_SAVEERR;
	}
	
}

/**********************************************************************
	Z[uf[^̈ꕔ[h(lϐ) /LP
**********************************************************************/
long CFileCtrl::LoadDataPart( long _num,   long _Page,
								 long _Index, long _Count )
{
	char *lpMem = NULL;
	try{
		// t@C߂ĊJ
		char lpCoreName[256];
		strcpy( lpCoreName, m_lpCoreName );
		lpCoreName[strlen(lpCoreName) - 1] += _num;
		string path( m_lpPath );
		path += lpCoreName;
		path += ".asd";
		CFilePtr fp( path.c_str(), "rb" );
		
		// TCY`FbN
		long FileSize = fp.GetSize();

		// ۓǂ
		lpMem = new char[FileSize];
		fread( lpMem, FileSize, 1, fp );
		
		// Z[uf[^Γǂݍ݈ʒu߂
		stSaveHeader *lpHeader = (stSaveHeader *)lpMem;
		if( lpHeader->SystemVar[_Page] == 0 )throw EALSD_Msg( "wy[Wt@CɊ܂܂Ȃ\n" );
		unsigned short *lpLoad =
			(unsigned short *)( (char *)(lpMem + lpHeader->SystemVar[_Page] + sizeof( long ) * 4) );
		lpLoad += _Index;
		
		// WJ(2b->4b)
		long *lpVar;
		if( _Page ){
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
			ALSD_EXPDRV::stVarArrayBuffer *lpArray = 
#else
			stVarArrayBuffer *lpArray = 
#endif
				m_lpSD->GetExpDriver()->GetArrayPointer( _Page );
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
			_Count= std::min( lpArray->Max - _Index, _Count );
#else
			_Count= min( lpArray->Max - _Index, _Count );
#endif
			lpVar = lpArray->lpValue + _Index;
		}else{
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
			_Count= std::min( ALSD_EXPDRV::ALSD_SYSVAR_MAX, _Count );
#else
			_Count= min( ALSD_SYSVAR_MAX, _Count );
#endif
			lpVar = m_lpSD->GetExpDriver()->GetSystemVar() + _Index;
		}
		for( long i = 0; i < _Count; i++){
			lpVar[i] = lpLoad[i];	
		}
		
		delete[] lpMem;
		return ALSD_FILECTR_LOADOK;
	}
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	catch( err::Exception &e ){
#else
	catch( Exception &e ){
#endif
		if( lpMem )delete[] lpMem;
		e.Message();
		return ALSD_FILECTR_LOADERR;
	}
}

/**********************************************************************
	Z[uf[^̃Rs[
**********************************************************************/
long CFileCtrl::CopyData( long _dst, long _src )
{
	unsigned char *lpMem = NULL;
	try{
		// t@C߂ĊJ
		char lpCoreName[256];
		strcpy( lpCoreName, m_lpCoreName );
		lpCoreName[strlen(lpCoreName) - 1] += _src;
		string path( m_lpPath );
		path += lpCoreName;
		path += ".asd";
		CFilePtr src( path.c_str(), "rb" );

		strcpy( lpCoreName, m_lpCoreName );
		lpCoreName[strlen(lpCoreName) - 1] += _dst;
		path = m_lpPath;
		path += lpCoreName;
		path += ".asd";
		CFilePtr dst( path.c_str(), "wb" );
	
		// Rs[
		fseek( src, 0, SEEK_END );
		long size = ftell( src );
		fseek( src, 0, SEEK_SET );

		lpMem = new unsigned char[size];
		fread( lpMem, sizeof(unsigned char), size, src );
		fwrite( lpMem, sizeof(unsigned char),size, dst );
	
		return ALSD_FILECTR_SAVEOK1;
	}
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	catch( err::Exception &e ){
#else
	catch( Exception &e ){
#endif
		if( lpMem )delete[] lpMem;
		e.Message();
		return ALSD_FILECTR_SAVEERR;
	}



}

