/*
 * GraphicDriver.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "GraphicDriver.h"

#include "AliceCG_PMS256.h"
#include "AliceCG_PMS64K.h"
#include "OffScreen.h"
#include "FileCtrl.h"
#include "FileData.h"
#include "FilePtr.h"
#include "CommandDriver.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace std;
using namespace err;
using namespace ALSD;
using namespace ALSD_ALCG;
using namespace ALSD_OFFSCR;
#endif

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace {
#endif
	// PC flag
	const long PC_LOADCG		= 0x00000001;
	const long PC_REALIZEPALETTE= 0x00000002;
	const long PC_LOADPALETTE	= 0x00000004;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif

// static member
bool		  CGraphicDriver::m_bInitialize = false;
unsigned char CGraphicDriver::m_AlphaBlendTable[ALSD_ABTABLE_DEPTH][256];
unsigned char CGraphicDriver::m_lp5BitTbl[32];
unsigned char CGraphicDriver::m_lp6BitTbl[64];
/**********************************************************************
	RXgNV
**********************************************************************/
CGraphicDriver::CGraphicDriver( CScenarioDecoder *_lpScenarioDecoder )
{
	//
//	m_ViewX = 640;
//	m_ViewY = 480;
	m_ViewC = 32;
	m_DecorateType = 0;
	m_DecorateColor= 255;
	// 
	m_VKeyCode	= 0;
	m_bUpdate	= true;
	m_lpSD		= _lpScenarioDecoder;

	m_bLBDownNow = false;
	m_bRBDownNow = false;

	m_ptCGLoad.x = m_ptCGLoad.y = 0;
	m_CGLoadPosFlag = -1;
	m_Brightness	= 255;

	m_lpCGLoadCount = NULL;

	m_GPC = 7;
	
	// pbg̏
	for( int i = 0; i < 256; i++){
		m_Palette.entry[i].alpha = 0;
		m_Palette.entry[i].red	 = 0;
		m_Palette.entry[i].green = 0;
		m_Palette.entry[i].blue	 = 0;
	}
	m_Palette.entry[255].red  = 255;
	m_Palette.entry[255].green= 255;
	m_Palette.entry[255].blue = 255;
	m_Palette.entry[15].red  = 255;
	m_Palette.entry[15].green= 255;
	m_Palette.entry[15].blue = 255;

	/////////////////////////////////////////////////////////
	// uhe[u𐶐
	if( m_bInitialize == false ){
		//
		m_bInitialize = true;
		// 0
		for( int j = 0; j < 256; j++){
			m_AlphaBlendTable[0][j] = 0;
		}
		//
		for( int i = 1; i < ALSD_ABTABLE_DEPTH - 1; i++ ){
			for( int j = 0; j < 256; j++){
				m_AlphaBlendTable[i][j] = 
					(unsigned char)(j * (i * (256 / ALSD_ABTABLE_DEPTH)) / 255);
			}
		}
		// 255
		for( int j = 0; j < 256; j++){
			m_AlphaBlendTable[ALSD_ABTABLE_DEPTH - 1][j] = j;
		}
	
		// 16bit>24bitWJe[u
		for( int i = 0; i < 32; i++)m_lp5BitTbl[i] = i * 255 / 31;
		for( int i = 0; i < 64; i++)m_lp6BitTbl[i] = i * 255 / 63;
	}

}

CGraphicDriver::~CGraphicDriver()
{
	for( long i = 0; i < m_vcSaveInfo.size(); i++){
		if( m_vcSaveInfo[i] ){
			delete[] m_vcSaveInfo[i]->lpPtr;
			delete[] m_vcSaveInfo[i];
			m_vcSaveInfo[i] = NULL;
		}
	}
}

/**********************************************************************
	F̏d˂킹iZł͂Ȃč
**********************************************************************/
void CGraphicDriver::AddColorRect(long _x, long _y, long _w, long _h,
								  unsigned char _c, long _rate )
{
	long dr, dg, db;
	long sr, sg, sb;

	// F擾
	dr = ((long)m_Palette.entry[_c].red)	* _rate / 255;
	dg = ((long)m_Palette.entry[_c].green)	* _rate / 255;
	db = ((long)m_Palette.entry[_c].blue)	* _rate / 255;
	long srate = (255 - _rate) >> ALSD_ABTABLE_BITS;
	// Lock
	stGrpScreenDesc	 Desc;
	stGrpScreenBits  Bits;
	if( !m_lpScreen->Lock( &Bits, &Desc ) ){
		return;
	}
	unsigned long	*lpSrcPos = Bits.lpCBits;

	
	long pitch = Desc.lPitch / (Desc.dgpfPixelFormat.dwRGBABitCount / 8 );
	// n_
	lpSrcPos += ( _x + _y * pitch );
	
	unsigned long *lpSrcWork;
	for( long y = 0; y < _h; y++ ){
		lpSrcWork = lpSrcPos;
		for( long x = 0; x < _w; x++ ){
			// \[X
			sr = ((*lpSrcWork & Desc.dgpfPixelFormat.dwRBitMask) >> 16);
			sg = ((*lpSrcWork & Desc.dgpfPixelFormat.dwGBitMask) >>  8);
			sb = ((*lpSrcWork & Desc.dgpfPixelFormat.dwBBitMask)      );
			// Z
			sr = dr + m_AlphaBlendTable[srate][sr];
			if( sr > 255 )sr = 255;
			sg = dg + m_AlphaBlendTable[srate][sg];
			if( sg > 255 )sg = 255;
			sb = db + m_AlphaBlendTable[srate][sb];
			if( sb > 255 )sb = 255;
			// 
			*lpSrcWork = (sr << 16) | (sg << 8) | sb;
			lpSrcWork++;
		}
		lpSrcPos += pitch;
	}
	// Unlock
	m_lpScreen->Unlock();
}

/**********************************************************************
**********************************************************************/
void CGraphicDriver::FillRect	( long _x, long _y, long _w, long _h, unsigned char _c )
{
	unsigned long rgb = (long)(m_Palette.entry[_c].red << 16) + 
						(long)(m_Palette.entry[_c].green <<8) + 
						(long)m_Palette.entry[_c].blue;

	m_lpScreen->FillRect( _x, _y, _w, _h, rgb );
}
//
//
void CGraphicDriver::StrokeRect	( long _x, long _y, long _w, long _h, unsigned char _c )
{
	unsigned long rgb = (long)(m_Palette.entry[_c].red << 16) + 
						(long)(m_Palette.entry[_c].green <<8) + 
						(long)m_Palette.entry[_c].blue;

	m_lpScreen->StrokeRect( _x, _y, _w, _h, rgb );
}
//
//
void CGraphicDriver::StrokeLine	( long _x1,long _y1,long _x2,long _y2,unsigned char _c )
{
	unsigned long rgb = (long)(m_Palette.entry[_c].red << 16) + 
						(long)(m_Palette.entry[_c].green <<8) + 
						(long)(m_Palette.entry[_c].blue);

	m_lpScreen->StrokeLine( _x1, _y1, _x2, _y2, rgb );
}

//
//
void CGraphicDriver::DrawString( long _x, long _y, const char *_lpString, unsigned char _c )
{
	unsigned long rgb = (long)(m_Palette.entry[_c].red << 16) + 
						(long)(m_Palette.entry[_c].green <<8) + 
						(long)(m_Palette.entry[_c].blue);
	unsigned long dec = (long)(m_Palette.entry[m_DecorateColor].red << 16) + 
						(long)(m_Palette.entry[m_DecorateColor].green <<8) + 
						(long)(m_Palette.entry[m_DecorateColor].blue);
	
	switch( m_DecorateType ){
		case 0:
			m_lpScreen->DrawString( _x, _y, _lpString, rgb );
			break;
		case 1: // etij
			m_lpScreen->DrawString( _x, _y + 1, _lpString, dec );
			m_lpScreen->DrawString( _x, _y    , _lpString, rgb );
			break;
		case 2: // etiEj
			m_lpScreen->DrawString( _x + 1, _y, _lpString, dec );
			m_lpScreen->DrawString( _x    , _y, _lpString, rgb );
			break;
		case 3: // etiEj
			m_lpScreen->DrawString( _x    , _y + 1, _lpString, dec );
			m_lpScreen->DrawString( _x + 1, _y    , _lpString, dec );
			m_lpScreen->DrawString( _x    , _y    , _lpString, rgb );
			break;
		case 4: // AEgCt...?
			m_lpScreen->DrawString( _x    , _y    , _lpString, rgb );
			break;
		case 6: // ij
			m_lpScreen->DrawString( _x, _y + 1, _lpString, rgb );
			m_lpScreen->DrawString( _x, _y    , _lpString, rgb );
			break;
		case 7: // iEj
			m_lpScreen->DrawString( _x + 1, _y, _lpString, rgb );
			m_lpScreen->DrawString( _x    , _y, _lpString, rgb );
			break;
		case 8: // iEj
			m_lpScreen->DrawString( _x    , _y + 1, _lpString, rgb );
			m_lpScreen->DrawString( _x + 1, _y    , _lpString, rgb );
			m_lpScreen->DrawString( _x    , _y    , _lpString, rgb );
			break;
		case 9: // A_[C
			long fontsize;
			fontsize = m_lpScreen->GetFontSize();
			m_lpScreen->DrawString( _x, _y, _lpString, rgb );
			m_lpScreen->StrokeLine( _x, _y + fontsize+1,
									_x + strlen(_lpString) * fontsize / 2,
									_y + fontsize+1, rgb );
			break;
		case 10: // 3 + 4
			m_lpScreen->DrawString( _x    , _y + 1, _lpString, dec );
			m_lpScreen->DrawString( _x + 1, _y    , _lpString, dec );
			m_lpScreen->DrawString( _x    , _y    , _lpString, rgb );
		default:
			m_lpScreen->DrawString( _x, _y, _lpString, rgb );
			break;
	}
}
//
//
void CGraphicDriver::Reverse( long _x, long _y, long _w, long _h )
{
//	m_lpScreen->Reverse( _x, _y, _w, _h );
	// bN
	stGrpScreenDesc Desc;
	stGrpScreenBits Bits;
	m_lpScreen->Lock( &Bits, &Desc );

	long pitch = Desc.lPitch / 4;
	unsigned long *lpSrcPos = Bits.lpCBits;
	lpSrcPos += ( _x + _y * pitch );
	
	unsigned long *lpSrcWork;
	//
	for( long i = 0; i < _h; i++ ){
		lpSrcWork = lpSrcPos;
		for( long x = 0; x < _w; x++ ){
			*lpSrcWork = (*lpSrcWork) ^ 0x00FFFFFF;
			lpSrcWork++;
		}
		lpSrcPos += pitch;
	}
	//
	m_lpScreen->Unlock();
}

/**********************************************************************
	PMSF`FbN
**********************************************************************/
long CGraphicDriver::CheckPMSFormat( const unsigned char *_lpMem )
{
	return _lpMem[6];
}

/**********************************************************************
**********************************************************************/
void CGraphicDriver::OnDraw( long _x, long _y, long _w, long _h )
{
#ifdef NDEBUG
	if( _y > m_ViewRect.bottom || _x > m_ViewRect.right ){
		return;
	}
#endif
	stGrpRect update;
	update.left = _x;
	update.top	= _y;
	update.right= _w + update.left;
	update.bottom=_h + update.top;

#ifdef NDEBUG
	if( update.left  < m_ViewRect.left  )update.left  = m_ViewRect.left;
	if( update.top   < m_ViewRect.top   )update.top	  = m_ViewRect.top;
	if( update.right > m_ViewRect.right )update.right = m_ViewRect.right;
	if( update.bottom> m_ViewRect.bottom)update.bottom= m_ViewRect.bottom;
#endif
	OnDraw( &update );
}
/**********************************************************************
	}EXnh
**********************************************************************/
void CGraphicDriver::OnLButtonDown( stGrpPoint *_lpPoint )
{
	m_bLBDownNow = true;
	TRACE(" LButtonDown\n");
}

void CGraphicDriver::OnLButtonUp( stGrpPoint *_lpPoint )
{
	m_bLBDownNow = false;
	TRACE(" LButtonUp\n");
}
void CGraphicDriver::OnRButtonDown( stGrpPoint *_lpPoint )
{
	m_bRBDownNow = true;
}

void CGraphicDriver::OnRButtonUp( stGrpPoint *_lpPoint )
{
	m_bRBDownNow = false;
}
void CGraphicDriver::OnMouseMove( stGrpPoint *_lpPoint )
{
	m_ptMousePos = *_lpPoint;

	stGrpRect rect1 = {50, 50, 50+100, 50+50};

}
/**********************************************************************
	}EX`FbN
**********************************************************************/
bool CGraphicDriver::IsLButtonDown( stGrpPoint *_lpPt )
{
	if( _lpPt ){
		*_lpPt = m_ptMousePos;
	}
	return m_bLBDownNow;
}

bool CGraphicDriver::IsRButtonDown( stGrpPoint *_lpPt )
{
	if( _lpPt ){
		*_lpPt = m_ptMousePos;
	}
	return m_bRBDownNow;
}
/**********************************************************************
	L[R[hnh
**********************************************************************/
void CGraphicDriver::OnKeyDown( unsigned int _KeyCode )
{
	m_VKeyCode = _KeyCode;
}

void CGraphicDriver::OnKeyUp( unsigned int _KeyCode )
{
	if( m_VKeyCode == _KeyCode ){
		m_VKeyCode = 0;
	}
}

/**********************************************************************
	WindowsDIB̃[h
**********************************************************************/
void CGraphicDriver::LoadDIB( long _x, long _y, const char *_lpFileName )
{
	try{
		string strFileName;
		strFileName = m_lpSD->GetFileCtrl()->GetPath();
		strFileName += _lpFileName;
		CFilePtr fp( strFileName.c_str(), "rb" );
		// TCYv
		fseek( fp, 0, SEEK_END );
		long size = ftell( fp );
		fseek( fp, 0, SEEK_SET );
		// ۂƓǂ
		unsigned char *lpMem = new unsigned char[size];
		fread( lpMem, 1, size, fp );
		// `FbN
		if( memcmp( lpMem, "BM", 2 ) ){
			delete[] lpMem;
			return;
		}
		// WJ 
		stBitmapInfoHeader *lpBMIHeader = (stBitmapInfoHeader *)(lpMem + 14);
		unsigned char *lpBitmap = new unsigned char[ lpBMIHeader->biWidth * abs(lpBMIHeader->biHeight) * 4 ];
		if( lpBMIHeader->biBitCount == 8 ){
			LoadDIB256( lpMem, lpBitmap );
		}else{
			LoadDIB16M( lpMem, lpBitmap );
		}

		stGrpScreenBits Bits;
		stGrpScreenDesc Desc;
		// bN
		if( !m_lpScreen->Lock( &Bits, &Desc ) )return;
		// ʒu
		long pitch = Desc.lPitch / 4;
		unsigned long *lpSrcPos = (unsigned long *)lpBitmap;
		unsigned long *lpDstPos = Bits.lpCBits;
		lpDstPos += ( _x + _y * pitch );
		//
		for( long i = 0; i < lpBMIHeader->biHeight; i++ ){
			memcpy( lpDstPos, lpSrcPos, lpBMIHeader->biWidth * 4 );
			lpSrcPos += lpBMIHeader->biWidth;
			lpDstPos += pitch;
		}
		// Unlock
		m_lpScreen->Unlock();
		OnDraw( _x, _y, lpBMIHeader->biWidth, lpBMIHeader->biHeight );

		delete[] lpBitmap;
		delete[] lpMem;
	}
	catch( Exception &e ){
		e.Message();
	}
}

/**********************************************************************
	rbg}bv̓WJQTUF
**********************************************************************/
void CGraphicDriver::LoadDIB256	( const unsigned char *_lpMem,
										unsigned char *_lpBitmap )
{
	stBitmapInfoHeader *lpBMIHeader = 
		(stBitmapInfoHeader *)(_lpMem + 14);
}

/**********************************************************************
	rbg}bv̓WJ24bittJ[
**********************************************************************/
void CGraphicDriver::LoadDIB16M	( const unsigned char *_lpMem,
										unsigned char *_lpBitmap )
{
	stBitmapInfoHeader *lpBMIHeader = 
		(stBitmapInfoHeader *)(_lpMem + 14);
	long WidthByte = WIDTHBYTES( (lpBMIHeader->biWidth) * ( lpBMIHeader->biBitCount ));
	const unsigned char *lpBits = (_lpMem + 14 + lpBMIHeader->biSize);

	if( lpBMIHeader->biHeight < 0 ){
		// gbv_Echa
		long Height = -lpBMIHeader->biHeight;
		for( long y = 0; y < Height; y++){
			for( long x = 0; x < lpBMIHeader->biWidth * 3; x+=3){
				_lpBitmap[y * lpBMIHeader->biWidth * 4 + x/3*4 + 0] = 
					lpBits[ y * WidthByte + x + 0];
				_lpBitmap[y * lpBMIHeader->biWidth * 4 + x/3*4 + 1] = 
					lpBits[ y * WidthByte + x + 1];
				_lpBitmap[y * lpBMIHeader->biWidth * 4 + x/3*4 + 2] = 
					lpBits[ y * WidthByte + x + 2];
				_lpBitmap[y * lpBMIHeader->biWidth * 4 + x/3*4 + 3] = 
					0;
			}
		}
	}else{
		// {gAbvcha
		long Height = lpBMIHeader->biHeight;
		for( long y = 0; y < Height; y++){
			for( long x = 0; x < lpBMIHeader->biWidth * 3; x+=3){
				_lpBitmap[(Height - 1 - y) * lpBMIHeader->biWidth * 4 + x/3*4 + 0] = 
					lpBits[ y * WidthByte + x + 0 ];
				_lpBitmap[(Height - 1 - y) * lpBMIHeader->biWidth * 4 + x/3*4 + 1] = 
					lpBits[ y * WidthByte + x + 1 ];
				_lpBitmap[(Height - 1 - y) * lpBMIHeader->biWidth * 4 + x/3*4 + 2] = 
					lpBits[ y * WidthByte + x + 2 ];
				_lpBitmap[(Height - 1 - y) * lpBMIHeader->biWidth * 4 + x/3*4 + 3] = 
					0;
			}			
		}
	
	}

}


/**********************************************************************
	CG̃TCY𓾂
**********************************************************************/
void CGraphicDriver::GetCGSizeInfo( long _Link, stCGSize *_lpSize )
{
	CFileData data;
	
	// t@Cǂ
	m_lpSD->GetFileCtrl()->LoadData( &data, DATATYPE_CG , _Link );
	// f[^`FbN
	CAliceCG *lpCG = NULL;
	switch( CheckPMSFormat( (unsigned char *)data.GetData() ) ){
		case  4:
			break;
		case  8:
			lpCG = new CAliceCG_PMS256( (unsigned char *)data.GetData() );
			break;
		case 16:
			lpCG = new CAliceCG_PMS64K( (unsigned char *)data.GetData() );
			break;
	}
	lpCG->GetCGSizeInfo(_lpSize);	
}

/**********************************************************************
	CG[h
**********************************************************************/
void CGraphicDriver::LoadCG( long _Link, long _SpriteColor )
{
	CFileData data;
	
	// t@Cǂ
	m_lpSD->GetFileCtrl()->LoadData( &data, DATATYPE_CG , _Link );
	if( data.GetData() == NULL ){
		return ; // S{noΉ
	}

	// f[^`FbN
	CAliceCG *lpCG = NULL;
	CAliceCG_PMS *lpPMS = NULL;
	switch( CheckPMSFormat( (unsigned char *)data.GetData() ) ){
		case  4:
			break;
		case  8:
			lpCG = lpPMS = new CAliceCG_PMS256( (unsigned char *)data.GetData() );
			lpCG->SetSpriteColor( _SpriteColor );
			stGrpRect update;
			stCGSize  size;
			lpCG->GetCGSizeInfo(&size);	
			// pbg̓WJ
			if( m_GPC & PC_LOADPALETTE ){
				lpPMS->ExpandPal( &m_Palette );
			}
			// pbg̃ACY
			if( m_GPC & PC_REALIZEPALETTE ){
			}
			// bfWJ
			if( m_GPC & PC_LOADCG ){
				lpCG->DrawBitmap( *m_lpScreen, m_ptCGLoad.x, m_ptCGLoad.y,
						   m_CGLoadPosFlag, &update, m_Brightness );
				OnDraw( &update );
			}
			break;
		case 16:
			lpCG = lpPMS =new CAliceCG_PMS64K( (unsigned char *)data.GetData() );
			if( m_GPC & PC_LOADCG ){
				BltCG64K( lpPMS, _SpriteColor );
			}
			break;
	}
	if( lpCG )delete lpCG;

	// JR}h̃NA
	if( m_bCGLoadPosUseOnce ){
		m_ptCGLoad.x = 0;
		m_ptCGLoad.y = 0;
		m_CGLoadPosFlag = -1;
		m_bCGLoadPosUseOnce = 0;
	}
	// ZG
	if( m_lpCGLoadCount ){
		m_lpCGLoadCount[_Link+1]++;
	}
}
/**********************************************************************
	ef[^w胍[h
**********************************************************************/
void CGraphicDriver::LoadCGX(long _Link, long _Shadow )
{
	CFileData data;
	CFileData shadow;
	CAliceCG_PMS256 *lpPMS256 = NULL;
	CAliceCG_PMS64K *lpPMS64K = NULL;
	
	// t@Cǂ
	// bf
	if( _Link != -1 ){
		m_lpSD->GetFileCtrl()->LoadData( &data,  DATATYPE_CG, _Link );
		lpPMS64K = new CAliceCG_PMS64K( (unsigned char *)data.GetData() );
	}
	// ef[^
	m_lpSD->GetFileCtrl()->LoadData( &shadow,DATATYPE_CG, _Shadow ); 
	lpPMS256 = new CAliceCG_PMS256( (unsigned char *)shadow.GetData() );

	// 
	if( lpPMS64K ){
		lpPMS64K->SetShadow( lpPMS256 );
		BltCG64K( lpPMS64K, 0 );
	}else{
		BltCGAlpha( lpPMS256 );
	}
	
	if( lpPMS256 )delete lpPMS256;
	if( lpPMS64K )delete lpPMS64K;

	// JR}h̃NA
	if( m_bCGLoadPosUseOnce ){
		m_ptCGLoad.x = 0;
		m_ptCGLoad.y = 0;
		m_CGLoadPosFlag = -1;
		m_bCGLoadPosUseOnce = 0;
	}
	// ZG
	if( m_lpCGLoadCount ){
		m_lpCGLoadCount[_Link+1]++;
	}
}

/**********************************************************************
	ւ̕`
**********************************************************************/
void CGraphicDriver::BltCGAlpha( CAliceCG_PMS *_lpAliceCG )
{
	stCGSize  size;
	_lpAliceCG->GetCGSizeInfo(&size);	

	long dx, dy;
	switch( m_CGLoadPosFlag ){
		case -1:	// ftHg
			dx = size.x;
			dy = size.y;
			break;
		case  0:	// ΍W
			dx = m_ptCGLoad.x;
			dy = m_ptCGLoad.y;
			break;
		case  1:	// ΍W
			dx = size.x + m_ptCGLoad.x;
			dy = size.y + m_ptCGLoad.y;
			break;
	}

	//
	stGrpRect rect;
	rect.left	= dx;
	rect.top	= dy;
	rect.right	= dx + size.Width;
	rect.bottom = dy + size.Height;

	// WJ
	unsigned char	*lpAlp = new unsigned char [size.Height * size.Width];
	_lpAliceCG->Expand256( lpAlp, -1 );

	// bN
	stGrpScreenDesc Desc;
	stGrpScreenBits Bits;
	m_lpScreen->Lock( &Bits, &Desc );

	//
	long pitch = Desc.lPitch / 4;
	unsigned char *lpASrcPos = lpAlp;
	unsigned char *lpADstPos = Bits.lpABits;
	lpADstPos= lpADstPos+ ( dx + dy * pitch ); 

	for( long y = 0; y < size.Height; y++){
		memcpy( lpADstPos, lpASrcPos, size.Width ); 
		lpASrcPos += pitch;
		lpADstPos += pitch;
	}

	// n
	m_lpScreen->Unlock();
	delete[] lpAlp;
}

/**********************************************************************
	CG`悷
**********************************************************************/
void CGraphicDriver::BltCG64K( CAliceCG_PMS *_lpAliceCG,
							   long _SpriteColor )
{
	stCGSize  size;
	_lpAliceCG->GetCGSizeInfo(&size);	

	long dx, dy;
	switch( m_CGLoadPosFlag ){
		case -1:	// ftHg
			dx = size.x;
			dy = size.y;
			break;
		case  0:	// ΍W
			dx = m_ptCGLoad.x;
			dy = m_ptCGLoad.y;
			break;
		case  1:	// ΍W
			dx = size.x + m_ptCGLoad.x;
			dy = size.y + m_ptCGLoad.y;
			break;
	}

	//
	stGrpRect rect;
	rect.left	= dx;
	rect.top	= dy;
	rect.right	= dx + size.Width;
	rect.bottom = dy + size.Height;

	// WJ
	unsigned short	*lpPic = new unsigned short[size.Height * size.Width];
	unsigned char	*lpAlp = new unsigned char [size.Height * size.Width];
	memset( lpAlp, 0xFF, size.Height * size.Width );
	_lpAliceCG->Expand64K( lpPic, lpAlp );

	//
	// bN
	stGrpScreenDesc Desc;
	stGrpScreenBits Bits;
	m_lpScreen->Lock( &Bits, &Desc );

	// 
	unsigned long *lpDstPos = Bits.lpCBits;
	unsigned long *lpDstWork;
	unsigned char *lpADstPos= Bits.lpABits;
	unsigned char *lpADstWork;

	unsigned short*lpSrcWork = lpPic;
	unsigned char *lpASrcWork= lpAlp;

	long sr, sg, sb, sa;
	long dr, dg, db, da;
	long pitch = Desc.lPitch / 4;
	lpDstPos = lpDstPos + ( dx + dy * pitch );
	lpADstPos= lpADstPos+ ( dx + dy * pitch ); 
	long brightness = m_Brightness >> ALSD_ABTABLE_BITS;

	for( long y = 0; y < size.Height; y++ ){
		lpDstWork = lpDstPos;
		lpADstWork= lpADstPos;
		for( long x = 0; x < size.Width; x++ ){
			sr = m_lp5BitTbl[(*lpSrcWork & 0xF800) >> 11];
			sg = m_lp6BitTbl[(*lpSrcWork & 0x07E0) >>  5];
			sb = m_lp5BitTbl[(*lpSrcWork & 0x001F)      ];

			if( m_Brightness != 255 ){
				sr = m_AlphaBlendTable[brightness][sr];
				sg = m_AlphaBlendTable[brightness][sg];
				sb = m_AlphaBlendTable[brightness][sb];
			}

			if( _SpriteColor != -1 ){
				sa = (*lpASrcWork) >> ALSD_ABTABLE_BITS;
		
				dr = (*lpDstWork & Desc.dgpfPixelFormat.dwRBitMask) >> 16;
				dg = (*lpDstWork & Desc.dgpfPixelFormat.dwGBitMask) >>  8;
				db = (*lpDstWork & Desc.dgpfPixelFormat.dwBBitMask);
				da = ALSD_ABTABLE_DEPTH - 1 - sa;

				sr = m_AlphaBlendTable[sa][sr] + m_AlphaBlendTable[da][dr];
				sg = m_AlphaBlendTable[sa][sg] + m_AlphaBlendTable[da][dg];
				sb = m_AlphaBlendTable[sa][sb] + m_AlphaBlendTable[da][db];
			}
			*lpDstWork = (sr << 16) | (sg << 8) | sb;
			*lpADstWork= *lpASrcWork;
			
			lpDstWork++;
			lpSrcWork++;
			lpADstWork++;
			lpASrcWork++;
		}
		lpDstPos += pitch;
		lpADstPos+= pitch;
	}

	delete[] lpPic;
	delete[] lpAlp;
	//
	m_lpScreen->Unlock();

	OnDraw( &rect );
}



/**********************************************************************
	JR}h
**********************************************************************/
void CGraphicDriver::SetLoadCGOpt( long _x, long _y,
								   long _PosFlag, bool _UseOnce )
{
	m_ptCGLoad.x = _x;
	m_ptCGLoad.y = _y;
	m_CGLoadPosFlag = _PosFlag;
	m_bCGLoadPosUseOnce = _UseOnce;
}


/**********************************************************************
	pbgݒ
**********************************************************************/
void CGraphicDriver::SetPalette( long _index,
								 long _red, long _green, long _blue )
{
	m_Palette.entry[_index].alpha = 0;
	m_Palette.entry[_index].red   = _red;
	m_Palette.entry[_index].green = _green;
	m_Palette.entry[_index].blue  = _blue;
}

/**********************************************************************
	XVXCb`
**********************************************************************/
void CGraphicDriver::SetUpdate( long _sw, bool _update )
{
	if( _sw ){
		m_bUpdate = true;
		// XV
		if( _update )OnDraw( &m_ViewRect );
	}else{
		m_bUpdate = false;
	}
}

/**********************************************************************
	[hTCY
**********************************************************************/
void CGraphicDriver::SetWorldSize(  long _x_size, long _y_size,
									long _color )
{
	m_lpScreen->Create( _x_size, _y_size, 32 );
	ClearAlpha( 0xFF );
	ClearColor( 0x00000000 );
}

/**********************************************************************
	[hTCY̎擾
**********************************************************************/
void CGraphicDriver::GetWorldSize( long *_lpX, long *_lpY, long *_lpC )
{
	stGrpScreenDesc Desc;
	m_lpScreen->Lock( NULL, &Desc );
	m_lpScreen->Unlock();

	*_lpX = Desc.dwWidth;
	*_lpY = Desc.dwHeight;
	*_lpC = Desc.dgpfPixelFormat.dwRGBABitCount;
}

/**********************************************************************
	r[TCY̎擾
**********************************************************************/
void CGraphicDriver::GetViewSize ( long *_lpX, long *_lpY, long *_lpC )
{
	*_lpX = m_ViewRect.right - m_ViewRect.left;
	*_lpY = m_ViewRect.bottom- m_ViewRect.top;
	*_lpC = m_ViewC;
}

/**********************************************************************
	r[̃tF[hAEg
**********************************************************************/
void CGraphicDriver::FadeOutView ( long _loop, long _wf )
{
	// 炩ɂ
	_loop *= 3;
	unsigned long time;
	unsigned long wait = 200 / 3;

	long *rnd = m_lpSD->GetExpDriver()->GetSystemVar();
	CCommandDriver *lpCD = m_lpSD->GetCmdDriver();

	float fade = 0.0f;
	for( long i = 0; i < _loop; i++){
		time = GetTickCount();
		m_lpScreen->SetFade( (long)fade );
		OnDraw( m_ViewRect.left, m_ViewRect.top,
				m_ViewRect.right - m_ViewRect.left,
				m_ViewRect.bottom- m_ViewRect.top );
		if( _wf ){
			*rnd = lpCD->CommandIK6_Sub();
			if( *rnd )break;
		}
		fade += 255.0 / (float)_loop;

		time = GetTickCount() - time;
		if( time < wait )Sleep( wait - time );
	}
	m_lpScreen->SetFade( 255 );
	OnDraw( m_ViewRect.left, m_ViewRect.top,
			m_ViewRect.right - m_ViewRect.left,
			m_ViewRect.bottom- m_ViewRect.top );
}

/**********************************************************************
	r[̃tF[hC
**********************************************************************/
void CGraphicDriver::FadeInView ( long _loop, long _wf )
{
	// 炩ɂ
	_loop *= 3;
	unsigned long time;
	unsigned long wait = 200 / 3;

	long *rnd = m_lpSD->GetExpDriver()->GetSystemVar();
	CCommandDriver *lpCD = m_lpSD->GetCmdDriver();

	float fade = 255.0f;
	for( long i = 0; i < _loop; i++){
		time = GetTickCount();
		m_lpScreen->SetFade( (long)fade );
		OnDraw( m_ViewRect.left, m_ViewRect.top,
				m_ViewRect.right - m_ViewRect.left,
				m_ViewRect.bottom- m_ViewRect.top );
		if( _wf ){
			*rnd = lpCD->CommandIK6_Sub();
			if( *rnd )break;
		}
		fade -= 255.0 / (float)_loop;

		time = GetTickCount() - time;
		if( time < wait )Sleep( wait - time );
	}
	m_lpScreen->SetFade( 0.0 );
	OnDraw( m_ViewRect.left, m_ViewRect.top,
			m_ViewRect.right - m_ViewRect.left,
			m_ViewRect.bottom- m_ViewRect.top );
}

/**********************************************************************
	r[̃zCgAEg
**********************************************************************/
void CGraphicDriver::WhiteOutView ( long _loop, long _wf )
{
	// 炩ɂ
	_loop *= 3;
	unsigned long time;
	unsigned long wait = 200 / 3;

	long *rnd = m_lpSD->GetExpDriver()->GetSystemVar();
	CCommandDriver *lpCD = m_lpSD->GetCmdDriver();

	float fade = 0.0f;
	for( long i = 0; i < _loop; i++){
		time = GetTickCount();
		m_lpScreen->SetFade( (long)fade );
		OnDraw( m_ViewRect.left, m_ViewRect.top,
				m_ViewRect.right - m_ViewRect.left,
				m_ViewRect.bottom- m_ViewRect.top );
		if( _wf ){
			*rnd = lpCD->CommandIK6_Sub();
			if( *rnd )break;
		}
		fade += -255.0 / (float)_loop;
		time = GetTickCount() - time;
		if( time < wait )Sleep( wait - time );
	}
	m_lpScreen->SetFade( -255 );
	OnDraw( m_ViewRect.left, m_ViewRect.top,
			m_ViewRect.right - m_ViewRect.left,
			m_ViewRect.bottom- m_ViewRect.top );
}

/**********************************************************************
	r[̃zCgC
**********************************************************************/
void CGraphicDriver::WhiteInView ( long _loop, long _wf )
{
	// 炩ɂ
	_loop *= 3;
	unsigned long time;
	unsigned long wait = 200 / 3;

	long *rnd = m_lpSD->GetExpDriver()->GetSystemVar();
	CCommandDriver *lpCD = m_lpSD->GetCmdDriver();

	float fade = -255.0f;
	for( long i = 0; i < _loop; i++){
		time = GetTickCount();
		m_lpScreen->SetFade( (long)fade );
		OnDraw( m_ViewRect.left, m_ViewRect.top,
				m_ViewRect.right - m_ViewRect.left,
				m_ViewRect.bottom- m_ViewRect.top );
		if( _wf ){
			*rnd = lpCD->CommandIK6_Sub();
			if( *rnd )break;
		}
		fade -= -255.0 / (float)_loop;
		time = GetTickCount() - time;
		if( time < wait )Sleep( wait - time );
	}
	m_lpScreen->SetFade( 0.0 );
	OnDraw( m_ViewRect.left, m_ViewRect.top,
			m_ViewRect.right - m_ViewRect.left,
			m_ViewRect.bottom- m_ViewRect.top );
}

/**********************************************************************
	ItXN[̈ꕔޔ^A
**********************************************************************/
long CGraphicDriver::SaveRect( stGrpRect *_lpRect )
{
	long w = _lpRect->right - _lpRect->left;
	long h = _lpRect->bottom- _lpRect->top;
	long size = w * h;

	// m
	stSaveRectInfo *lpInfo = new stSaveRectInfo;
	lpInfo->rect = *_lpRect;
	lpInfo->lpPtr = new unsigned long[ size + 7];
	lpInfo->lpBits=(unsigned long *)( ((int)(lpInfo->lpPtr) + 7) & ~7 );
	///////////////////////
	// obNAbv
	// bN
	stGrpScreenBits Bits;
	stGrpScreenDesc Desc;
	m_lpScreen->Lock( &Bits, &Desc );
	// ʒu
	long pitch = Desc.lPitch / 4;
	unsigned long *lpSrcPos = Bits.lpCBits;
	lpSrcPos += ( _lpRect->left + _lpRect->top * pitch );
	unsigned long *lpDstPos = lpInfo->lpBits;
	//
	for( long i = 0; i < h; i++ ){
		memcpy( lpDstPos, lpSrcPos, w * 4 );
		lpSrcPos += pitch; lpDstPos += w;
	}
	m_lpScreen->Unlock();	

	// 󂫂T
	long i;
	for( i = 0; i < m_vcSaveInfo.size(); i++){
		if( m_vcSaveInfo[i] == NULL ){
			m_vcSaveInfo[i] = lpInfo;
			return i;
		}
	}
	// ΂Kɂ
	m_vcSaveInfo.push_back( lpInfo );

	return i;
}

void CGraphicDriver::RestoreRect( long _ID )
{
	if( m_vcSaveInfo[_ID] == NULL )return;

	stSaveRectInfo *lpInfo = m_vcSaveInfo[_ID];
	m_vcSaveInfo[_ID] = NULL;

	// bN
	stGrpScreenBits Bits;
	stGrpScreenDesc Desc;
	m_lpScreen->Lock( &Bits, &Desc );
	// ʒu
	long pitch = Desc.lPitch / 4;
	unsigned long *lpSrcPos = lpInfo->lpBits;
	unsigned long *lpDstPos = Bits.lpCBits;
	lpDstPos += ( lpInfo->rect.left + lpInfo->rect.top * pitch );
	//
	long w = lpInfo->rect.right - lpInfo->rect.left;
	long h = lpInfo->rect.bottom- lpInfo->rect.top;
	for( long i = 0; i < h; i++ ){
		memcpy( lpDstPos, lpSrcPos, w * 4 );
		lpSrcPos += w;
		lpDstPos += pitch;
	}
	m_lpScreen->Unlock();	

	// ̂Ă
	delete[] lpInfo->lpPtr;
	delete lpInfo;

}

void CGraphicDriver::DeleteSaveRect( long _ID )
{
	if( m_vcSaveInfo[_ID] == NULL )return;

	stSaveRectInfo *lpInfo = m_vcSaveInfo[_ID];
	m_vcSaveInfo[_ID] = NULL;

	// ̂Ă
	delete[] lpInfo->lpPtr;
	delete lpInfo;
}

/**********************************************************************
	bf̃[h񐔂擾
**********************************************************************/
void CGraphicDriver::SetCGLoadCount( long *_lpVar )
{
	m_lpCGLoadCount = _lpVar;
}

/**********************************************************************
	̑I܂Ői
**********************************************************************/
void CGraphicDriver::OnJumpMessage( long _bool )
{
	m_lpSD->GetMsgCtrl()->SetAutoNextPage( _bool != 0 );
}


