/*
 * GraphicDriver.h - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#ifndef __GraphicDriver_h__
#define __GraphicDriver_h__
/**********************************************************************
	scenario decoder system graphics driver
**********************************************************************/
#include "ScenarioDecoderSystem.h"
#include "GraphicView.h"
#include "AliceCG.h"
#include "OffScreen.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD_ALCG {
#endif
	class CAliceCG_PMS;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD {
#endif
	class CGraphicView;
	class CGraphicDriver;
	//
#if 0
	const unsigned long ALSD_ABTABLE_DEPTH	= 256; // 8bit
	const unsigned long	ALSD_ABTABLE_BITS	=  8 - 8;
#else
	const unsigned long ALSD_ABTABLE_DEPTH	= 64; // 6bit
	const unsigned long	ALSD_ABTABLE_BITS	=  8 - 6;
#endif
	
	// ItXN[ޔ
	struct stSaveRectInfo {
		stGrpRect rect;
		unsigned long *lpPtr;
		unsigned long *lpBits;
	};
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	typedef std::vector< stSaveRectInfo* > vcSaveInfo;
#else
	typedef vector< stSaveRectInfo* > vcSaveInfo;
#endif

	// Windows DIB
	struct stBitmapInfoHeader {
		unsigned long	biSize;
		long			biWidth;
		long			biHeight;
		unsigned short	biPlanes;
		unsigned short	biBitCount;
		unsigned long	biCompression;
		unsigned long	biSizeImage;
		long			biXPelsPerMeter;
		long			biYPelsPerMeter;
		unsigned long	biClrUsed;
		unsigned long	biClrImportant;
	};
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif

//
// class
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
class ALSD::CGraphicDriver : public ALSD::CGraphicView {
#else
class CGraphicDriver : public CGraphicView {
#endif
public:
	CGraphicDriver( CScenarioDecoder *_lpScenarioDecoder );
	virtual ~CGraphicDriver();
	
	// Interface
	virtual void OnDraw( stGrpRect *_lpUpdateRect ) = 0;
	virtual void SetWindowTitle( const char *_lpTitle ) = 0;
	virtual void SetViewSize( long _start_x, long _start_y, long _size_x, long _size_y ) = 0;
	virtual void Sleep( unsigned long Milliseconds ) = 0;
	virtual void GetTickCount( unsigned long *_Milliseconds ) = 0;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	virtual void GetSystemName( std::string &_str ) = 0;
#else
	virtual void GetSystemName( string &_str ) = 0;
#endif
	virtual void SetMousePos( long _x, long _y ) = 0;
	virtual void PumpMessage( void ){};
	virtual void ExitSystem( void ) = 0;

	// Input driver...?
	bool IsLButtonDown( stGrpPoint *_lpPt );
	bool IsRButtonDown( stGrpPoint *_lpPt );
	unsigned long GetKeyCode( void ){ return m_VKeyCode;};
	void SetDecorateType( long _mode ){ m_DecorateType = _mode;};
	void SetDecorateColor( long _color ){ m_DecorateColor = _color;};
	
	// interface
	virtual void OnJumpMessage	( long _bool );
	virtual void SetWorldSize( long _x_size, long _y_size, long _color );
	void GetViewSize ( long *_lpX, long *_lpY, long *_lpC );
	void GetWorldSize( long *_lpX, long *_lpY, long *_lpC );
	void GetViewRect ( stGrpRect *_lpRect ){ *_lpRect = m_ViewRect;};
	void SetPaletteCtrl( long _pc ){ m_GPC = _pc;};


	virtual void OnDraw( long _x, long _y, long _w, long _h );
	virtual void OnLButtonDown	( stGrpPoint *_lpPoint );
	virtual void OnLButtonUp	( stGrpPoint *_lpPoint );
	virtual void OnRButtonDown	( stGrpPoint *_lpPoint );
	virtual void OnRButtonUp	( stGrpPoint *_lpPoint );
	virtual void OnMouseMove	( stGrpPoint *_lpPoint );
	virtual void OnKeyDown		( unsigned int _KeyCode );
	virtual void OnKeyUp		( unsigned int _KeyCode );

	// return 8, 16
	virtual long CheckPMSFormat( const unsigned char *_lpMem );

	void SetUpdate( long _sw, bool _update = true );
	void SetPalette( long _index, long _red, long _green, long _blue );
	void SetLoadCGOpt( long _x, long _y, long _PosFlag, bool _UseOnce );
	void SetCGLoadBrightness( long _Brightness ){ m_Brightness = _Brightness;};
	void LoadCG( long _Link, long _SpriteColor );
	void LoadCGX(long _Link, long _Shadow );
	void SetCGLoadCount( long *_lpVar );
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	void GetCGSizeInfo( long _Link, ALSD_ALCG::stCGSize *_lpSize );
#else
	void GetCGSizeInfo( long _Link, stCGSize *_lpSize );
#endif
	void LoadDIB	( long _x, long _y, const char *_lpFileName );
	void LoadDIB256	( const unsigned char *_lpMem, unsigned char *_lpBitmap );
	void LoadDIB16M	( const unsigned char *_lpMem, unsigned char *_lpBitmap );

	// OffScreen interface
	COffScreen *GetScreen(void) const { return m_lpScreen;};
	void ClearAlpha ( unsigned char _a8 );
	void ClearColor ( unsigned long _rgb32 );
	void Paint		( long _x, long _y, unsigned char _c );
	
	void FillRect	( long _x, long _y, long _w, long _h, unsigned char _c );
	void StrokeRect	( long _x, long _y, long _w, long _h, unsigned char _c );
	void AddColorRect(long _x, long _y, long _w, long _h, unsigned char _c, long _rate );
	void StrokeLine	( long _x1,long _y1,long _x2,long _y2,unsigned char _c );
	void SetFont	( long _num ){ m_lpScreen->SetFont( _num );};
	void SetFontSize( long _size ){ m_lpScreen->SetFontSize( _size );};
	void SetFontWeight(long _w   ){ m_lpScreen->SetFontWeight( _w ) ;};
	long GetFontSize( void		 ){ return m_lpScreen->GetFontSize();};
	void DrawString	( long _x, long _y, const char *_lpString, unsigned char _c );
	void Reverse	( long _x, long _y, long _w, long _h );
	
	void BltFast	( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy );
	void BltSprite	( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy, unsigned char _c );
	void BltEffect	( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
					  long _ef, long _op, long _wf, long _sc );
	void BltMirror	( stGrpRect *_lpSrcRect, stGrpRect *_lpDstRect, long _mf );
	void BltWithAlpha(long _sx, long _sy, long _sw, long _sh, long _dx, long _dy );
	
	void FadeInView  ( long _loop, long _wf );
	void FadeOutView ( long _loop, long _wf );
	void WhiteInView ( long _loop, long _wf );
	void WhiteOutView( long _loop, long _wf );


#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	void BltCG64K	( ALSD_ALCG::CAliceCG_PMS *_lpAliceCG, long _SpriteColor );
	void BltCGAlpha	( ALSD_ALCG::CAliceCG_PMS *_lpAliceCG );
//	void BltCG256	( ALSD_ALCG::CAliceCG_PMS *_lpAliceCG, long _SpriteColor );
#else
	void BltCG64K	( CAliceCG_PMS *_lpAliceCG, long _SpriteColor );
	void BltCGAlpha	( CAliceCG_PMS *_lpAliceCG );
//	void BltCG256	( CAliceCG_PMS *_lpAliceCG, long _SpriteColor );
#endif

	// 摜̑Ҕ
	long SaveRect	( stGrpRect *_lpRect );
	void RestoreRect( long _ID );
	void DeleteSaveRect( long _ID );

	//
	virtual unsigned long GetTickCount( void ){
		unsigned long ret;
		GetTickCount( &ret );
		return ret;
	}
protected:
	// 
	COffScreen	*m_lpScreen;	// ItXN[
	vcSaveInfo	 m_vcSaveInfo;	// 
	
	// ViewSize
//	long m_ViewX;
//	long m_ViewY;
	long m_ViewC;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	ALSD_ALCG::stALCG_Palette	 m_Palette;
#else
	stALCG_Palette	m_Palette;
#endif
	stGrpRect					 m_ViewRect;
	// ݒ
	long m_DecorateType;
	long m_DecorateColor;

	// e[u
	static	bool			m_bInitialize;
	static	unsigned char	m_AlphaBlendTable[ALSD_ABTABLE_DEPTH][256];
	// 16bit->24bitWJe[u
	static unsigned char	m_lp5BitTbl[32];	// 5bit > 8bit table
	static unsigned char	m_lp6BitTbl[64];	// 6bit > 8bit table
	//
	bool				 m_bUpdate;		// XVtO
	CScenarioDecoder	*m_lpSD;		// 

	// 
	bool		m_bLBDownNow;	// {^Ă
	bool		m_bRBDownNow;
	stGrpPoint	m_ptMousePos;	// }EẌʒu
	// zL[R[h(WIN32݊)
	unsigned int m_VKeyCode;

	// CG[h֘A
	stGrpPoint  m_ptCGLoad;
	long		m_Brightness;	// x
	long		m_CGLoadPosFlag; // OF΍W@PF΍W@|PFftHg
	bool		m_bCGLoadPosUseOnce; // 񂾂H
	long		*m_lpCGLoadCount;
	long		m_GPC;			// pbgWJtO

	//
	void BltFast32(	long _sx, long _sy, long _sw, long _sh,	long _dx, long _dy );
	void BltFast32c(long _sx, long _sy, long _sw, long _sh,	long _dx, long _dy );
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	void BltFast32b(long _sx, long _sy, long _sw, long _sh, ALSD_OFFSCR::stGrpScreenBits *_lpBits, 
															ALSD_OFFSCR::stGrpScreenDesc *_lpDesc );
#else
	void BltFast32b(long _sx, long _sy, long _sw, long _sh, stGrpScreenBits *_lpBits, 
															stGrpScreenDesc *_lpDesc );
#endif
	
	void BltMirror32 ( stGrpRect *_lpSrcRect, stGrpRect *_lpDstRect, long _MirrorFlag );
	void BltMirror32c( stGrpRect *_lpSrcRect, stGrpRect *_lpDstRect, long _MirrorFlag );
	
	void BltSprite32 ( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy, unsigned char _c );
	void BltSprite32c( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy, unsigned char _c );

	void BltWithAlpha32( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy );

	// pCと
	void BltEffect32_01( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
					 	 long _ef, long _op, long _wf, long _sc );
	// pC
	void BltEffect32_02( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
					 	 long _ef, long _op, long _wf, long _sc );
	// PCƂ΂㉺dˍ킹
	void BltEffect32_03( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
					 	 long _ef, long _op, long _wf, long _sc );
	// PCƂ΂Edˍ킹
	void BltEffect32_04( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
					 	 long _ef, long _op, long _wf, long _sc );
	// s͗lۂ
	void BltEffect32_05( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
					 	 long _ef, long _op, long _wf, long _sc );
	//
	void BltEffect32_06( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
					 	 long _ef, long _op, long _wf, long _sc );
	//
	void BltEffect32_07( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
					 	 long _ef, long _op, long _wf, long _sc );
	//
	void BltEffect32_08( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// 킶SSiSij
	void BltEffect32_09( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// ꗎ
	void BltEffect32_11( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// E
	void BltEffect32_12( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// E
	void BltEffect32_13( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// と
	void BltEffect32_14( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// CL
	void BltEffect32_16( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// CL
	void BltEffect32_17( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// cCAO
	void BltEffect32_18( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// CAO
	void BltEffect32_19( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// CAO
	void BltEffect32_20( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// CAO
	void BltEffect32_21( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// 킶QQiQij
	void BltEffect32_22( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// 킶QQiSij
	void BltEffect32_23( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// tF[hC
	void BltEffect32_27( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// zCgC
	void BltEffect32_28( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// tF[hAEg
	void BltEffect32_29( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// zCgAEg
	void BltEffect32_30( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// NXtF[h
	void BltEffect32_31( long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
						 long _ef, long _op, long _wf, long _sc );
	// xw
	void BltEffect32_2000(	long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
							long _ef, long _op, long _wf, long _sc );
	// ߎw
	void BltEffect32_2001(	long _sx, long _sy, long _sw, long _sh, long _dx, long _dy,
							long _ef, long _op, long _wf, long _sc );

private:
};

#endif
