/*
 * GraphicDriver_WIN32.cpp 
 *		- "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 */

#include "stdafx.h"
#include "GraphicDriver_WIN32.h"

#include "OffScreen_WIN32.h"

using namespace ALSD;
using namespace ALSD_WIN32;
/**********************************************************************
	RXgNV
**********************************************************************/
CGraphicDriver_WIN32::CGraphicDriver_WIN32( ALSD::CScenarioDecoder *_lpScenarioDecoder )
: CGraphicDriver( _lpScenarioDecoder )
{
	m_lpScreen = new COffScreen_WIN32();
	m_lpScreen->Create( 640, 480, 32 );
	ClearAlpha( 0xFF );
	ClearColor( 0x00000000 );
}

CGraphicDriver_WIN32::~CGraphicDriver_WIN32()
{
	if( m_lpScreen )delete m_lpScreen;
}

/**********************************************************************
	OnDrawnh
**********************************************************************/
void CGraphicDriver_WIN32::OnDraw( stGrpRect *_lpUpdateRect )
{
	// Abvf[g\ȂItXN[] 
	if( m_bUpdate ){
		//
		m_lpScreen->UpdateView( _lpUpdateRect, (void *)&m_hWnd );
	}
}

/**********************************************************************
	r[TCY
**********************************************************************/
void CGraphicDriver_WIN32::SetViewSize( long _start_x, long _start_y,
									    long _size_x , long _size_y )
{
	//
	m_lpScreen->CreateView( _size_x, _size_y );
	
	//
	m_ViewRect.left = _start_x;
	m_ViewRect.top  = _start_y;
	m_ViewRect.right= _start_x + _size_x;
	m_ViewRect.bottom=_start_y + _size_y;
	m_ViewC = 32;

	// r[̐egbvxWindowƌ߂
	HWND hPWnd = ::GetParent( m_hWnd );

	RECT rect;
	::GetWindowRect( hPWnd, &rect );
	
	rect.right	= rect.left + _size_x;
	rect.bottom	= rect.top  + _size_y;

	::AdjustWindowRectEx( &rect, WS_OVERLAPPEDWINDOW & ~WS_THICKFRAME , TRUE, WS_EX_OVERLAPPEDWINDOW );

	if( rect.left < 0 ){
		rect.right -= rect.left;
		rect.left = 0;
	}
	if( rect.top < 0 ){
		rect.bottom -= rect.top;
		rect.top = 0;
	}

	::MoveWindow( hPWnd, rect.left, rect.top,
				  rect.right - rect.left,
				  rect.bottom - rect.top, TRUE );
}

/**********************************************************************
	Window̃^Cgݒ肷
**********************************************************************/
void CGraphicDriver_WIN32::SetWindowTitle( const char *_lpTitle )
{
	HWND hPWnd = ::GetParent( m_hWnd );
	::SetWindowText( hPWnd, _lpTitle );
}

/**********************************************************************
	}EXJ[\𓮂
**********************************************************************/
void CGraphicDriver_WIN32::SetMousePos( long _x, long _y )
{
	POINT st;
	::GetCursorPos( &st );

	POINT pt;
	pt.x = _x;
	pt.y = _y;
	::ClientToScreen( m_hWnd, &pt );
	
	float ax = (float)(pt.x - st.x) / 10.0f;
	float ay = (float)(pt.y - st.y) / 10.0f;
	float nx = ax;
	float ny = ay;
	for( long i = 0; i < 10; i++){
		Sleep( 10 );
		::SetCursorPos( st.x + nx, st.y + ny );
		nx += ax;
		ny += ay;
	}
}

/**********************************************************************
	bZ[W𗬂
**********************************************************************/
void CGraphicDriver_WIN32::PumpMessage( void )
{
	MSG msg;
	while( ::PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ){
		::TranslateMessage( &msg );
		::DispatchMessage(  &msg );
	}
}

/**********************************************************************
	XbhԎ~߂
**********************************************************************/
void CGraphicDriver_WIN32::Sleep( unsigned long Milliseconds )
{
	static long count = 0;
	if( count > 5 ){
		count = 0;
		PumpMessage();
	}
	count++;
	::Sleep( Milliseconds );
}

/**********************************************************************
	AvP[V̏I
**********************************************************************/
void CGraphicDriver_WIN32::ExitSystem( void )
{
	HWND hWndTop, hWnd = m_hWnd;
	do{
		hWndTop = hWnd;
		hWnd = ::GetParent(hWnd);
	}while (hWnd != NULL);
	::PostMessage( hWndTop, WM_CLOSE, 0, 0);
}
