/*
 * OffScreen_BeOS.cpp
 *		- "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "OffScreen_BeOS.h"

#include <UTF8.h>

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_BeOS;
using namespace ALSD_OFFSCR;
#endif
/**********************************************************************
	construction
**********************************************************************/
COffScreen_BeOS::COffScreen_BeOS()
{
	m_lpBits		= NULL;
	m_lpBitmap		= NULL;
	m_lpView		= NULL;
	m_lpAPtr		= NULL;
	m_lpAlphaBits	= NULL;

	m_lpAlphaBackup	= NULL;
	m_lpABackupPtr	= NULL;
	m_lpColorBackup	= NULL;
	m_lpCBackupPtr	= NULL;

	m_FontSize		= 16;
//	m_lpFont = new BFont( be_fixed_font );
	m_lpFont = new BFont( be_plain_font );
	m_lpFont->SetSize( m_FontSize );
//	m_lpFont->SetFlags(B_DISABLE_ANTIALIASING);
	m_lpFont->SetFlags(B_FORCE_ANTIALIASING);
}

COffScreen_BeOS::~COffScreen_BeOS()
{
	Destroy();
	if( m_lpFont ){
		delete m_lpFont;
		m_lpFont = NULL;
	}
}

void COffScreen_BeOS::Create( long _w, long _h, long _bpp )
{
	Destroy();
	
	BRect bitmapRect( 0, 0, _w + 1, _h + 1 );
	
	// 
	m_lpView = new BView( bitmapRect, "", B_FOLLOW_NONE, 0 );
	// 
#if __POWERPC__
	m_lpBitmap = new BBitmap( bitmapRect, B_RGB32_BIG, true );
#else
	m_lpBitmap = new BBitmap( bitmapRect, B_RGB32, true );
#endif
	m_lpBitmap->AddChild( m_lpView );
	
	m_lpBits = m_lpBitmap->Bits();

	// 
	long memsize = _w * _h;
	m_lpAPtr = new unsigned char[memsize + 7];
	m_lpAlphaBits= (void *)( ((int)m_lpAPtr + 7) & ~7);

	// 
	m_lpABackupPtr = new unsigned char[memsize + 7];
	m_lpAlphaBackup= (void *)( ((int)m_lpABackupPtr + 7) & ~7 );
	m_lpCBackupPtr = new unsigned long[memsize + 7];
	m_lpColorBackup= (void *)( ((int)m_lpCBackupPtr + 7) & ~7 );

	// 
	m_Desc.dwSize	= sizeof( m_Desc );
	m_Desc.dwWidth	= (unsigned long)_w;
	m_Desc.dwHeight= (unsigned long)_h;
	m_Desc.lPitch	= m_lpBitmap->BytesPerRow();
	m_Desc.dgpfPixelFormat.dwSize = sizeof( stGrpPixelFormat );
	m_Desc.dgpfPixelFormat.dwRGBABitCount = _bpp;
	if( _bpp == 32 ){
		m_Desc.dgpfPixelFormat.dwABitMask = 0;
		m_Desc.dgpfPixelFormat.dwRBitMask = 0x00FF0000;
		m_Desc.dgpfPixelFormat.dwGBitMask = 0x0000FF00;
		m_Desc.dgpfPixelFormat.dwBBitMask = 0x000000FF;
	}

	// 
	if( m_lpBitmap->Lock() ){
		m_lpView->SetFont( m_lpFont );
		m_lpBitmap->Unlock();
	}
}

/**********************************************************************
**********************************************************************/
void COffScreen_BeOS::Destroy( void )
{
	if( m_lpABackupPtr ){
		delete[] m_lpABackupPtr;
		m_lpABackupPtr = NULL;
		m_lpAlphaBackup= NULL;
	}
	if( m_lpCBackupPtr ){
		delete[] m_lpCBackupPtr;
		m_lpCBackupPtr = NULL;
		m_lpColorBackup= NULL;
	}

	if( m_lpBitmap ){
		delete m_lpBitmap;
		m_lpBits	= NULL;
		m_lpBitmap	= NULL;
		m_lpView	= NULL;
	}
	if( m_lpAPtr ){
		delete[] m_lpAPtr;
		m_lpAPtr		= NULL;
		m_lpAlphaBits	= NULL;
	}

}

/**********************************************************************
	Fade view
**********************************************************************/
void COffScreen_BeOS::CreateView( long _w, long _h )
{
}

void COffScreen_BeOS::DestroyView( void )
{
}

/**********************************************************************
	Window
**********************************************************************/
void COffScreen_BeOS::UpdateView( stGrpRect *_lpRect, void *_lpInfo )
{
	if( m_lpBitmap ){
		BRect rect( _lpRect->left , _lpRect->top,
					_lpRect->right - 1, _lpRect->bottom - 1 ); 
		BView *lpView = (BView *)_lpInfo;
		if( m_lpBitmap->Lock() ){
			lpView->DrawBitmap( m_lpBitmap, rect, rect );
			m_lpBitmap->Unlock();
		}


	}	
}

/**********************************************************************
	Lock
**********************************************************************/
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
bool COffScreen_BeOS::Lock(	ALSD_OFFSCR::stGrpScreenBits *_lpBits,
							ALSD_OFFSCR::stGrpScreenDesc *_lpDesc )
#else
bool COffScreen_BeOS::Lock(	stGrpScreenBits *_lpBits,
							stGrpScreenDesc *_lpDesc )
#endif
{
	if( MyLock() ){
		if( _lpBits ){
			_lpBits->lpCBits = (unsigned long *)m_lpBits;
			_lpBits->lpABits = (unsigned char *)m_lpAlphaBits;
			_lpBits->lpBackupCBits = (unsigned long *)m_lpColorBackup;
			_lpBits->lpBackupABits = (unsigned char *)m_lpAlphaBackup;
		}
		if( _lpDesc )*_lpDesc = m_Desc;
		return true;
	}
	return false;
}

bool COffScreen_BeOS::Unlock(void)
{
	return MyUnlock();
}

bool COffScreen_BeOS::MyLock( void )
{
	return m_lpBitmap->Lock();
}

bool COffScreen_BeOS::MyUnlock( void )
{
	m_lpView->Sync();
	m_lpBitmap->Unlock();
	return true;
}

/**********************************************************************
	FillRect
**********************************************************************/
void COffScreen_BeOS::FillRect( long _x, long _y,
								 long _w, long _h,
								 unsigned long _rgb32)
								 
{
	BRect rect( (float)_x, (float)_y,
				(float)_x + _w - 1, (float)_y + _h - 1 );
	if( m_lpBitmap->Lock() ){
		rgb_color oldColor = m_lpView->HighColor();
		m_lpView->SetHighColor( ( _rgb32 >> 16 ) & 0xFF,
								( _rgb32 >>  8 ) & 0xFF,
								( _rgb32       ) & 0xFF );
		m_lpView->FillRect( rect );
		m_lpView->SetHighColor( oldColor );
		m_lpView->Sync();
		m_lpBitmap->Unlock();
	}
}

/**********************************************************************
	StrokeRect
**********************************************************************/
void COffScreen_BeOS::StrokeRect( long _x, long _y,
								   long _w, long _h,
								   unsigned long _rgb32)
{
	BRect rect( (float)_x, (float)_y,
				(float)_x + _w - 1, (float)_y + _h - 1 );
	if( m_lpBitmap->Lock() ){
		rgb_color oldColor = m_lpView->HighColor();
		m_lpView->SetHighColor( ( _rgb32 >> 16 ) & 0xFF,
								( _rgb32 >>  8 ) & 0xFF,
								( _rgb32       ) & 0xFF );
		m_lpView->StrokeRect( rect );
		m_lpView->SetHighColor( oldColor );
		m_lpView->Sync();
		m_lpBitmap->Unlock();
	}
}

/**********************************************************************
	StrokeLine
**********************************************************************/
void COffScreen_BeOS::StrokeLine( long _x1, long _y1,
							 long _x2, long _y2,
							 unsigned long _rgb32 )
{
	if( m_lpBitmap->Lock() ){
		rgb_color oldColor = m_lpView->HighColor();
		m_lpView->SetHighColor( ( _rgb32 >> 16 ) & 0xFF,
								( _rgb32 >>  8 ) & 0xFF,
								( _rgb32       ) & 0xFF );
		m_lpView->StrokeLine( BPoint( (float)_x1, (float)_y1 ),
							  BPoint( (float)_x2, (float)_y2 ) );
		m_lpView->SetHighColor( oldColor );
		m_lpView->Sync();
		m_lpBitmap->Unlock();
	}
}

/**********************************************************************
	SetFontSize
**********************************************************************/
void COffScreen_BeOS::SetFontSize( long _size )
{
	if( m_FontSize == _size )return;

	m_FontSize = _size;
	if( m_lpBitmap->Lock() ){
		m_lpView->SetFontSize( _size );
		m_lpBitmap->Unlock();
	}
}

/**********************************************************************
	DrawString
**********************************************************************/
void COffScreen_BeOS::DrawString( long _x, long _y,
								  const char *_lpString,
								  unsigned long _rgb32 )
{
	// SJIS->UTF8
	char *lpStringUTF8 = new char[(strlen( _lpString ) + 1) * 3];
	int32 srcLen = strlen( _lpString ) + 1;
	int32 dstLen = (strlen( _lpString ) + 1) * 3;
	int32 state  = 0;
	::convert_to_utf8( 	 B_SJIS_CONVERSION,_lpString,&srcLen,
						lpStringUTF8,&dstLen, &state );
	
	//
	if( m_lpBitmap->Lock() ){
		if( m_lpFont->Flags() == B_FORCE_ANTIALIASING ){
			rgb_color color = m_lpView->LowColor();
			drawing_mode mode = m_lpView->DrawingMode();
			rgb_color rgb;
			rgb.red = (_rgb32 & 0x00FF0000) >> 16;
			rgb.green=(_rgb32 & 0x0000FF00) >> 8;
			rgb.blue =(_rgb32 & 0x000000FF);

			m_lpView->SetHighColor( rgb );
			m_lpView->SetLowColor( B_TRANSPARENT_32_BIT );
			m_lpView->SetDrawingMode( B_OP_OVER );
			m_lpView->DrawString( lpStringUTF8, BPoint(_x,_y+m_FontSize) );

			m_lpView->SetLowColor( color );
			m_lpView->SetDrawingMode( mode );
		}else{
			rgb_color color = m_lpView->ViewColor();
			rgb_color rgb;
			rgb.red = (_rgb32 & 0x00FF0000) >> 16;
			rgb.green=(_rgb32 & 0x0000FF00) >> 8;
			rgb.blue =(_rgb32 & 0x000000FF);

			m_lpView->SetHighColor( rgb );
			m_lpView->SetViewColor( B_TRANSPARENT_32_BIT );
			m_lpView->DrawString( lpStringUTF8, BPoint(_x,_y+m_FontSize) );
			m_lpView->SetViewColor( color );
		}
		m_lpView->Sync();
		m_lpBitmap->Unlock();
	}
}
