/*
 * OffScreen_WIN32.cpp 
 *		- "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 */

#include "stdafx.h"
#include "Offscreen_WIN32.h"

using namespace ALSD;
using namespace ALSD_OFFSCR;
using namespace ALSD_WIN32;
/**********************************************************************
	RXgNV
**********************************************************************/
COffScreen_WIN32::COffScreen_WIN32()
{
	m_lpBits		= NULL;
	m_lpAPtr		= NULL;
	m_lpAlphaBits	= NULL;
	m_lLock			= 0;
	m_lViewLock		= 0;
	m_hBitmap		= NULL;
	m_hDC			= NULL;
	m_hOldBitmap	= NULL;
	m_hNullBrush	= (HBRUSH)::GetStockObject( NULL_BRUSH );

	m_lpViewBits	= NULL;
	m_hViewDC		= NULL;
	m_hViewOldBitmap= NULL;
	m_hViewBitmap	= NULL;

	m_lpAlphaBackup	= NULL;
	m_lpABackupPtr	= NULL;
	m_lpColorBackup	= NULL;
	m_lpCBackupPtr	= NULL;

	m_hOldFont		= NULL;
	m_hFont			= NULL;
	m_FontNo		= 0;
	m_FontSize		= 0;
	m_FontWeight	= 6;
	SetFontSize( 16 );
}

COffScreen_WIN32::~COffScreen_WIN32()
{
	Destroy();
	DestroyView();
}

void COffScreen_WIN32::Create( long _w, long _h, long _bpp )
{
	Destroy();

	// rbg}bv쐬
	BITMAPINFO bmiScr;
	// DIBSection̍쐬
	bmiScr.bmiHeader.biSize			= sizeof(BITMAPINFOHEADER);
	bmiScr.bmiHeader.biWidth		= _w;
	bmiScr.bmiHeader.biHeight		= -_h;
	bmiScr.bmiHeader.biPlanes		= 1;
	bmiScr.bmiHeader.biBitCount		= (unsigned short)_bpp;
	bmiScr.bmiHeader.biCompression	= BI_RGB;
	m_hBitmap = CreateDIBSection( NULL, &bmiScr, DIB_RGB_COLORS, &m_lpBits, NULL, 0 );
		
	// cb쐬
	m_hDC = ::CreateCompatibleDC( NULL );
	m_hOldBitmap = (HBITMAP)::SelectObject( m_hDC, m_hBitmap );
	::SetBkMode( m_hDC, TRANSPARENT );

	// `l̐
	long memsize = _w * _h;
	m_lpAPtr = new unsigned char[memsize + 7];
	m_lpAlphaBits= (void *)( ((int)m_lpAPtr + 7) & ~7 );

	// obNAbv̐
	m_lpABackupPtr = new unsigned char[memsize + 7];
	m_lpAlphaBackup= (void *)( ((int)m_lpABackupPtr + 7) & ~7 );
	m_lpCBackupPtr = new unsigned long[memsize + 7];
	m_lpColorBackup= (void *)( ((int)m_lpCBackupPtr + 7) & ~7 );


	// SDESC̐ݒ
	m_Desc.dwSize	= sizeof( m_Desc );
	m_Desc.dwWidth	= (unsigned long)_w;
	m_Desc.dwHeight= (unsigned long)_h;
	m_Desc.lPitch	= WIDTHBYTES(_w * _bpp);
	m_Desc.dgpfPixelFormat.dwSize = sizeof( stGrpPixelFormat );
	m_Desc.dgpfPixelFormat.dwRGBABitCount = _bpp;
	if( _bpp == 32 ){
		m_Desc.dgpfPixelFormat.dwABitMask = 0;
		m_Desc.dgpfPixelFormat.dwRBitMask = 0x00FF0000;
		m_Desc.dgpfPixelFormat.dwGBitMask = 0x0000FF00;
		m_Desc.dgpfPixelFormat.dwBBitMask = 0x000000FF;
	}

	long FontSize = m_FontSize;
	m_FontSize = 0;
	SetFontSize( FontSize );
}

void COffScreen_WIN32::Destroy( void )
{
	if( m_lpCBackupPtr ){
		delete[] m_lpCBackupPtr;
	}
	if( m_lpABackupPtr ){
		delete[] m_lpABackupPtr;
	}
	if( m_lpAPtr ){
		delete[] m_lpAPtr;
	}
	if( m_hBitmap ){
		::SelectObject( m_hDC, m_hOldBitmap );
		::DeleteObject( m_hBitmap );
	}
	if( m_hFont ){
		::SelectObject( m_hDC, m_hFont );
		::DeleteObject( m_hFont );
	}
	if( m_hDC ){
		::DeleteDC( m_hDC );
	}

	m_lpBits		= NULL;
	m_lpAPtr		= NULL;
	m_lpAlphaBits	= NULL;
	m_lLock			= 0;
	m_hBitmap		= NULL;
	m_hDC			= NULL;
	m_hOldBitmap	= NULL;

	m_lpAlphaBackup	= NULL;
	m_lpABackupPtr	= NULL;
	m_lpColorBackup	= NULL;
	m_lpCBackupPtr	= NULL;

	m_hOldFont		= NULL;
	m_hFont			= NULL;
//	m_FontSize		= 0;

}

/**********************************************************************
	CreateView
**********************************************************************/
void COffScreen_WIN32::CreateView( long _w, long _h )
{
	DestroyView();
	//
	m_ViewX = _w;
	m_ViewY = _h;

	// rbg}bv쐬
	BITMAPINFO bmiScr;
	// DIBSection̍쐬
	bmiScr.bmiHeader.biSize			= sizeof(BITMAPINFOHEADER);
	bmiScr.bmiHeader.biWidth		= _w;
	bmiScr.bmiHeader.biHeight		= -_h;
	bmiScr.bmiHeader.biPlanes		= 1;
	bmiScr.bmiHeader.biBitCount		= 32;
	bmiScr.bmiHeader.biCompression	= BI_RGB;
	m_hViewBitmap = CreateDIBSection( NULL, &bmiScr, DIB_RGB_COLORS, &m_lpViewBits, NULL, 0 );
		
	// cb쐬
	m_hViewDC = ::CreateCompatibleDC( NULL );
	m_hViewOldBitmap = (HBITMAP)::SelectObject( m_hViewDC, m_hViewBitmap );
}

void COffScreen_WIN32::DestroyView( void )
{
	if( m_hViewBitmap ){
		::SelectObject( m_hViewDC, m_hViewOldBitmap );
		::DeleteObject( m_hViewBitmap );
	}
	if( m_hViewDC ){
		::DeleteDC( m_hViewDC );
	}

	m_lpViewBits	= NULL;
	m_hViewDC		= NULL;
	m_hViewOldBitmap= NULL;
	m_hViewBitmap	= NULL;
}


/**********************************************************************
	Window֓]
**********************************************************************/
void COffScreen_WIN32::UpdateView( stGrpRect *_lpRect, void *_lpInfo )
{
	HWND hWnd = *(HWND *)_lpInfo;
	
	while( ::InterlockedExchange( &m_lViewLock, 1 ) )::Sleep( 20 );

	// foCXReLXg擾
	HDC hDC;
	hDC = ::GetDC( hWnd );

	if( m_Fade ){
		::BitBlt( m_hViewDC, _lpRect->left,
							 _lpRect->top,
							 _lpRect->right - _lpRect->left,
							 _lpRect->bottom- _lpRect->top,
			 				 m_hDC, _lpRect->left, _lpRect->top, SRCCOPY );	

		// tF[h
		DrawFade( m_lpViewBits, _lpRect );
		::BitBlt( hDC,	_lpRect->left,
						_lpRect->top,
						_lpRect->right - _lpRect->left,
						_lpRect->bottom- _lpRect->top,
						m_hViewDC, _lpRect->left, _lpRect->top, SRCCOPY );	
	}else{
		::BitBlt(	hDC,
						_lpRect->left,
						_lpRect->top,
						_lpRect->right - _lpRect->left,
						_lpRect->bottom- _lpRect->top,
		 				 m_hDC, _lpRect->left, _lpRect->top, SRCCOPY );	
	}
	// n
	::ReleaseDC( hWnd, hDC );

	// 
	::InterlockedExchange( &m_lViewLock, 0 );
}

bool COffScreen_WIN32::Lock( ALSD_OFFSCR::stGrpScreenBits *_lpBits,
							 ALSD_OFFSCR::stGrpScreenDesc *_lpDesc )
{
	if( MyLock() ){
		if( _lpBits ){
			_lpBits->lpCBits = (unsigned long *)m_lpBits;
			_lpBits->lpABits = (unsigned char *)m_lpAlphaBits;
			_lpBits->lpBackupCBits = (unsigned long *)m_lpColorBackup;
			_lpBits->lpBackupABits = (unsigned char *)m_lpAlphaBackup;
		}
		if( _lpDesc )*_lpDesc = m_Desc;
		return true;
	}
	return false;
}


bool COffScreen_WIN32::Unlock(void)
{
	return MyUnlock();
}

bool COffScreen_WIN32::MyLock( void )
{
	long old = ::InterlockedExchange( &m_lLock, 1 );
	if( old ){
		return false;
	}
	return true;
}

bool COffScreen_WIN32::MyUnlock( void )
{
	::InterlockedExchange( &m_lLock, 0 );

	return true;
}

/**********************************************************************
	`R}h
**********************************************************************/
void COffScreen_WIN32::FillRect( long _x, long _y,
								 long _w, long _h,
								 unsigned long _rgb32)
{
	DWORD winRGB;
	winRGB = RGB( _rgb32 >> 16 & 0xFF,
				  _rgb32 >>  8 & 0xFF,
				  _rgb32	   & 0xFF );

	RECT rect;
	rect.left	= _x;
	rect.top	= _y;
	rect.right	= _x + _w;
	rect.bottom	= _y + _h;
	::SetBkColor(m_hDC, winRGB );
	::ExtTextOut(m_hDC, 0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL);
}

void COffScreen_WIN32::StrokeRect( long _x, long _y,
								   long _w, long _h,
								   unsigned long _rgb32)
{
	DWORD winRGB;
	winRGB = RGB( _rgb32 >> 16 & 0xFF,
				  _rgb32 >>  8 & 0xFF,
				  _rgb32	   & 0xFF );

	HPEN hOldPen;
	HPEN hPen = ::CreatePen( PS_SOLID, 0, winRGB );
	hOldPen = (HPEN)::SelectObject( m_hDC, hPen );
	
	HBRUSH hOldBrush = (HBRUSH)::SelectObject( m_hDC, m_hNullBrush );

	::Rectangle( m_hDC, _x, _y, _x + _w, _y + _h );
	
	//
	::SelectObject( m_hDC, hOldPen );
	::SelectObject( m_hDC, hOldBrush );
	::DeleteObject( hPen );
}

void COffScreen_WIN32::StrokeLine( long _x1, long _y1,
							 long _x2, long _y2,
							 unsigned long _rgb32 )
{
	DWORD winRGB;
	winRGB = RGB( _rgb32 >> 16 & 0xFF,
				  _rgb32 >>  8 & 0xFF,
				  _rgb32	   & 0xFF );

	//
	HPEN hOldPen;
	HPEN hPen = ::CreatePen( PS_SOLID, 0, winRGB );
	hOldPen = (HPEN)::SelectObject( m_hDC, hPen );
	//
	::MoveToEx( m_hDC, _x1, _y1, NULL );
	::LineTo  ( m_hDC, _x2, _y2 );
	//
	::SelectObject( m_hDC, hOldPen );
	::DeleteObject( hPen );
}

/**********************************************************************
	tHg̐ݒ
**********************************************************************/
void COffScreen_WIN32::SetFontSize( long _size )
{
	if( m_FontSize == _size ){
		return;
	}
	
	m_FontSize = _size;
	if( m_hFont ){
		if( m_hOldFont )::SelectObject( m_hDC, m_hOldFont );
		::DeleteObject( m_hFont );
		m_hFont = 0;
	}
	m_hFont = CreateFont( _size );

	m_hOldFont = (HFONT)::SelectObject( m_hDC, m_hFont );

}

void COffScreen_WIN32::SetFont( long _num )
{
	if( m_FontNo == _num ){
		return;
	}
	m_FontNo = _num;
	if( m_hFont ){
		if( m_hOldFont )::SelectObject( m_hDC, m_hOldFont );
		::DeleteObject( m_hFont );
		m_hFont = 0;
	}
	m_hFont = CreateFont( m_FontSize );

	m_hOldFont = (HFONT)::SelectObject( m_hDC, m_hFont );

}

void COffScreen_WIN32::SetFontWeight( long _w )
{
	if( m_FontWeight == _w ){
		return;
	}
	m_FontWeight = _w;
	if( m_hFont ){
		if( m_hOldFont )::SelectObject( m_hDC, m_hOldFont );
		::DeleteObject( m_hFont );
		m_hFont = 0;
	}
	m_hFont = CreateFont( m_FontSize );

	m_hOldFont = (HFONT)::SelectObject( m_hDC, m_hFont );

}

HFONT COffScreen_WIN32::CreateFont( long _size )
{
	HFONT hFont;

	// tHg쐬
	LOGFONT logfont;
	logfont.lfHeight		= _size;
	logfont.lfWidth			= 0;
	logfont.lfEscapement	= 0;
	logfont.lfOrientation	= 0;
	logfont.lfWeight		= FW_NORMAL;
	logfont.lfItalic		= 0;
	logfont.lfUnderline		= 0;
	logfont.lfStrikeOut		= 0;
	logfont.lfCharSet		= DEFAULT_CHARSET;
	logfont.lfOutPrecision	= OUT_CHARACTER_PRECIS;
	logfont.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
	logfont.lfQuality		= PROOF_QUALITY;DRAFT_QUALITY;
	logfont.lfPitchAndFamily= FIXED_PITCH | FF_MODERN;
	if( m_FontNo == 0 ){
		::lstrcpy(logfont.lfFaceName, "lr SVbN");
	}else{
		::lstrcpy(logfont.lfFaceName, "lr ");
	}

	hFont = ::CreateFontIndirect( &logfont );
		
	//
	return hFont;
}

/**********************************************************************
	o
*********************************************************************/
void COffScreen_WIN32::DrawString( long _x, long _y,
								   const char *_lpString,
								   unsigned long _rgb32 )
{
	DWORD winRGB;
	winRGB = RGB( _rgb32 >> 16 & 0xFF,
				  _rgb32 >>  8 & 0xFF,
				  _rgb32	   & 0xFF );

	::SetTextColor( m_hDC, winRGB ); 
	::TextOut( m_hDC, _x, _y, _lpString, strlen( _lpString ) );
#ifdef _DEBUG
	::OutputDebugString( _lpString );
	::OutputDebugString( "\n" );
#endif
}

