/*
 * ScenarioDecoder.cpp
 *		- "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "ScenarioDecoder.h"

#include "GraphicDriver.h"
#include "CommandDriver.h"
#include "ExpressionDriver.h"
#include "ScenarioParser.h"
#include "EALSD.h"
#include "FileCtrl.h"
#include "SelectionCtrl.h"
#include "MessageCtrl.h"

//#define _BEOS

#if		defined _DIRECTX
#elif	_MSC_VER > 1000
	#include "GraphicDriver_WIN32.h"
	#include "CDPlayer_WIN32.h"
	using namespace ALSD_WIN32;
#else
	#include "GraphicDriver_BeOS.h"
	#include "CDPlayer_BeOS.h"
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	using namespace ALSD_BeOS;
#endif
#endif

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
#endif

/**********************************************************************
	RXgNV
**********************************************************************/
CScenarioDecoder::CScenarioDecoder( const char *_lpFileList )
{
	m_ScenarioVersion	= 0;
	m_bExit				= false;

	m_lpFileCtrl	= new CFileCtrl( _lpFileList, this );
	m_lpSelCtrl		= new CSelectionCtrl( this );
	m_lpMsgCtrl		= new CMessageCtrl( this );
	m_lpParser		= new CScenarioParser( this );
	m_lpCmdDriver	= new CCommandDriver( this );
	m_lpExpDriver	= new CExpressionDriver( this );

	// @ˑ
#if		defined _DIRECTX
#elif	_MSC_VER > 1000
	m_lpGrpDriver	= new CGraphicDriver_WIN32( this );
	m_lpCDPlayer	= new CCDPlayer_WIN32;
#else
	m_lpGrpDriver	= new CGraphicDriver_BeOS( this );
	m_lpCDPlayer	= new CCDPlayer_BeOS;
#endif

	// ㏈
	m_lpCmdDriver->Initialize();
}

CScenarioDecoder::~CScenarioDecoder()
{
	if( m_lpGrpDriver )delete m_lpGrpDriver;
	if( m_lpExpDriver )delete m_lpExpDriver;
	if( m_lpCmdDriver )delete m_lpCmdDriver;
	if( m_lpParser	)delete m_lpParser;
	if( m_lpSelCtrl )delete m_lpSelCtrl;	
	if( m_lpMsgCtrl	)delete m_lpMsgCtrl;
	if( m_lpFileCtrl )delete m_lpFileCtrl;
	if( m_lpCDPlayer )delete m_lpCDPlayer;
}

/**********************************************************************
	C[v
**********************************************************************/
long CScenarioDecoder::Run( void )
{
	try {
		//
		for(;;){
			m_lpGrpDriver->PumpMessage();
			if( m_bExit )throw EALSD_Exit();
			m_lpParser->CheckCommand();
		}
	}
	catch( EALSD_Exit &e ){
		e.Message();
		// I
		m_lpGrpDriver->ExitSystem();
		return ALSD_OK;
	}
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	catch( err::Exception &e ){
#else
	catch( Exception &e ){
#endif
		// G[O
		e.Message();
	}
#ifdef NDEBUG
	catch( ... ){
		return ALSD_ERR_GENERIC;
	}
#endif	
	//
	return ALSD_OK;
}

