/*
 * ScenarioDriver.h - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#ifndef __ScenarioDriver_h__
#define __ScenarioDriver_h__
/**********************************************************************
	Scenario driver class
	
**********************************************************************/
#include "ScenarioDecoderSystem.h"
#include <vector>

//
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD {
#endif
	class CScenarioDriver;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif
// 
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD_SDRV {
#endif
	// stack type
	const long SDRV_STACKTYPE_NEARJAMP	= 1;
	const long SDRV_STACKTYPE_FARJAMP	= 2;
	const long SDRV_STACKTYPE_VARIABLE	= 3;
	// stack class
	class stack1k;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif

//
// class
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
class ALSD::CScenarioDriver {
#else
class CScenarioDriver {
#endif
public:
	CScenarioDriver( CScenarioDecoder *_lpScenarioDecoder );
	virtual ~CScenarioDriver();
	
	// Branch control
	void NearJamp( long _Address );
	void FarJamp ( long _Page );
	void FarJamp ( long _Page, long _Address );
	void NearCall( long _Address );
	void FarCall ( long _Page );
	void FarCall ( long _Page, long _Address );
	void NearRet ( void );
	void FarRet  ( void );
	
	// Stack control
	void PushVar( const long *_lpMem, long _Count );
	void PopVar (       long *_lpMem, long _Count );

	size_t SaveStack( FILE *fp );
	size_t LoadStack( FILE *fp );
	
	// Scenario control
	long GetAddress	( void )const { return m_Address;};
	long GetPage	( void )const { return m_Page;};
	
	long GetC		( void ){ return (long)m_lpMem[m_Address++];};
	long GetRC		( void )const { return (long)m_lpMem[m_Address];};
	long GetW		( void );
	long GetRW		( void )const ;
	long GetRW		( long _Address )const ;
	long GetDW		( void );

protected:
	//
	CScenarioDecoder	*m_lpSD;	// t@CRg[
private:
	//
	unsigned char	*m_lpMem;		// ViIf[^
	long			 m_Page;		// y[W
	long			 m_Address;		// AhX
	long	 m_lpStackBuff[1024];
	long	*m_lpStack;

	CFileData	*m_lpFileData;	// t@C
	
	// Stack
	void _Push( long _Type, long *_lpMem, long _Size );
	void _Pop( long *_lpType, long **_lplpMem, long *_lpSize );
};

#endif
