/*
 * ScenarioParser.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "ScenarioParser.h"

#include "ScenarioDecoder.h"
#include "CommandDriver.h"
#include "ExpressionDriver.h"
#include "SelectionCtrl.h"
#include "MessageCtrl.h"

#include "Kanakan.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace std;
using namespace kanakan;
#endif
/**********************************************************************
	RXgNV
**********************************************************************/
CScenarioParser::CScenarioParser( CScenarioDecoder *_lpScenarioDecoder )
: CScenarioDriver( _lpScenarioDecoder )
{
}

CScenarioParser::~CScenarioParser()
{
}

/**********************************************************************
	ϐ
**********************************************************************/
void CScenarioParser::SetVariable(void)
{
	CExpressionDriver *lpED = m_lpSD->GetExpDriver();

	long *lpVar = lpED->GetVariable();
	*lpVar		= lpED->GetExpValue();
}

/**********************************************************************
	for < Jn
**********************************************************************/
void CScenarioParser::ForStart( void )
{
	CExpressionDriver *lpED = m_lpSD->GetExpDriver();

	long First = GetC();
	if (First == 0) { 
		GetC();
		GetC();
	} else if (First != 1) {
		throw EALSD_Msg( "error:for loop start\n" );
	}
	
	long	 EndJampAdr = GetDW();
	long	*lpVar		= lpED->GetVariableNo();
	long	 Limit		= lpED->GetExpValue();
	long	 Direction	= lpED->GetExpValue(); // ++ or --
	long	 Step		= lpED->GetExpValue();
	
	if( Direction == 1){
		// ++
		if( First == 1 ){
			*lpVar += Step;
		}
		if( *lpVar > Limit ){
			if( *lpVar > 65535 )*lpVar = 65535;
			NearJamp( EndJampAdr );
			return;
		}
	}else if( Direction == 0 ){
		// --
		if( First == 1 ){
			*lpVar -= Step;
		}
		if( *lpVar < Limit ){
			if( *lpVar < 0 )*lpVar = 0;
			NearJamp( EndJampAdr );
			return;
		}
	}else{
		throw EALSD_Msg( "error:unknown for loop direction(%d)\n", Direction);
	}

	return;
}

/**********************************************************************
	f[^e[u̐ݒ
**********************************************************************/
void CScenarioParser::GetDataTable( void )
{
	long index;
	long save;

	index  = GetDW();
	index += (m_lpSD->GetExpDriver()->GetExpValue() - 1) * 4;
	save   = GetAddress();
	NearJamp( index );
	m_lpSD->GetCmdDriver()->SetDataTableAddress( GetDW() );
	NearJamp( save );
}

/**********************************************************************
	R}h`FbN
**********************************************************************/
long CScenarioParser::CheckCommand( void )
{
	CExpressionDriver	*lpED = m_lpSD->GetExpDriver();
	CSelectionCtrl		*lpSC;	
	long Cmd = CheckMessage();
	long Page;
	long Index;
	long bRet;

	//
	if( Cmd >= 'A' && Cmd <= 'Z' ){
		// 
		m_lpSD->GetCmdDriver()->ExecCommandA2Z( Cmd );
	}else{
		//
		switch( Cmd ){
			case '!': // ϐ
				SetVariable();
				TRACE("!\n");
				break;
			case '#': // f[^e[uAhXw
				GetDataTable();
				TRACE( "#\n" );
				break;
			case '$': // I
				lpSC = m_lpSD->GetSelCtrl();
				if( lpSC->IsSelecting() ){
					// II
					lpSC->CloseChoice();
					lpSC->SetSelecting( false );
				}else{
					// IJn
					lpSC->OpenChoice( GetDW() ); 
					lpSC->SetSelecting( true );
				}
				break;
			case ']': // I[h
				lpSC = m_lpSD->GetSelCtrl();
				Index = lpSC->Select();
				if( Index != -1 ){
					NearJamp( Index );
				}
				break;
			case '%': // y[WR[
				Page = lpED->GetExpValue();
				if( Page ){
					TRACE("%FarCall\n" );
					FarCall( Page );
				}else{
					TRACE("%Far Return\n" );
					FarRet();
				}
				break;
			case '&': // y[WWv
				FarCall( lpED->GetExpValue() );
				TRACE( "&FarCall\n" );
				break;
			case '<': // For loop start
				ForStart();
			//	TRACE( "<--- for start\n" );
				break;
			case '>': // For loop end == Label jamp
				NearJamp( GetDW() );
			//	TRACE("     for end --->\n" );
				break;
			case '@': // xWv
				NearJamp( GetDW() );
				TRACE("@NearJamp\n" );
				break;
			case '\\':// xR[
				Index = GetDW();
				if( Index ){
					TRACE("\\\\NearCall\n");
					NearCall( Index );
				}else{
					TRACE("\\\\Near Return\n");
					NearRet();
				}
				break;
			case '{': // 
				TRACE( "/" );
				bRet = lpED->GetExpValue();
				Index= GetDW();
				if( !bRet ){
					TRACE("񐬗\n");
					NearJamp( Index );
				}else{
					TRACE("\n");
				}
				break;
			case '~': // xy[WWv
				Page = GetW();
				if( Page == 0 ){
					m_RetValue = lpED->GetExpValue();
					TRACE("~FarReturn\n");
					FarRet();
				}else if( Page == 0xFFFF ){
					TRACE("~return value\n" );
					*lpED->GetVariableNo() = m_RetValue;
				}else{
					TRACE("~FarCall\n");
					FarCall( Page - 1, GetDW() );
				}
				break;
			default:
				break;
		};
	}
	
	return ALSD_OK;
}

/**********************************************************************
	bZ[W`FbN
**********************************************************************/
long CScenarioParser::CheckMessage( void )
{
	string MsgBuffer;
	const unsigned char *lpKana;

	long Cmd = GetC();

	while (Cmd == 0x20 || Cmd > 0x80) {
		if (Cmd == 0x20) {
			MsgBuffer += (char)0x81;
			MsgBuffer += 0x40;
		} else if (Cmd >= 0xe0) {
			MsgBuffer += (char)Cmd;
			MsgBuffer += (char)GetC();
		// pJȉ
		} else if (Cmd >= 0xa0) {
			lpKana = Hankana2Sjis( Cmd );
			MsgBuffer += *(lpKana);
			MsgBuffer += *(lpKana + 1);
		} else {
			MsgBuffer += (char)Cmd;
			MsgBuffer += (char)GetC();
		}
		Cmd = GetC();
	}
	
	if ( MsgBuffer.length() ) {
		if( m_lpSD->GetSelCtrl()->IsSelecting() ){
			// Io^
			m_lpSD->GetSelCtrl()->AddChoice( MsgBuffer );
		}else{
			// bZ[Wo^
			m_lpSD->GetMsgCtrl()->Message( MsgBuffer.c_str() );
		}
	}

	return Cmd;
}

/**********************************************************************
	w蕶܂Ŏ擾
*********************************************************************/
void CScenarioParser::GetString( string &_strString, char term )
{
	string MsgBuffer;
//	MsgBuffer = "";
	const unsigned char *lpKana;

	long Cmd = GetC();

	while ( Cmd != term ) {
		if (Cmd == 0x20) {
			MsgBuffer += (char)0x81;
			MsgBuffer += 0x40;
		} else if (Cmd >= 0xe0) {
			MsgBuffer += (char)Cmd;
			MsgBuffer += (char)GetC();
		// pJȉ
		} else if (Cmd >= 0xa0) {
			lpKana = Hankana2Sjis( Cmd );
			MsgBuffer += *(lpKana);
			MsgBuffer += *(lpKana + 1);
		} else {
			MsgBuffer += (char)Cmd;
			Cmd = GetC();
			if( Cmd != term ){
				MsgBuffer += Cmd;
			}else{
				break;
			}
		}
		Cmd = GetC();
	}

	if( m_lpSD->GetMsgCtrl()->IsHankaku() ){
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
		_strString = kanakan::zentohan2( (unsigned char *)MsgBuffer.c_str());
#else
		_strString = zentohan2( (unsigned char *)MsgBuffer.c_str());
#endif
	}else{
		_strString = MsgBuffer;
	}
}



