/*
 * SelectionCtrl.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#include "stdafx.h"
#include "SelectionCtrl.h"

#include "GraphicDriver.h"
#include "MessageCtrl.h"
#include "FileCtrl.h"

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using namespace ALSD;
using namespace ALSD_SELCTR;
using namespace ALSD_FILECTR;
#endif
/**********************************************************************
	RXgNV
**********************************************************************/
CSelectionCtrl::CSelectionCtrl(  CScenarioDecoder *_lpScenarioDecoder )
{
	m_lpSD			= _lpScenarioDecoder;
	m_NowSetting	= 0;
	m_bSelecting	= false;
	m_NowWindow		= 0;
	m_MaxStrLength	= 0;
	m_FontSize		= 16;
	m_FontColor		= 255;
	m_BackColor		= 0;
	m_AlphaRate		= 255;
	m_FrameColor	= 255;
	m_SeleColor		= 255;
	m_CurType		= 0;
	m_LineIncrement = 2;

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	for( long i = 0; i < InfoMax; i++){
#else
	for( long i = 0; i < SelectionInfoMax; i++){
#endif
		m_lpSelInfo[i].x = 450;
		m_lpSelInfo[i].y = 100;
		m_lpSelInfo[i].w = 150;
		m_lpSelInfo[i].h = 100;
	}

}

CSelectionCtrl::~CSelectionCtrl()
{
}

/**********************************************************************
	IWindow̐ݒ
**********************************************************************/
void CSelectionCtrl::SetSelInfo( long _num, long _x, long _y,
								 long _w, long _h, long _s )
{
	m_lpSelInfo[_num].x = _x;
	m_lpSelInfo[_num].y = _y;
	m_lpSelInfo[_num].w = _w;
	m_lpSelInfo[_num].h = _h;
	m_lpSelInfo[_num].s = _s;
}

void CSelectionCtrl::ChangeSelector( long _num, long _w,
					 long _c1, long _c2, long _c3, long _c4 )
{
	m_NowWindow = _num;
	m_lpSelInfo[_num].win = _w;
	m_lpSelInfo[_num].CGNoTop = _c1;
	m_lpSelInfo[_num].CGNoMid = _c2;
	m_lpSelInfo[_num].CGNoBottom = _c3;
	if( _w == 0 )m_lpSelInfo[_num].FrameSize = 0;
	else if( _w == 1 )m_lpSelInfo[_num].FrameSize = 8;
	else m_lpSelInfo[_num].FrameSize = _c4;
}

/**********************************************************************
	I
**********************************************************************/
void CSelectionCtrl::OpenChoice( long _Result )
{
	m_strChoices  [m_NowSetting] = ""; 
	m_ChoiceResult[m_NowSetting] = _Result;

}

void CSelectionCtrl::CloseChoice( void )
{
	if( m_MaxStrLength < m_strChoices[m_NowSetting].length() ){
		m_MaxStrLength = m_strChoices[m_NowSetting].length();
	}
	m_NowSetting++;
}

/**********************************************************************
	I̒ǉ
**********************************************************************/
void CSelectionCtrl::AddChoice( const string &_str )
{
	m_strChoices[m_NowSetting] += _str;
}

/**********************************************************************
	I
**********************************************************************/
long CSelectionCtrl::Select( void )
{
	/////////////////////////////////////////////////////////
	// I

	// g͈
	stGrpRect rect;
	rect.left	= m_lpSelInfo[m_NowWindow].x;
	rect.top	= m_lpSelInfo[m_NowWindow].y;
	rect.right	= m_lpSelInfo[m_NowWindow].w;
	rect.bottom	= m_lpSelInfo[m_NowWindow].h;
	if( m_MaxStrLength * m_FontSize < rect.right ){
		rect.right = m_MaxStrLength * m_FontSize ;
	}
	if( m_NowSetting * ( m_FontSize + m_LineIncrement ) != rect.bottom ){
		rect.bottom = m_NowSetting * ( m_FontSize + m_LineIncrement );
	}

	CGraphicDriver *lpGD = m_lpSD->GetGrpDriver();
	stGrpRect update;
	update.left = rect.left	- m_lpSelInfo[m_NowWindow].FrameSize;
	update.top	= rect.top	- m_lpSelInfo[m_NowWindow].FrameSize;
	update.right= rect.right+ m_lpSelInfo[m_NowWindow].FrameSize  * 2;
	update.bottom=rect.bottom+ m_lpSelInfo[m_NowWindow].FrameSize * 2;
	
	// ޔ
	long SaveID = -1;
	if( m_lpSelInfo[m_NowWindow].s ){
		update.right += update.left; update.bottom+= update.top;
		SaveID = m_lpSD->GetGrpDriver()->SaveRect( &update );
		update.right -= update.left; update.bottom-= update.top;
	
		lpGD->AddColorRect( update.left, update.top,
							update.right,update.bottom,
							m_BackColor, m_AlphaRate );
	}else{
		// hԂ
		lpGD->FillRect( update.left, update.top,
						update.right,update.bottom,
						m_BackColor );
	}
	// g`
	DrawSelectFrame( rect );

	// `
	lpGD->SetFontSize( m_FontSize );
	for( int i = 0; i < m_NowSetting; i++){
		lpGD->DrawString( rect.left, rect.top + i * (m_FontSize + m_LineIncrement), 
						  m_strChoices[i].c_str(), m_FontColor );
	}
	// XV
	update.right += update.left;
	update.bottom+= update.top;
	lpGD->OnDraw( &update );

	// J[\ړ
	lpGD->SetMousePos(	rect.left + (m_FontSize / 2 ),
						rect.top  + (m_FontSize / 2 ) );

	// I
	long ret = _Select( rect );

	// 
	m_NowSetting = 0;

	// ޔĂ̂߂
	if( SaveID != -1 ){
		m_lpSD->GetGrpDriver()->RestoreRect( SaveID );
	}
	lpGD->OnDraw( &update );
	
	// ̑I֐iނ̉
	if( m_lpSD->GetMsgCtrl()->GetAutoPageStop() ){
		m_lpSD->GetMsgCtrl()->SetAutoNextPage( false );
	}
	return ret;
}

/**********************************************************************
	I[v
**********************************************************************/
long CSelectionCtrl::_Select( const stGrpRect &_rect )
{
	CGraphicDriver *lpGD = m_lpSD->GetGrpDriver();

	stGrpRect update;
	update.left = _rect.left	- m_lpSelInfo[m_NowWindow].FrameSize;
	update.top	= _rect.top	- m_lpSelInfo[m_NowWindow].FrameSize;
	update.right= _rect.left+ _rect.right+ m_lpSelInfo[m_NowWindow].FrameSize * 2;
	update.bottom=_rect.top + _rect.bottom+ m_lpSelInfo[m_NowWindow].FrameSize * 2;

	stGrpPoint pt;
	long	reverse = -1; // ]\ȍ
	long	select;		// I 
	while( !m_lpSD->m_bExit ){
		lpGD->Sleep( 50 );
		
		// ENbNLZ
		if( lpGD->IsRButtonDown( &pt ) )return -1;
		if( lpGD->IsLButtonDown( &pt ) )break;
		// ͈̓`FbN
		int i;
		for( i = 0; i < m_NowSetting; i++){
			if( _rect.left <= pt.x && _rect.left + _rect.right > pt.x &&
				_rect.top + (i + 0) * (m_FontSize + m_LineIncrement) <= pt.y  && 
				_rect.top + (i + 1) * (m_FontSize + m_LineIncrement) >= pt.y ){
				// ͈͒
				select = i;
				break;
			}
		}
		if( i != m_NowSetting ){
			// ڂ𔽓]
			if( select != reverse ){
				// ȑOIĂڂ߂
				if( reverse > -1 ){
					lpGD->Reverse(	_rect.left,
									_rect.top + (reverse) * (m_FontSize + m_LineIncrement),
									_rect.right,
									m_FontSize );
				}
				lpGD->Reverse(	_rect.left,
								_rect.top + (select) * (m_FontSize + m_LineIncrement),
								_rect.right,
								m_FontSize );
				reverse = select;
				lpGD->OnDraw( &update );
			}
		}else{
			// HitTestɎs
			// ȑOIĂڂ߂
			if( reverse > -1 ){
				lpGD->Reverse(	_rect.left,
								_rect.top + (reverse) * (m_FontSize + m_LineIncrement),
								_rect.right,
								m_FontSize );
				lpGD->OnDraw( &update );
				reverse = -1;
			}
		}
	}
	// [vIAI߂
	if( reverse >= 0 ){
			lpGD->Reverse(	_rect.left,
							_rect.top + (reverse) * (m_FontSize + m_LineIncrement),
							_rect.right,
							m_FontSize );
			lpGD->OnDraw( &update );
	}

	//
	if( reverse >= 0 ){
		lpGD->Sleep( 100 );
		return m_ChoiceResult[reverse];
	}

	return -1;
}


/**********************************************************************
	Ig̕\
**********************************************************************/
void CSelectionCtrl::DrawSelectFrame( const stGrpRect &rect )
{
	CGraphicDriver *lpGD = m_lpSD->GetGrpDriver();

	lpGD->StrokeRect(	rect.left	 - 5,  
						rect.top	 - 5,
						rect.right	 +10,
						rect.bottom	 +10,
						m_FrameColor );
	
	lpGD->StrokeRect(	rect.left	 - 7,  
						rect.top	 - 7,
						rect.right	 +14,
						rect.bottom	 +14,
						m_FrameColor );
	
	lpGD->StrokeRect(	rect.left	 - 8,  
						rect.top	 - 8,
						rect.right	 +16,
						rect.bottom	 +16,
						m_FrameColor );

}

/**********************************************************************
	Z[uf[^Zbg
**********************************************************************/
void CSelectionCtrl::GetSaveData( stSaveHeader *_lpSaveHeader )
{
	_lpSaveHeader->selFontSize = m_FontSize;
	_lpSaveHeader->selFontColor= m_FontColor;
	_lpSaveHeader->selClearColor=m_BackColor;
	_lpSaveHeader->selFrameColor=m_FrameColor;

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	for( long i = 0; i < InfoMax; i++){
#else
	for( long i = 0; i < SelectionInfoMax; i++){
#endif
		_lpSaveHeader->selWinInfo[i].x = m_lpSelInfo[i].x;
		_lpSaveHeader->selWinInfo[i].y = m_lpSelInfo[i].y;
		_lpSaveHeader->selWinInfo[i].w = m_lpSelInfo[i].w;
		_lpSaveHeader->selWinInfo[i].h = m_lpSelInfo[i].h;
	}
}

/**********************************************************************
	Z[uf[^Zbg
**********************************************************************/
void CSelectionCtrl::SetLoadData( stSaveHeader *_lpSaveHeader )
{
	m_FontSize	= _lpSaveHeader->selFontSize;
	m_FontColor	= _lpSaveHeader->selFontColor;
	m_BackColor	= _lpSaveHeader->selClearColor;
	m_FrameColor= _lpSaveHeader->selFrameColor;

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	for( long i = 0; i < InfoMax; i++){
#else
	for( long i = 0; i < SelectionInfoMax; i++){
#endif
		m_lpSelInfo[i].x = _lpSaveHeader->selWinInfo[i].x;
		m_lpSelInfo[i].y = _lpSaveHeader->selWinInfo[i].y;
		m_lpSelInfo[i].w = _lpSaveHeader->selWinInfo[i].w;
		m_lpSelInfo[i].h = _lpSaveHeader->selWinInfo[i].h;
	}
}

