/*
 * SelectionCtrl.h - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 *      99/05/24 KISAKA Toshihiro       modify for mwcc compatiblility
 *
 */

#ifndef __SelectionCtrl_h__
#define __SelectionCtrl_h__
/**********************************************************************
	scenairo decoder system 
**********************************************************************/
#include "ScenarioDecoderSystem.h"
#include <string>

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
using std::string;
#endif
//
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD {
#endif
	class CSelectionCtrl;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif
//
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD_SELCTR {
#endif
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	const unsigned long InfoMax = 128;
#else
	const unsigned long SelectionInfoMax = 128;
#endif
	struct stSelectorInfo {
		long x, y, w, h, s, win;
		long CGNoTop, CGNoMid, CGNoBottom, FrameSize;
	};
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif
//
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
namespace ALSD_FILECTR {
#endif
	struct stSaveHeader;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
};
#endif

//
// class
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
class ALSD::CSelectionCtrl {
#else
class CSelectionCtrl {
#endif
public:
	CSelectionCtrl( CScenarioDecoder *_lpScenarioDecoder );
	virtual ~CSelectionCtrl();
	
	//
	void OpenChoice( long _Result );
	void CloseChoice( void );
	void AddChoice( const string &_str );
	long Select( void );

	//
	void SetSelecting( bool _sel ){ m_bSelecting = _sel;};
	bool IsSelecting( void )const { return m_bSelecting;};

#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	void GetSaveData( ALSD_FILECTR::stSaveHeader *_lpSaveHeader );
	void SetLoadData( ALSD_FILECTR::stSaveHeader *_lpSaveHeader );
#else
	void GetSaveData( stSaveHeader *_lpSaveHeader );
	void SetLoadData( stSaveHeader *_lpSaveHeader );
#endif

	// settings
	void SetSelInfo( long _num, long _x, long _y, long _w, long _h, long _s );
	void ChangeSelector( long _num, long _w, long _c1, long c2, long _c3, long _c4 );
	void SetFontSize ( long _s ){ m_FontSize  = _s;};
	void SetFontColor( long _c ){ m_FontColor = _c;};
	void SetBackColor( long _c ){ m_BackColor = _c;};
	void SetColorKey ( long _c ){ m_AlphaRate = _c;};
	void SetFrameColor(long _c ){ m_FrameColor= _c;};
	void SetCurType  ( long _c ){ m_CurType   = _c;};
	void SetSeleColor( long _c ){ m_SeleColor = _c;};

protected:
	CScenarioDecoder	*m_lpSD;

	bool	m_bSelecting;
	long	m_NowSetting;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	string	m_strChoices	[ALSD::ALSD_SEL_CHOICESMAX];
	long	m_ChoiceResult	[ALSD::ALSD_SEL_CHOICESMAX];
#else
	string	m_strChoices	[ALSD_SEL_CHOICESMAX];
	long	m_ChoiceResult	[ALSD_SEL_CHOICESMAX];
#endif
	long	m_MaxStrLength;

	long	m_AlphaRate;
	long	m_FontSize;
	long	m_FontColor;
	long	m_BackColor;
	long	m_FrameColor;
	long	m_SeleColor;

	long	m_CurType;

	long	m_LineIncrement;


	// window setting
	long	m_NowWindow;
#if (!__MWERKS__ || defined(MSIPL_USING_NAMESPACE))
	ALSD_SELCTR::stSelectorInfo m_lpSelInfo[ALSD_SELCTR::InfoMax];	
#else
	stSelectorInfo m_lpSelInfo[SelectionInfoMax];	
#endif

	//
	long _Select( const stGrpRect &_rect );
	void DrawSelectFrame( const stGrpRect &rect );


private:
};

#endif
